/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


void generateCSSReport(struct nipperConfig *nipper)
{
	// Variables
	struct snmpCSSCommunity *snmpCSSCommunityPointer = 0;
	struct snmpCSSHost *snmpCSSHostPointer = 0;
	struct cssAclClause *cssAclClausePointer = 0;
	struct cssAcl *cssAclPointer = 0;
	int tempInt = 0;
	int issues = 0;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];

	// Counter Variables...
	int aclIssues = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (nipper->outputFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (nipper->outputFile[0] == 0)
		nipper->reportFile = stdout;
	else
	{
		nipper->reportFile = fopen(nipper->outputFile, "w");
		if (nipper->reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, nipper->outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(nipper->hostname, nipper->friendlyHostname, nipper->maxSize, nipper->outputFormat);

	// Output file heading (if required)
	output_filestart(nipper);

	// -------------------------
	// Security Section Contents
	// -------------------------
	if (nipper->fullReport == true)
	{
		output_parseSectionIndex(report_security_title, nipper);

		// Start of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "<ul>\n");
				break;
		}
		output_parseIndex("Introduction", nipper, section_security);

		// SNMP
		if (nipper->css->snmpAccess == true)
		{
			output_parseIndex(report_snmp_title, nipper, section_security);
			issues++;
		}

		// Telnet
		if (nipper->css->telnetAccess == true)
		{
			output_parseIndex(report_telnet_title, nipper, section_security);
			issues++;
		}

		// ACL
		aclIssues = 0;
		if (nipper->css->acl != 0)
		{
			cssAclPointer = nipper->css->acl;
			while (cssAclPointer != 0)
			{
				cssAclClausePointer = cssAclPointer->clause;
				while (cssAclClausePointer != 0)
				{
					if ((cssAclClausePointer->access != css_deny) && ((((nipper->strictACL == true) && (strcmp(cssAclClausePointer->source, "any") == 0)) || (strcmp(cssAclClausePointer->dest, "any") == 0) || (strcmp(cssAclClausePointer->destPort, "any") == 0)) && (strcmp(cssAclClausePointer->protocol, "icmp") != 0)))
						aclIssues++;
					if ((nipper->denyLog == true) && (cssAclClausePointer->next == 0) && ((cssAclClausePointer->access != css_deny) || (cssAclClausePointer->log == false) || (strcmp(cssAclClausePointer->source, "any") != 0) || (strcmp(cssAclClausePointer->dest, "any") != 0) || (strcmp(cssAclClausePointer->destPort, "any") != 0)))
						aclIssues++;
					cssAclClausePointer = cssAclClausePointer->next;
				}
				cssAclPointer = cssAclPointer->next;
			}
		}
		else
			aclIssues = -1;
		if (aclIssues != 0)
		{
			output_parseIndex(report_accesslist_title, nipper, section_security);
			issues++;
		}

		// End of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "</ul>\n");
				break;
			case output_text:
				fprintf(nipper->reportFile, "\n");
				break;
		}
	}

	// ------------------------------
	// Configuration Section Contents
	// ------------------------------
	output_parseSectionIndex(report_config_title, nipper);

	// Start of configuration index
	switch (nipper->outputFormat)
	{
		case output_html:
			fprintf(nipper->reportFile, "<ul>\n");
			break;
	}
	output_parseIndex("Introduction", nipper, section_config);

	// In Configuration Index
	output_parseIndex(settings_general_title, nipper, section_config);
	output_parseIndex(settings_services_title, nipper, section_config);
	if (nipper->css->snmp != 0)
		output_parseIndex(settings_snmp_title, nipper, section_config);
	if (nipper->css->acl != 0)
		output_parseIndex(settings_accesslists_title, nipper, section_config);

	// End of index
	output_abbrevContents(nipper);
	output_endsection(nipper, subsection_end);

	// Security Report section
	if (nipper->fullReport == true)
	{
		// Introduction
		output_parseTitle(report_security_title, nipper, rate_none, section_main_sec);
		output_parseTitle("Introduction", nipper, rate_none, section_security);
		output_parseText(report_security_intro, nipper, section_none, rate_none, issues, 0);
		output_endsection(nipper, subsection_end);

		// SNMP
		if (nipper->css->snmpAccess == true)
		{
			output_parseTitle(report_snmp_title, nipper, rate_med, section_security);
			output_parseText(settings_snmp_desc, nipper, section_obs, rate_none, 0, nipper->tableNum);
			if (nipper->css->snmp != 0)
			{
				if (nipper->css->snmp->host != 0)
					output_parseText(report_snmp_css_obs, nipper, section_none, rate_none, -1, nipper->tableNum);
				else
					output_parseText(report_snmp_css_obs, nipper, section_none, rate_none, 0, nipper->tableNum);
			}
			else
				output_parseText(report_snmp_css_obs, nipper, section_none, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_snmp_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_snmp_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			if (nipper->css->snmp != 0)
			{
				if (nipper->css->snmp->host != 0)
					output_parseText(report_snmp_css_rec, nipper, section_rec, rate_none, -1, nipper->tableNum);
				else
					output_parseText(report_snmp_css_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			}
			else
				output_parseText(report_snmp_css_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Telnet
		if (nipper->css->telnetAccess == true)
		{
			output_parseTitle(report_telnet_title, nipper, rate_med, section_security);
			output_parseText(report_telnet_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_telnet_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_telnet_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_css_telnet_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Access Control Lists
		if (aclIssues != 0)
		{
			output_parseTitle(report_accesslist_title, nipper, rate_med, section_security);
			output_parseText(report_accesslist_css_obs, nipper, section_obs, rate_none, aclIssues, nipper->tableNum);
			if (aclIssues > 1)
			{
				aclIssues = 1;
				output_table(nipper, true, report_accesslist_css_table, &report_accesslists_heading);
				cssAclPointer = nipper->css->acl;
				while (cssAclPointer != 0)
				{
					cssAclClausePointer = cssAclPointer->clause;
					tempInt = 1;
					while (cssAclClausePointer != 0)
					{
						if ((cssAclClausePointer->access != css_deny) && ((((nipper->strictACL == true) && (strcmp(cssAclClausePointer->source, "any") == 0)) || (strcmp(cssAclClausePointer->dest, "any") == 0) || (strcmp(cssAclClausePointer->destPort, "any") == 0)) && (strcmp(cssAclClausePointer->protocol, "icmp") != 0)))
						{
							// Any Source Any Dest Any Service
							if ((strcmp(cssAclClausePointer->source, "any") == 0) && (strcmp(cssAclClausePointer->dest, "any") == 0) && (strcmp(cssAclClausePointer->destPort, "any") == 0))
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue1, tablerow_end);

							// Any Dest Any Service
							else if ((strcmp(cssAclClausePointer->dest, "any") == 0) && (strcmp(cssAclClausePointer->destPort, "any") == 0))
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue2, tablerow_end);

							// Any Source Any Service
							else if ((strcmp(cssAclClausePointer->source, "any") == 0) && (strcmp(cssAclClausePointer->destPort, "any") == 0))
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue3, tablerow_end);

							// Any Source Any Dest
							else if ((strcmp(cssAclClausePointer->source, "any") == 0) && (strcmp(cssAclClausePointer->dest, "any") == 0))
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue4, tablerow_end);

							// Any Source
							else if (strcmp(cssAclClausePointer->source, "any") == 0)
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue5, tablerow_end);

							// Any Dest
							else if (strcmp(cssAclClausePointer->dest, "any") == 0)
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue6, tablerow_end);

							// Any Service
							else if (strcmp(cssAclClausePointer->destPort, "any") == 0)
								fprintf(nipper->reportFile, "%s%d%s%d%s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tempInt, tablerow_mid, report_accesslist_tabissue7, tablerow_end);
						}
						if ((nipper->denyLog == true) && (cssAclClausePointer->next == 0) && ((cssAclClausePointer->access != css_deny) || (cssAclClausePointer->log == false) || (strcmp(cssAclClausePointer->source, "any") != 0) || (strcmp(cssAclClausePointer->dest, "any") != 0) || (strcmp(cssAclClausePointer->destPort, "any") != 0)))
						{
							aclIssues++;
							fprintf(nipper->reportFile, "%s%d%s %s%s%s", tablerow_start, cssAclPointer->index, tablerow_mid, tablerow_mid, report_accesslist_tabissue8, tablerow_end);
						}
						tempInt++;
						cssAclClausePointer = cssAclClausePointer->next;
					}
					cssAclPointer = cssAclPointer->next;
				}
				output_table(nipper, false, report_accesslist_css_table, &report_accesslists_heading);
			}
			else if (aclIssues > 0)
			{
				cssAclPointer = nipper->css->acl;
				while (cssAclPointer != 0)
				{
					cssAclClausePointer = cssAclPointer->clause;
					tempInt = 1;
					while (cssAclClausePointer != 0)
					{
						if ((cssAclClausePointer->access != css_deny) && ((((nipper->strictACL == true) && (strcmp(cssAclClausePointer->source, "any") == 0)) || (strcmp(cssAclClausePointer->dest, "any") == 0) || (strcmp(cssAclClausePointer->destPort, "any") == 0)) && (strcmp(cssAclClausePointer->protocol, "icmp") != 0)))
						{
							fprintf(nipper->reportFile, "%d line %d ", cssAclPointer->index, tempInt);
							// Any Source Any Dest Any Service
							if ((strcmp(cssAclClausePointer->source, "any") == 0) && (strcmp(cssAclClausePointer->dest, "any") == 0) && (strcmp(cssAclClausePointer->destPort, "any") == 0))
								fprintf(nipper->reportFile, "%s", report_accesslist_issue1);

							// Any Dest Any Service
							else if ((strcmp(cssAclClausePointer->dest, "any") == 0) && (strcmp(cssAclClausePointer->destPort, "any") == 0))
								fprintf(nipper->reportFile, "%s", report_accesslist_issue2);

							// Any Source Any Service
							else if ((strcmp(cssAclClausePointer->source, "any") == 0) && (strcmp(cssAclClausePointer->destPort, "any") == 0))
								fprintf(nipper->reportFile, "%s", report_accesslist_issue3);

							// Any Source Any Dest
							else if ((strcmp(cssAclClausePointer->source, "any") == 0) && (strcmp(cssAclClausePointer->dest, "any") == 0))
								fprintf(nipper->reportFile, "%s", report_accesslist_issue4);

							// Any Source
							else if (strcmp(cssAclClausePointer->source, "any") == 0)
								fprintf(nipper->reportFile, "%s", report_accesslist_issue5);

							// Any Dest
							else if (strcmp(cssAclClausePointer->dest, "any") == 0)
								fprintf(nipper->reportFile, "%s", report_accesslist_issue6);

							// Any Service
							else if (strcmp(cssAclClausePointer->destPort, "any") == 0)
								fprintf(nipper->reportFile, "%s", report_accesslist_issue7);
						}
						if ((nipper->denyLog == true) && (cssAclClausePointer->next == 0) && ((cssAclClausePointer->access != css_deny) || (cssAclClausePointer->log == false) || (strcmp(cssAclClausePointer->source, "any") != 0) || (strcmp(cssAclClausePointer->dest, "any") != 0) || (strcmp(cssAclClausePointer->destPort, "any") != 0)))
						{
							aclIssues++;
							fprintf(nipper->reportFile, "%s", report_accesslist_issue8);
						}
						tempInt++;
						cssAclClausePointer = cssAclClausePointer->next;
					}
					cssAclPointer = cssAclPointer->next;
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_accesslist_css_impact, nipper, section_imp, rate_med, aclIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_accesslist_ease, nipper, section_ease, rate_noease, aclIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, aclIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// End Section
		output_endsection(nipper, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	output_parseTitle(report_config_title, nipper, rate_none, section_main_con);
	output_parseTitle("Introduction", nipper, rate_none, section_config);
	output_parseText(report_config_intro, nipper, section_none, rate_none, 0, 0);
	output_endsection(nipper, subsection_end);

	// General Settings
	output_parseTitle(settings_general_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_general_table, &settings_general_heading);
	fprintf(nipper->reportFile, "%sHostname%s%s%s", tablerow_start, tablerow_mid, nipper->friendlyHostname, tablerow_end);
	if (nipper->version[0] != 0)
		fprintf(nipper->reportFile, "%sCSS Version%s%s%s", tablerow_start, tablerow_mid, nipper->version, tablerow_end);
	output_table(nipper, false, settings_general_table, &settings_general_heading);
	output_endsection(nipper, subsection_end);
	addAbbreviation("CSS", false);

	// Device Services
	output_parseTitle(settings_services_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_services_table, &report_services_heading);
	if (nipper->css->ftpAccess == true)
		fprintf(nipper->reportFile, "%sFTP Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sFTP Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->css->snmpAccess == true)
		fprintf(nipper->reportFile, "%sSNMP Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sSNMP Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->css->sshAccess == true)
		fprintf(nipper->reportFile, "%sSSH Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sSSH Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->css->telnetAccess == true)
		fprintf(nipper->reportFile, "%sTelnet Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sTelnet Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->css->webManagementAccess == true)
		fprintf(nipper->reportFile, "%sWeb Management Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sWeb Management Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(nipper, false, settings_services_table, &report_services_heading);
	output_endsection(nipper, subsection_end);
	addAbbreviation("FTP", false);
	addAbbreviation("SNMP", false);
	addAbbreviation("SSH", false);

	// SNMP
	if (nipper->css->snmp != 0)
	{
		output_parseTitle(settings_snmp_title, nipper, rate_none, section_config);
		output_parseText(settings_snmp_desc, nipper, section_none, rate_none, -1, 0);

		// SNMP Settings
		output_table(nipper, true, settings_snmp_table, &settings_general_heading);
		outputFriendly(nipper->css->snmp->name, tempString, nipper->maxSize, nipper->outputFormat);
		fprintf(nipper->reportFile, "%sHostname%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		outputFriendly(nipper->css->snmp->contact, tempString, nipper->maxSize, nipper->outputFormat);
		fprintf(nipper->reportFile, "%sContact%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		outputFriendly(nipper->css->snmp->location, tempString, nipper->maxSize, nipper->outputFormat);
		fprintf(nipper->reportFile, "%sLocation%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		if (nipper->css->snmp->reloadEnabled == true)
			fprintf(nipper->reportFile, "%sSNMP Device Reload%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sSNMP Device Reload%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(nipper, false, settings_snmp_table, &settings_general_heading);

		// SNMP Community
		if (nipper->passwords == true)
		{
			snmpCSSCommunityPointer = nipper->css->snmp->community;
			output_table(nipper, true, settings_snmpstring_table, &settings_snmp_css_heading);
			while (snmpCSSCommunityPointer != 0)
			{
				outputFriendly(snmpCSSCommunityPointer->community, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				if (snmpCSSCommunityPointer->readOnly)
					fprintf(nipper->reportFile, "Read Only%s", tablerow_end);
				else
					fprintf(nipper->reportFile, "Read-Write%s", tablerow_end);
				snmpCSSCommunityPointer = snmpCSSCommunityPointer->next;
			}
			output_table(nipper, false, settings_snmpstring_table, &settings_snmp_css_heading);
		}

		snmpCSSHostPointer = nipper->css->snmp->host;
		output_table(nipper, true, settings_snmphost_table, &settings_snmphost_heading);
		while (snmpCSSHostPointer != 0)
		{
			outputFriendly(snmpCSSHostPointer->host, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
			if (snmpCSSHostPointer->snmpType == snmp1)
				fprintf(nipper->reportFile, "SNMP v1");
			else
				fprintf(nipper->reportFile, "SNMP v2c");
			if (nipper->passwords == true)
			{
				outputFriendly(snmpCSSHostPointer->community, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tablerow_mid, tempString);
			}
			fprintf(nipper->reportFile, "%s", tablerow_end);
			snmpCSSHostPointer = snmpCSSHostPointer->next;
		}
		output_table(nipper, false, settings_snmphost_table, &settings_snmphost_heading);
		output_endsection(nipper, subsection_end);
	}

	// ACL
	if (nipper->css->acl != 0)
	{
		output_parseTitle(settings_accesslists_title, nipper, rate_none, section_config);
		output_parseText(settings_accesslist_css, nipper, section_none, rate_none, -1, 0);
		cssAclPointer = nipper->css->acl;
		while (cssAclPointer != 0)
		{
			if (cssAclPointer->applyTo[0] != 0)
				sprintf(tempString2, "ACL %d (applied to %s)", cssAclPointer->index, cssAclPointer->applyTo);
			else
				sprintf(tempString2, "ACL %d", cssAclPointer->index);
			output_table(nipper, true, tempString2, &settings_acl_css_heading);
			cssAclClausePointer = cssAclPointer->clause;
			tempInt = 1;
			while (cssAclClausePointer != 0)
			{
				fprintf(nipper->reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
				switch (cssAclClausePointer->access)
				{
					case css_permit:
						fprintf(nipper->reportFile, "Permit");
						break;
					case css_deny:
						fprintf(nipper->reportFile, "Deny");
						break;
					case css_bypass:
						fprintf(nipper->reportFile, "Bypass");
						break;
				}
				outputFriendly(cssAclClausePointer->protocol, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_mid);
				outputFriendly(cssAclClausePointer->source, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
				addAbbreviation(cssAclClausePointer->sourcePort, false);
				switch (cssAclClausePointer->sourcePortOper)
				{
					case perm_op_lt:
						if (nipper->outputFormat == output_html)
							fprintf(nipper->reportFile, "&lt;");
						else
							fprintf(nipper->reportFile, "<");
						fprintf(nipper->reportFile, " %s", cssAclClausePointer->sourcePort);
						break;
					case perm_op_gt:
						if (nipper->outputFormat == output_html)
							fprintf(nipper->reportFile, "&gt;");
						else
							fprintf(nipper->reportFile, ">");
						fprintf(nipper->reportFile, " %s", cssAclClausePointer->sourcePort);
						break;
					case perm_op_eq:
						fprintf(nipper->reportFile, "%s", cssAclClausePointer->sourcePort);
						break;
					case perm_op_neq:
						fprintf(nipper->reportFile, "not %s", cssAclClausePointer->sourcePort);
						break;
					case perm_op_range:
						fprintf(nipper->reportFile, "%s - %s", cssAclClausePointer->sourcePort, cssAclClausePointer->sourcePortEnd);
						addAbbreviation(cssAclClausePointer->sourcePortEnd, false);
						break;
					default:
						fprintf(nipper->reportFile, "any");
						break;
				}
				outputFriendly(cssAclClausePointer->dest, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_mid);
				addAbbreviation(cssAclClausePointer->destPort, false);
				switch (cssAclClausePointer->destPortOper)
				{
					case perm_op_lt:
						if (nipper->outputFormat == output_html)
							fprintf(nipper->reportFile, "&lt;");
						else
							fprintf(nipper->reportFile, "<");
						fprintf(nipper->reportFile, " %s", cssAclClausePointer->destPort);
						break;
					case perm_op_gt:
						if (nipper->outputFormat == output_html)
							fprintf(nipper->reportFile, "&gt;");
						else
							fprintf(nipper->reportFile, ">");
						fprintf(nipper->reportFile, " %s", cssAclClausePointer->destPort);
						break;
					case perm_op_eq:
						fprintf(nipper->reportFile, "%s", cssAclClausePointer->destPort);
						break;
					case perm_op_neq:
						fprintf(nipper->reportFile, "not %s", cssAclClausePointer->destPort);
						break;
					case perm_op_range:
						fprintf(nipper->reportFile, "%s - %s", cssAclClausePointer->destPort, cssAclClausePointer->destPortEnd);
						addAbbreviation(cssAclClausePointer->destPortEnd, false);
						break;
					default:
						fprintf(nipper->reportFile, "any");
						break;
				}
				if (cssAclClausePointer->log == true)
					fprintf(nipper->reportFile, "%sYes%s", tablerow_mid, tablerow_end);
				else
					fprintf(nipper->reportFile, "%sNo%s", tablerow_mid, tablerow_end);
				tempInt++;
				cssAclClausePointer = cssAclClausePointer->next;
			}
			output_table(nipper, false, tempString2, &settings_acl_css_heading);
			cssAclPointer = cssAclPointer->next;
		}
		output_endsection(nipper, subsection_end);
	}

	// End Section
	output_endsection(nipper, section_end);

	// Output end of file stuff
	output_fileend(nipper);
}

