/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by the following authors:                   *
 *                                                                         *
 *       Ian Ventura-Whiting (fizz@titania.co.uk)                          *
 *       David Croft                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


// Access List Permission
struct accessListPermission
{
	int permission;					// perm_allow or perm_deny
	char protocol[8];
	char source[64];
	int sourceOperator;				// perm_op_lt, perm_op_gt...
	char sourceService[32];
	char sourceServiceEnd[32];
	char dest[64];
	int destOperator;				// perm_op_lt, perm_op_gt...
	char destService[32];
	char destServiceEnd[32];
	int established;				// true or false
	int fragments;					// true or false
	int log;						// true or false
	struct accessListPermission *next;
};

#define ios_acl_standard 0
#define ios_acl_extended 1

// Access List
struct accessList
{
	int type;						// ios_acl_standard...
	char name[65];
	char remark[128];
	struct accessListPermission *permission;
	struct accessList *next;
};


struct accessList *getAccessList(char listName[64], int aclType, struct nipperConfig *nipper)
{
	// Variables...
	struct accessList *accessPointer;

	// If there are no access lists, then create one...
	if (nipper->ios->access == 0)
	{
		// Create storage for access-list...
		nipper->ios->access = malloc(sizeof(struct accessList));
		accessPointer = nipper->ios->access;
		// Init...
		memset(nipper->ios->access, 0, sizeof(struct accessList));
		strncpy(accessPointer->name, listName, sizeof(accessPointer->name) - 1);
		accessPointer->type = aclType;
	}
	else
	{
		accessPointer = nipper->ios->access;
		while ((accessPointer->next != 0) && (strcmp(accessPointer->name, listName) != 0))
			accessPointer = accessPointer->next;

		// If not found, create...
		if (strcmp(accessPointer->name, listName) != 0)
		{
			// Create storage for access-list...
			accessPointer->next = malloc(sizeof(struct accessList));
			accessPointer = accessPointer->next;
			// Init...
			memset(accessPointer, 0, sizeof(struct accessList));
			strncpy(accessPointer->name, listName, sizeof(accessPointer->name) - 1);
			accessPointer->type = aclType;
		}
	}

	return accessPointer;
}


// Process Access List
void processAccessList(char *line, struct nipperConfig *nipper)
{
	// Variables...
	int tempInt = 0;
	int loop = 0;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];
	struct accessList *accessPointer = 0;
	struct accessListPermission *permissionPointer = 0;
	struct ciscoCommand command;

	// Debug
	if (nipper->debugMode == true)
	{
		printf("Access-List Line: %s\n", line);
	}

	// Init
	command = splitLine(line);
	tempString[0] = 0;
	if (command.parts < 4)
		return;

	// Get access-list type
	if (atoi(command.part[1]) < 100)
			tempInt = ios_acl_standard;
		else
			tempInt = ios_acl_extended;

	// Get / create access list
	accessPointer = getAccessList(command.part[1], tempInt, nipper);

	// Is the line a remark?
	if ((strncmp(command.part[2], "remark", 6) == 0) && (command.parts > 3))
	{
		strcpy(tempString, command.part[3]);
		if (command.parts > 4)
		{
			for (loop = 4; loop < command.parts; loop++)
			{
				sprintf(tempString2, "%s %s",tempString, command.part[loop]);
				strcpy(tempString, tempString2);
			}
		}
		strncpy(accessPointer->remark, tempString, sizeof(accessPointer->remark) - 1);
	}
	else
	{
		tempInt = 2;

		// Is the line a permit / deny line?
		if ((strncmp(command.part[tempInt], "permit", 6) == 0) || (strncmp(command.part[tempInt], "deny", 4) == 0))
		{
			// Is this the first permission
			if (accessPointer->permission == 0)
			{
				// Create a new permission storage structure
				permissionPointer = malloc(sizeof(struct accessListPermission));
				// sort out pointer
				accessPointer->permission = permissionPointer;
			}
			else
			{
				permissionPointer = accessPointer->permission;
				while (permissionPointer->next != 0)
					permissionPointer = permissionPointer->next;
				// Create a new permission storage structure
				permissionPointer->next = malloc(sizeof(struct accessListPermission));
				// Sort out pointer
				permissionPointer = permissionPointer->next;
			}

			// Init...
			memset(permissionPointer, 0, sizeof(struct accessListPermission));
			permissionPointer->sourceOperator = perm_op_none;
			permissionPointer->destOperator = perm_op_none;
			permissionPointer->established = false;
			permissionPointer->fragments = false;
			permissionPointer->log = false;

			// Was it a permit line?
			if (strncmp(command.part[tempInt], "permit", 6) == 0)
				permissionPointer->permission = perm_permit;
			else
				permissionPointer->permission = perm_deny;
			tempInt++;

			// Is it an extended access-list?
			if (accessPointer->type == ios_acl_extended)
			{
				strncpy(permissionPointer->protocol, command.part[tempInt], sizeof(permissionPointer->protocol) -1);
				tempInt++;
			}

			// Get source
			if (strncmp(command.part[tempInt], "host", 4) == 0)
			{
				tempInt++;
				strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "any", 3) == 0)
			{
				strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source) -1);
				tempInt++;
			}
			else
			{
				// Standard access-list?
				if (accessPointer->type == ios_acl_standard)
				{
					strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source) -1);
					tempInt++;
					if ((tempInt < command.parts) && (strncmp(command.part[tempInt], "log", 3) != 0))
					{
						sprintf(permissionPointer->source, "%s %s", command.part[tempInt - 1], command.part[tempInt]);
						tempInt++;
					}
				}
				else
				{
					sprintf(permissionPointer->source, "%s %s", command.part[tempInt], command.part[tempInt + 1]);
					tempInt = tempInt + 2;
				}
			}

			// Standard access-list?
			if (accessPointer->type == ios_acl_standard)
			{
				if ((tempInt < command.parts) && (strncmp(command.part[tempInt], "log", 3) == 0))
				{
					permissionPointer->log = true;
				}
				return;
			}

			// Is operator used with source?
			if (strncmp(command.part[tempInt], "lt", 2) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_lt;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "gt", 2) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_gt;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "eq", 2) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_eq;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "neq", 3) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_neq;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "range", 5) == 0)
			{
				tempInt++;
				permissionPointer->sourceOperator = perm_op_range;
				strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
				tempInt++;
				strncpy(permissionPointer->sourceServiceEnd, command.part[tempInt], sizeof(permissionPointer->sourceServiceEnd) -1);
				tempInt++;
			}

			// Get destination
			if (strncmp(command.part[tempInt], "host", 4) == 0)
			{
				tempInt++;
				strncpy(permissionPointer->dest, command.part[tempInt], sizeof(permissionPointer->dest) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "any", 3) == 0)
			{
				strncpy(permissionPointer->dest, command.part[tempInt], sizeof(permissionPointer->dest) -1);
				tempInt++;
			}
			else
			{
				sprintf(permissionPointer->dest, "%s %s", command.part[tempInt], command.part[tempInt + 1]);
				tempInt = tempInt + 2;
			}

			// If no more, then exit
			if (tempInt == command.parts)
				return;

			// Is operator used with destination?
			if (strncmp(command.part[tempInt], "lt", 2) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_lt;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "gt", 2) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_gt;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "eq", 2) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_eq;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "neq", 3) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_neq;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "range", 5) == 0)
			{
				tempInt++;
				permissionPointer->destOperator = perm_op_range;
				strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
				tempInt++;
				strncpy(permissionPointer->destServiceEnd, command.part[tempInt], sizeof(permissionPointer->destServiceEnd) -1);
				tempInt++;
			}

			// If no more, then exit
			if (tempInt == command.parts)
				return;

			// established?
			if (strncmp(command.part[tempInt], "established", 11) == 0)
			{
				permissionPointer->established = true;
				tempInt++;
			}

			// fragments?
			if (strncmp(command.part[tempInt], "fragments", 9) == 0)
			{
				permissionPointer->fragments = true;
				tempInt++;
			}

			// If no more, then exit
			if (tempInt == command.parts)
				return;

			// Log?
			if ((strncmp(command.part[tempInt], "log", 3) == 0) || (strncmp(command.part[tempInt], "log-input", 9) == 0))
			{
				permissionPointer->log = true;
				tempInt++;
			}
		}
	}
}


// Process IP Access List
void processIPAccessList(char *line, struct nipperConfig *nipper)
{
	// Variables...
	int tempInt = 0;
	int loop;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];
	fpos_t filePosition;
	struct accessList *accessPointer;
	struct accessListPermission *permissionPointer;
	struct ciscoCommand command;

	// Debug
	if (nipper->debugMode == true)
	{
		printf("IP Access-List Line: %s\n", line);
	}

	// Init
	command = splitLine(line);

	// ACL Type
	if (strcmp(command.part[2], "standard") == 0)
		tempInt = ios_acl_standard;
	else
		tempInt = ios_acl_extended;

	// Get / Create ACL...
	accessPointer = getAccessList(command.part[3], tempInt, nipper);

	// Loop through the interface lines
	fgetpos(nipper->input, &filePosition);
	readLine(nipper->input, line, nipper->maxSize);
	while ((feof(nipper->input) == 0) && (line[0] == ' '))
	{
		// Debug
		if (nipper->debugMode == true)
		{
			printf("IP Access-List Line: %s\n", line);
		}

		// Init
		command = splitLine(line);

		// Permit or deny
		if ((strcmp(command.part[0], "permit") == 0) || (strcmp(command.part[0], "deny") == 0))
		{
			// If first permission
			if (accessPointer->permission == 0)
			{
				// Create a new permission storage structure
				permissionPointer = malloc(sizeof(struct accessListPermission));
				// sort out pointer
				accessPointer->permission = permissionPointer;
			}
			else
			{
				permissionPointer = accessPointer->permission;
				while (permissionPointer->next != 0)
					permissionPointer = permissionPointer->next;
				// Create a new permission storage structure
				permissionPointer->next = malloc(sizeof(struct accessListPermission));
				permissionPointer = permissionPointer->next;
			}

			// Init...
			memset(permissionPointer, 0, sizeof(struct accessListPermission));
			permissionPointer->sourceOperator = perm_op_none;
			permissionPointer->destOperator = perm_op_none;
			permissionPointer->established = false;
			permissionPointer->fragments = false;
			permissionPointer->log = false;

			// Permit or Deny
			if (strcmp(command.part[0], "permit") == 0)
				permissionPointer->permission = perm_permit;
			else
				permissionPointer->permission = perm_deny;

			// Protocol
			if (accessPointer->type == ios_acl_extended)
			{
				strncpy(permissionPointer->protocol, command.part[1], sizeof(permissionPointer->protocol) -1);
				tempInt = 2;
			}
			else
				tempInt = 1;

			// Source
			if (strncmp(command.part[tempInt], "host", 4) == 0)
			{
				tempInt++;
				strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source) -1);
				tempInt++;
			}
			else if (strncmp(command.part[tempInt], "any", 3) == 0)
			{
				strncpy(permissionPointer->source, command.part[tempInt], sizeof(permissionPointer->source) -1);
				tempInt++;
			}
			else
			{
				// Standard access-list?
				if (accessPointer->type == ios_acl_standard)
				{
					strcpy(tempString, command.part[tempInt]);
					tempInt++;
					if ((tempInt < command.parts) && (strncmp(command.part[tempInt], "log", 3) != 0))
					{
						sprintf(tempString, "%s %s", command.part[tempInt - 1], command.part[tempInt]);
						tempInt++;
					}
				}
				else
				{
					sprintf(tempString, "%s %s", command.part[tempInt], command.part[tempInt + 1]);
					tempInt += 2;
				}
				strncpy(permissionPointer->source, tempString, sizeof(permissionPointer->source) -1);
			}

			// Standard ACL only...
			if (accessPointer->type == ios_acl_standard)
			{
				// Log?
				if ((strncmp(command.part[tempInt], "log", 3) == 0))
				{
					permissionPointer->log = true;
					tempInt++;
				}
			}

			// Extended ACL only...
			if (accessPointer->type == ios_acl_extended)
			{
				// Is operator used with source?
				if (strncmp(command.part[tempInt], "lt", 2) == 0)
				{
					tempInt++;
					permissionPointer->sourceOperator = perm_op_lt;
					strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
					tempInt++;
				}
				else if (strncmp(command.part[tempInt], "gt", 2) == 0)
				{
					tempInt++;
					permissionPointer->sourceOperator = perm_op_gt;
					strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
					tempInt++;
				}
				else if (strncmp(command.part[tempInt], "eq", 2) == 0)
				{
					tempInt++;
					permissionPointer->sourceOperator = perm_op_eq;
					strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
					tempInt++;
				}
				else if (strncmp(command.part[tempInt], "neq", 3) == 0)
				{
					tempInt++;
					permissionPointer->sourceOperator = perm_op_neq;
					strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
					tempInt++;
				}
				else if (strncmp(command.part[tempInt], "range", 5) == 0)
				{
					tempInt++;
					permissionPointer->sourceOperator = perm_op_range;
					strncpy(permissionPointer->sourceService, command.part[tempInt], sizeof(permissionPointer->sourceService) -1);
					tempInt++;
					strncpy(permissionPointer->sourceServiceEnd, command.part[tempInt], sizeof(permissionPointer->sourceServiceEnd) -1);
					tempInt++;
				}

				// Get destination
				if (strncmp(command.part[tempInt], "host", 4) == 0)
				{
					tempInt++;
					strncpy(permissionPointer->dest, command.part[tempInt], sizeof(permissionPointer->dest) -1);
					tempInt++;
				}
				else if (strncmp(command.part[tempInt], "any", 3) == 0)
				{
					strncpy(permissionPointer->dest, command.part[tempInt], sizeof(permissionPointer->dest) -1);
					tempInt++;
				}
				else
				{
					sprintf(permissionPointer->dest, "%s %s", command.part[tempInt], command.part[tempInt + 1]);
					tempInt = tempInt + 2;
				}

				// Get all the rest...
				while (tempInt < command.parts)
				{
					// Is operator used with destination?
					if (strncmp(command.part[tempInt], "lt", 2) == 0)
					{
						tempInt++;
						permissionPointer->destOperator = perm_op_lt;
						strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
					}
					else if (strncmp(command.part[tempInt], "gt", 2) == 0)
					{
						tempInt++;
						permissionPointer->destOperator = perm_op_gt;
						strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
					}
					else if (strncmp(command.part[tempInt], "eq", 2) == 0)
					{
						tempInt++;
						permissionPointer->destOperator = perm_op_eq;
						strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
					}
					else if (strncmp(command.part[tempInt], "neq", 3) == 0)
					{
						tempInt++;
						permissionPointer->destOperator = perm_op_neq;
						strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
					}
					else if (strncmp(command.part[tempInt], "range", 5) == 0)
					{
						tempInt++;
						permissionPointer->destOperator = perm_op_range;
						strncpy(permissionPointer->destService, command.part[tempInt], sizeof(permissionPointer->destService) -1);
						tempInt++;
						strncpy(permissionPointer->destServiceEnd, command.part[tempInt], sizeof(permissionPointer->destServiceEnd) -1);
					}

					// established?
					else if (strncmp(command.part[tempInt], "established", 11) == 0)
						permissionPointer->established = true;
		
					// fragments?
					else if (strncmp(command.part[tempInt], "fragments", 9) == 0)
						permissionPointer->fragments = true;

					// Log?
					else if ((strncmp(command.part[tempInt], "log", 3) == 0) || (strncmp(command.part[tempInt], "log-input", 9) == 0))
						permissionPointer->log = true;

					tempInt++;
				}
			}
		}

		// Remark...
		else if (strcmp(command.part[0], "remark") == 0)
		{
			strcpy(tempString, command.part[1]);
			if (command.parts > 2)
			{
				for (loop = 2; loop < command.parts; loop++)
				{
					sprintf(tempString2, "%s %s",tempString, command.part[loop]);
					strcpy(tempString, tempString2);
				}
			}
			strncpy(accessPointer->remark, tempString, sizeof(accessPointer->remark) - 1);
		}

		// Debug (lines not processed)
		else if (nipper->linesnotprocessed == true)
			printf("%s\n", line);

		// Get next line...
		fgetpos(nipper->input, &filePosition);
		readLine(nipper->input, line, nipper->maxSize);
	}

	// Set to previous position...
	fsetpos(nipper->input, &filePosition);
}
