/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

void generateNMPReport(struct nipperConfig *nipper)
{
	// Variables
	FILE *reportFile;
	int tempInt = 0;
	int issues = 0;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];
	struct snmpNMPCommunity *communityNMPPointer = 0;
	struct snmpNMPHost *snmpNMPHostPointer = 0;
	struct snmpTrap *snmpTrapPointer = 0;
	struct hostNMP *hostNMPPointer = 0;
	struct moduleConfig *moduleNMPPointer = 0;
	struct portConfig *portNMPPointer = 0;
	struct vlanConfig *vlanNMPPointer = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (nipper->outputFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (nipper->outputFile[0] == 0)
		nipper->reportFile = stdout;
	else
	{
		nipper->reportFile = fopen(nipper->outputFile, "w");
		if (reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, nipper->outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(nipper->hostname, nipper->friendlyHostname, nipper->maxSize, nipper->outputFormat);

	// Output file heading (if required)
	output_filestart(nipper);

	// Report section 
	if (nipper->fullReport == true)
	{
		output_parseSectionIndex(report_security_title, nipper);

		// Start of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "<ul>\n");
				break;
		}
		output_parseIndex("Introduction", nipper, section_security);

		// Simple Passwords
		if (nipper->simplePasswords > 1)
		{
			output_parseIndex(report_dict_title1, nipper, section_security);
			issues++;
		}
		else if (nipper->simplePasswords > 0)
		{
			output_parseIndex(report_dict_title2, nipper, section_security);
			issues++;
		}

		// Password Strength
		if (nipper->passwordStrengths > 1)
		{
			output_parseIndex(report_strength_title1, nipper, section_security);
			issues++;
		}
		else if (nipper->passwordStrengths > 0)
		{
			output_parseIndex(report_strength_title2, nipper, section_security);
			issues++;
		}

		// Timeout
		if (nipper->nmp->timeout * 60 > nipper->connectionTimeout)
			nipper->insecureTimeouts++;
		if (nipper->insecureTimeouts > 0)
		{
			output_parseIndex(report_timeout_title, nipper, section_security);
			issues++;
		}

		// IP Redirects
		if (nipper->nmp->switchConfig != 0)
		{
			if (nipper->nmp->switchConfig->ipRedirect == true)
			{
				output_parseIndex(report_redirects_title, nipper, section_security);
				issues++;
			}
		}

		// CDP
		if (nipper->nmp->cdp == true)
		{
			output_parseIndex(report_cdp_title, nipper, section_security);
			issues++;
		}

		// IP Unreachables
		if (nipper->nmp->switchConfig != 0)
		{
			if (nipper->nmp->switchConfig->ipUnreachable == true)
			{
				output_parseIndex(report_icmpunreach_title, nipper, section_security);
				issues++;
			}
		}

		// End of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "</ul>\n");
				break;
			case output_text:
				fprintf(nipper->reportFile, "\n");
				break;
		}
	}

	// ------------------------------
	// Configuration Section Contents
	// ------------------------------
	output_parseSectionIndex(report_config_title, nipper);

	// Start of configuration index
	switch (nipper->outputFormat)
	{
		case output_html:
			fprintf(nipper->reportFile, "<ul>\n");
			break;
	}
	output_parseIndex("Introduction", nipper, section_config);

	// In Configuration Index
	output_parseIndex(settings_general_title, nipper, section_config);
	if (((nipper->nmp->enablePassword != 0) || (nipper->nmp->password != 0)) && (nipper->passwords == true))
		output_parseIndex(settings_passwords_title, nipper, section_config);
	if (nipper->nmp->switchConfig != 0)
		output_parseIndex(settings_switch_title, nipper, section_config);
	output_parseIndex(settings_cdp_title, nipper, section_config);
	if (nipper->nmp->snmp != 0)
		output_parseIndex(settings_snmp_title, nipper, section_config);
	if (nipper->nmp->permit != 0)
		output_parseIndex(settings_permit_title, nipper, section_config);
	if (nipper->nmp->vlan != 0)
		output_parseIndex(settings_vlan_title, nipper, section_config);
	if (nipper->nmp->module != 0)
		output_parseIndex(settings_ports_title, nipper, section_config);

	// End of index
	output_abbrevContents(nipper);
	output_endsection(nipper, subsection_end);

	// Security Report section
	if (nipper->fullReport == true)
	{
		// Output report introduction
		output_parseTitle(report_security_title, nipper, rate_none, section_main_sec);
		output_parseTitle("Introduction", nipper, rate_none, section_security);
		output_parseText(report_security_intro, nipper, section_none, rate_none, issues, 0);
		output_endsection(nipper, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		// Dictionary-based passwords / keys
		if (nipper->simplePasswords > 0)
		{
			if (nipper->simplePasswords > 1)
				output_parseTitle(report_dict_title1, nipper, rate_high, section_security);
			else
				output_parseTitle(report_dict_title2, nipper, rate_high, section_security);
			output_parseText(report_dict_obs, nipper, section_obs, rate_none, nipper->simplePasswords, nipper->tableNum);
			if (nipper->passwordStrengths > 1)
			{
				output_table(nipper, true, report_dict_table, &report_dict_heading);
				output_NMPTablePasswords(nipper, true);
				output_table(nipper, false, report_dict_table, &report_dict_heading);
			}
			else
				output_NMPLinePasswords(nipper, true);
			output_endsection(nipper, section_obs);
			output_parseText(report_dict_impact, nipper, section_imp, rate_high, nipper->simplePasswords, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_dict_ease, nipper, section_ease, rate_trivial, nipper->simplePasswords, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_dict_rec, nipper, section_rec, rate_none, nipper->simplePasswords, nipper->tableNum);
			numToWord(nipper->reportFile, nipper->passwordLength);
			fprintf(nipper->reportFile, " characters in length and contain");
			if (nipper->passwordUppers == true)
				tempInt = 1;
			else
				tempInt = 0;
			if (nipper->passwordLowers == true)
				tempInt++;
			if (nipper->passwordEither == true)
				tempInt++;
			if (nipper->passwordNumbers == true)
				tempInt++;
			if (nipper->passwordSpecials == true)
				tempInt++;
			if (nipper->passwordUppers == true)
			{
				fprintf(nipper->reportFile, " uppercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordLowers == true)
			{
				fprintf(nipper->reportFile, " lowercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordEither == true)
			{
				fprintf(nipper->reportFile, " either uppercase or lowercase characters");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordNumbers == true)
			{
				fprintf(nipper->reportFile, " numbers");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordSpecials == true)
			{
				fprintf(nipper->reportFile, " special");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (((nipper->passwordNumbers == true) && (nipper->passwordSpecials == false)) || ((nipper->passwordEither == false) && (nipper->passwordNumbers == false) && (nipper->passwordSpecials == false)))
				fprintf(nipper->reportFile, ".");
			else
				fprintf(nipper->reportFile, "characters.");
			if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "</text>\n");
			output_newPara(nipper);
			output_endsection(nipper, section_rec);
		}

		// Weak passwords / keys
		if (nipper->passwordStrengths > 0)
		{
			if (nipper->passwordStrengths > 1)
				output_parseTitle(report_strength_title1, nipper, rate_high, section_security);
			else
				output_parseTitle(report_strength_title2, nipper, rate_high, section_security);
			output_parseText(report_strength_obs, nipper, section_obs, rate_none, nipper->passwordStrengths, nipper->tableNum);
			if (nipper->passwordStrengths > 1)
			{
				output_table(nipper, true, report_strength_table, &report_dict_heading);
				output_NMPTablePasswords(nipper, false);
				output_table(nipper, false, report_strength_table, &report_dict_heading);
			}
			else
				output_NMPLinePasswords(nipper, false);
			output_endsection(nipper, section_obs);
			output_parseText(report_strength_impact, nipper, section_imp, rate_high, 0, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_strength_ease, nipper, section_ease, rate_trivial, 0, 0);
			output_endsection(nipper, section_ease);
			output_parseText(report_strength_rec, nipper, section_rec, rate_trivial, 0, 0);
			numToWord(nipper->reportFile, nipper->passwordLength);
			fprintf(nipper->reportFile, " characters in length and contain");
			if (nipper->passwordUppers == true)
				tempInt = 1;
			else
				tempInt = 0;
			if (nipper->passwordLowers == true)
				tempInt++;
			if (nipper->passwordEither == true)
				tempInt++;
			if (nipper->passwordNumbers == true)
				tempInt++;
			if (nipper->passwordSpecials == true)
				tempInt++;
			if (nipper->passwordUppers == true)
			{
				fprintf(nipper->reportFile, " uppercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordLowers == true)
			{
				fprintf(nipper->reportFile, " lowercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordEither == true)
			{
				fprintf(nipper->reportFile, " either uppercase or lowercase characters");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordNumbers == true)
			{
				fprintf(nipper->reportFile, " numbers");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordSpecials == true)
			{
				fprintf(nipper->reportFile, " special");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (((nipper->passwordNumbers == true) && (nipper->passwordSpecials == false)) || ((nipper->passwordEither == false) && (nipper->passwordNumbers == false) && (nipper->passwordSpecials == false)))
				fprintf(nipper->reportFile, ".");
			else
				fprintf(nipper->reportFile, "characters.");
			if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "</text>\n");
			output_newPara(nipper);
			output_endsection(nipper, section_rec);
		}

		// Timeout
		if (nipper->insecureTimeouts > 0)
		{
			output_parseTitle(report_timeout_title, nipper, rate_med, section_security);
			output_parseText(report_timeout_obs1, nipper, section_obs, rate_none, nipper->insecureTimeouts, nipper->tableNum);
			if ((nipper->connectionTimeout % 60) == 0)
			{
				numToWord(nipper->reportFile, (nipper->connectionTimeout / 60));
				fprintf(nipper->reportFile, " minutes");
			}
			else
			{
				numToWord(nipper->reportFile, nipper->connectionTimeout / 60);
				fprintf(nipper->reportFile, " minutes and ");
				numToWord(nipper->reportFile, nipper->connectionTimeout % 60);
				fprintf(nipper->reportFile, " seconds");
			}
			output_parseText(report_timeout_obs2, nipper, section_none, rate_none, nipper->insecureTimeouts, nipper->tableNum);
			if (nipper->insecureTimeouts > 1)
			{
				output_table(nipper, true, report_timeout_table, &report_timeout_heading);
				if (nipper->nmp->timeout * 60 > nipper->connectionTimeout)
				{
					fprintf(nipper->reportFile, "%sSession Timeout%s", tablerow_start, tablerow_mid);
					numToWord(nipper->reportFile, nipper->nmp->timeout);
					if (nipper->nmp->timeout > 1)
						fprintf(nipper->reportFile, " minutes%s", tablerow_end);
					else
						fprintf(nipper->reportFile, " minute%s", tablerow_end);
				}
				output_table(nipper, false, report_timeout_table, &report_timeout_heading);
			}
			else
			{
				if (nipper->nmp->timeout * 60 > nipper->connectionTimeout)
				{
					fprintf(nipper->reportFile, "The session timeout was ");
					numToWord(nipper->reportFile, nipper->nmp->timeout);
					if (nipper->nmp->timeout > 1)
						fprintf(nipper->reportFile, " minutes.");
					else
						fprintf(nipper->reportFile, " minute.");
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_timeout_impact, nipper, section_imp, rate_high, 0, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_timeout_ease, nipper, section_ease, rate_difficult, 0, 0);
			output_endsection(nipper, section_ease);
			output_parseText(report_timeout_rec1, nipper, section_rec, rate_none, 0, 0);
			if ((nipper->connectionTimeout % 60) == 0)
			{
				numToWord(nipper->reportFile, (nipper->connectionTimeout / 60));
				fprintf(nipper->reportFile, " minutes");
			}
			else
			{
				numToWord(nipper->reportFile, nipper->connectionTimeout / 60);
				fprintf(nipper->reportFile, " minutes and ");
				numToWord(nipper->reportFile, (nipper->connectionTimeout % 60));
				fprintf(nipper->reportFile, " seconds");
			}
			output_parseText(report_timeout_rec2, nipper, section_none, rate_none, 0, 0);
			output_endsection(nipper, section_rec);
		}

		// ICMP Redirects
		if (nipper->nmp->switchConfig != 0)
		{
			if (nipper->nmp->switchConfig->ipRedirect == true)
			{
				output_parseTitle(report_redirects_title, nipper, rate_med, section_security);
				output_parseText(report_redirects_obs, nipper, section_obs, rate_none, -1, nipper->tableNum);
				output_endsection(nipper, section_obs);
				output_parseText(report_redirects_impact, nipper, section_imp, rate_med, 0, 0);
				output_endsection(nipper, section_imp);
				output_parseText(report_redirects_ease, nipper, section_ease, rate_trivial, 0, 0);
				output_endsection(nipper, section_ease);
				output_parseText(report_redirects_rec2, nipper, section_rec, rate_none, 0, 0);
				output_endsection(nipper, section_rec);
			}
		}

		// CDP
		if (nipper->nmp->cdp == true)
		{
			output_parseTitle(report_cdp_title, nipper, rate_low, section_security);
			output_parseText(report_cdp_obs, nipper, section_none, rate_none, -1, 0);
			output_endsection(nipper, section_obs);
			output_parseText(report_cdp_impact, nipper, section_imp, rate_low, 0, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_cdp_ease, nipper, section_ease, rate_trivial, 0, 0);
			output_endsection(nipper, section_ease);
			output_parseText(report_cdp_rec2, nipper, section_rec, rate_none, 0, 0);
			output_endsection(nipper, section_rec);
		}

		// IP Unreachables
		if (nipper->nmp->switchConfig != 0)
		{
			if (nipper->nmp->switchConfig->ipUnreachable == true)
			{
				output_parseTitle(report_icmpunreach_title, nipper, rate_low, section_security);
				output_parseText(report_icmpunreach_obs, nipper, section_obs, rate_none, -1, nipper->tableNum);
				output_endsection(nipper, section_obs);
				output_parseText(report_icmpunreach_impact, nipper, section_imp, rate_low, 0, 0);
				output_endsection(nipper, section_imp);
				output_parseText(report_icmpunreach_ease, nipper, section_ease, rate_trivial, 0, 0);
				output_endsection(nipper, section_ease);
				output_parseText(report_icmpunreach_rec2, nipper, section_rec, rate_none, 0, 0);
				output_endsection(nipper, section_rec);
			}
		}

		// End Section
		output_endsection(nipper, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	output_parseTitle(report_config_title, nipper, rate_none, section_main_con);
	output_parseTitle("Introduction", nipper, rate_none, section_config);
	output_parseText(report_config_intro, nipper, section_none, rate_none, 0, 0);
	output_endsection(nipper, subsection_end);

	// General Device Settings
	output_parseTitle(settings_general_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_general_table, &settings_general_heading);
	fprintf(nipper->reportFile, "%sHostname%s%s %s", tablerow_start, tablerow_mid, nipper->friendlyHostname, tablerow_end);
	if (nipper->version[0] != 0)
		fprintf(nipper->reportFile, "%sNMP Version%s%s %s", tablerow_start, tablerow_mid, nipper->version, tablerow_end);
	if (nipper->nmp->location[0] != 0)
		fprintf(nipper->reportFile, "%sLocation%s%s %s", tablerow_start, tablerow_mid, nipper->nmp->location, tablerow_end);
	if (nipper->nmp->contact[0] != 0)
		fprintf(nipper->reportFile, "%sContact%s%s %s", tablerow_start, tablerow_mid, nipper->nmp->contact, tablerow_end);
	if (nipper->nmp->coreFile[0] != 0)
		fprintf(nipper->reportFile, "%sCore File%s%s %s", tablerow_start, tablerow_mid, nipper->nmp->coreFile, tablerow_end);
	if (nipper->nmp->syslogFile[0] != 0)
		fprintf(nipper->reportFile, "%sSyslog File%s%s %s", tablerow_start, tablerow_mid, nipper->nmp->syslogFile, tablerow_end);
	if (nipper->nmp->timeout == 0)
		fprintf(nipper->reportFile, "%sIdle Session Timeout%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
	else
	{
		fprintf(nipper->reportFile, "%sIdle Session Timeout%s", tablerow_start, tablerow_mid);
		numToWord(nipper->reportFile, nipper->nmp->timeout);
		fprintf(nipper->reportFile, " minutes%s", tablerow_end);
	}
	if (nipper->nmp->portSecurityAuto == true)
		fprintf(nipper->reportFile, "%sPort Security Auto Configure%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sPort Security Auto Configure%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(nipper, false, settings_general_table, &settings_general_heading);
	output_endsection(nipper, subsection_end);
	addAbbreviation("NMP", false);
	addAbbreviation("CDP", false);

	// Passwords
	if ((nipper->nmp->enablePassword != 0) || (nipper->nmp->password != 0))
	{
		output_parseTitle(settings_passwords_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_passwords_title, &settings_passwords_heading);
		if (nipper->nmp->enablePassword[0] != 0)
		{
			fprintf(nipper->reportFile, "%sEnable Password", tablerow_start);
			if (nipper->passwords == true)
				fprintf(nipper->reportFile, "%s%s %s", tablerow_mid, nipper->nmp->enablePassword, tablerow_end);
			else
				fprintf(nipper->reportFile, "%s", tablerow_end);
		}
		if (nipper->nmp->password[0] != 0)
		{
			fprintf(nipper->reportFile, "%sLogin Password", tablerow_start);
			if (nipper->passwords == true)
				fprintf(nipper->reportFile, "%s%s %s", tablerow_mid, nipper->nmp->password, tablerow_end);
			else
				fprintf(nipper->reportFile, "%s", tablerow_end);
		}
		output_table(nipper, false, settings_passwords_title, &settings_passwords_heading);
		output_endsection(nipper, subsection_end);
	}

	// Switch Configuration
	if (nipper->nmp->switchConfig != 0)
	{
		output_parseTitle(settings_switch_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_switch_table, &settings_general_heading);
		if (nipper->nmp->switchConfig->ipRedirect == true)
			fprintf(nipper->reportFile, "%sICMP Redirects%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sICMP Redirects%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->nmp->switchConfig->ipUnreachable == true)
			fprintf(nipper->reportFile, "%sICMP Unreachables%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sICMP Unreachables%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->nmp->switchConfig->ipFragmentation == true)
			fprintf(nipper->reportFile, "%sIP Fragmentation%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sIP Fragmentation%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(nipper, false, settings_switch_table, &settings_general_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("IP", false);
		addAbbreviation("ICMP", false);
	}

	// CDP
	output_parseTitle(settings_cdp_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_cdp_table, &settings_general_heading);
	if (nipper->nmp->cdp == true)
		fprintf(nipper->reportFile, "%sCisco Discovery Protocol%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sCisco Discovery Protocol%sGlobally Disabled%s", tablerow_start, tablerow_mid, tablerow_end);
	fprintf(nipper->reportFile, "%sCDP Version%s%d%s", tablerow_start, tablerow_mid, nipper->nmp->cdpVersion, tablerow_end);
	fprintf(nipper->reportFile, "%sCDP Message Interval%s", tablerow_start, tablerow_mid);
	if (nipper->nmp->cdpInterval % 60 == 0)
	{
		numToWord(nipper->reportFile, nipper->nmp->cdpInterval / 60);
		if (nipper->nmp->cdpInterval / 60 > 1)
			fprintf(nipper->reportFile, " minutes%s", tablerow_end);
		else
			fprintf(nipper->reportFile, " minute%s", tablerow_end);
	}
	else
	{
		numToWord(nipper->reportFile, nipper->nmp->cdpInterval / 60);
		if (nipper->nmp->cdpInterval / 60 > 1)
			fprintf(nipper->reportFile, " minutes ");
		else
			fprintf(nipper->reportFile, " minute ");
		numToWord(nipper->reportFile, nipper->nmp->cdpInterval % 60);
		if (nipper->nmp->cdpInterval % 60 > 1)
			fprintf(nipper->reportFile, " seconds%s", tablerow_end);
		else
			fprintf(nipper->reportFile, " second%s", tablerow_end);
	}
	fprintf(nipper->reportFile, "%sCDP TTL%s", tablerow_start, tablerow_mid);
	if (nipper->nmp->cdpHoldtime % 60 == 0)
	{
		numToWord(nipper->reportFile, nipper->nmp->cdpHoldtime / 60);
		if (nipper->nmp->cdpHoldtime / 60 > 1)
			fprintf(nipper->reportFile, " minutes%s", tablerow_end);
		else
			fprintf(nipper->reportFile, " minute%s", tablerow_end);
	}
	else
	{
		numToWord(nipper->reportFile, nipper->nmp->cdpHoldtime / 60);
		if (nipper->nmp->cdpHoldtime / 60 > 1)
			fprintf(nipper->reportFile, " minutes ");
		else
			fprintf(nipper->reportFile, " minute ");
		numToWord(nipper->reportFile, nipper->nmp->cdpHoldtime % 60);
		if (nipper->nmp->cdpHoldtime % 60 > 1)
			fprintf(nipper->reportFile, " seconds%s", tablerow_end);
		else
			fprintf(nipper->reportFile, " second%s", tablerow_end);
	}
	output_table(nipper, false, settings_cdp_table, &settings_general_heading);
	addAbbreviation("CDP", false);
	addAbbreviation("TTL", false);

	// SNMP
	if (nipper->nmp->snmp != 0)
	{
		output_parseTitle(settings_snmp_title, nipper, rate_none, section_config);
		
		if (nipper->nmp->snmp->community != 0)
		{
			output_table(nipper, true, settings_snmpstring_table, &settings_snmpcomm_heading);
			communityNMPPointer = nipper->nmp->snmp->community;
			while (communityNMPPointer != 0)
			{
				if (communityNMPPointer->type == snmp_nmp_standard)
					fprintf(nipper->reportFile, "%sStandard%s", tablerow_start, tablerow_mid);
				else
					fprintf(nipper->reportFile, "%sExtended%s", tablerow_start, tablerow_mid);
				if (nipper->passwords == true)
					fprintf(nipper->reportFile, "%s %s", communityNMPPointer->community, tablerow_mid);
				switch (communityNMPPointer->access)
				{
					case snmp_nmp_readonly:
						fprintf(nipper->reportFile, "Read-Only%s", tablerow_end);
						break;
					case snmp_nmp_readwrite:
						fprintf(nipper->reportFile, "Read-Write%s", tablerow_end);
						break;
					case snmp_nmp_readwrite_all:
						fprintf(nipper->reportFile, "Read-Write-All%s", tablerow_end);
						break;
				}
				communityNMPPointer = communityNMPPointer->next;
			}
			output_table(nipper, false, settings_snmpstring_table, &settings_snmpcomm_heading);
		}
		if (nipper->nmp->snmp->trap != 0)
		{
			output_table(nipper, true, settings_snmptrap_table, &settings_snmptraps_heading);
			snmpTrapPointer = nipper->nmp->snmp->trap;
			while (snmpTrapPointer != 0)
			{
				if (snmpTrapPointer->enabled == true)
					fprintf(nipper->reportFile, "%sEnabled", tablerow_start);
				else
					fprintf(nipper->reportFile, "%sDisabled", tablerow_start);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, snmpTrapPointer->trap, tablerow_end);
				addAbbreviation(snmpTrapPointer->trap, false);
				snmpTrapPointer = snmpTrapPointer->next;
			}
			output_table(nipper, false, settings_snmptrap_table, &settings_snmptraps_heading);
		}
		if (nipper->nmp->snmp->host != 0)
		{
			output_table(nipper, true, settings_snmphost_table, &settings_snmphosts_heading);
			snmpNMPHostPointer = nipper->nmp->snmp->host;
			while (snmpNMPHostPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, snmpNMPHostPointer->host, tablerow_mid);
				if (nipper->passwords == true)
					fprintf(nipper->reportFile, "%s%s", snmpNMPHostPointer->community, tablerow_mid);
				fprintf(nipper->reportFile, "%d%s", snmpNMPHostPointer->version, tablerow_mid);
				fprintf(nipper->reportFile, "%d%s", snmpNMPHostPointer->port, tablerow_mid);
				fprintf(nipper->reportFile, "%s%s", snmpNMPHostPointer->owner, tablerow_mid);
				fprintf(nipper->reportFile, "%d%s", snmpNMPHostPointer->index, tablerow_end);
				snmpNMPHostPointer = snmpNMPHostPointer->next;
			}
			output_table(nipper, false, settings_snmphost_table, &settings_snmphosts_heading);
		}
		output_endsection(nipper, subsection_end);
		addAbbreviation("SNMP", false);
	}

	// Management Service Permit Lists
	if (nipper->nmp->permit != 0)
	{
		output_parseTitle(settings_permit_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_permit_table, &settings_permit_heading);
		if (nipper->nmp->permit->telnet == true)
			fprintf(nipper->reportFile, "%sTelnet%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->nmp->permit->telnet == false)
			fprintf(nipper->reportFile, "%sTelnet%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sTelnet%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->nmp->permit->ssh == true)
			fprintf(nipper->reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->nmp->permit->ssh == false)
			fprintf(nipper->reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sSSH%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->nmp->permit->snmp == true)
			fprintf(nipper->reportFile, "%sSNMP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->nmp->permit->snmp == false)
			fprintf(nipper->reportFile, "%sSNMP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sSNMP%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(nipper, false, settings_permit_table, &settings_permit_heading);
		addAbbreviation("SNMP", false);
		addAbbreviation("SSH", false);

		// Telnet Permission List
		if (nipper->nmp->permit->telnet == true)
		{
			output_table(nipper, true, "Telnet permit list", &settings_permitlist_heading);
			hostNMPPointer = nipper->nmp->permit->telnetHost;
			while (hostNMPPointer != 0)
			{
				outputFriendly(hostNMPPointer->host, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s%s%s", tablerow_start, tempString, tablerow_mid, hostNMPPointer->netMask, tablerow_end);
				hostNMPPointer = hostNMPPointer->next;
			}
			output_table(nipper, false, "Telnet permit list", &settings_permitlist_heading);
		}

		// SSH Permission List
		if (nipper->nmp->permit->ssh == true)
		{
			output_table(nipper, true, "SSH permit list", &settings_permitlist_heading);
			hostNMPPointer = nipper->nmp->permit->sshHost;
			while (hostNMPPointer != 0)
			{
				outputFriendly(hostNMPPointer->host, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s%s%s", tablerow_start, tempString, tablerow_mid, hostNMPPointer->netMask, tablerow_end);
				hostNMPPointer = hostNMPPointer->next;
			}
			output_table(nipper, false, "SSH permit list", &settings_permitlist_heading);
		}

		// SNMP Permission List
		if (nipper->nmp->permit->snmp == true)
		{
			output_table(nipper, true, "SNMP permit list", &settings_permitlist_heading);
			hostNMPPointer = nipper->nmp->permit->snmpHost;
			while (hostNMPPointer != 0)
			{
				outputFriendly(hostNMPPointer->host, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s%s%s", tablerow_start, tempString, tablerow_mid, hostNMPPointer->netMask, tablerow_end);
				hostNMPPointer = hostNMPPointer->next;
			}
			output_table(nipper, false, "SNMP permit list", &settings_permitlist_heading);
		}
	}

	// VLAN Configuration
	if (nipper->nmp->vlan != 0)
	{
		output_parseTitle(settings_vlan_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_vlan_table, &settings_vlan_heading);
		vlanNMPPointer = nipper->nmp->vlan;
		while(vlanNMPPointer != 0)
		{
			fprintf(nipper->reportFile, "%s%d%s", tablerow_start, vlanNMPPointer->vlan, tablerow_mid);
			fprintf(nipper->reportFile, "%s%s", vlanNMPPointer->name, tablerow_mid);
			if (vlanNMPPointer->active == true)
				fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "No%s", tablerow_mid);
			fprintf(nipper->reportFile, "%s%s", vlanNMPPointer->type, tablerow_mid);
			if (vlanNMPPointer->firewall == -1)
				fprintf(nipper->reportFile, "None%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d%s", vlanNMPPointer->firewall, tablerow_mid);
			fprintf(nipper->reportFile, "%d%s", vlanNMPPointer->said, tablerow_mid);
			fprintf(nipper->reportFile, "%d%s", vlanNMPPointer->mtu, tablerow_mid);
			fprintf(nipper->reportFile, "%d%s", vlanNMPPointer->spanTreeMaxHops, tablerow_mid);
			fprintf(nipper->reportFile, "%d%s", vlanNMPPointer->allRoutesMaxHops, tablerow_end);
			vlanNMPPointer = vlanNMPPointer->next;
		}
		output_table(nipper, false, settings_vlan_table, &settings_vlan_heading);
		addAbbreviation("SAID", false);
		addAbbreviation("VLAN", false);
		addAbbreviation("MTU", false);
	}

	if (nipper->nmp->module != 0)
	{
		output_parseTitle(settings_ports_title, nipper, rate_none, section_config);
		// Modules / Ports
		moduleNMPPointer = nipper->nmp->module;
		while (moduleNMPPointer != 0)
		{
			if (moduleNMPPointer->name[0] == 0)
				sprintf(tempString, "Module %d port configuration", moduleNMPPointer->module);
			else
				sprintf(tempString, "Module %d (%s) port configuration", moduleNMPPointer->module, moduleNMPPointer->name);
			output_table(nipper, true, tempString, &settings_ports_heading);
			portNMPPointer = moduleNMPPointer->ports;
			while (portNMPPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%d%s", tablerow_start, portNMPPointer->port, tablerow_mid);
				outputFriendly(portNMPPointer->name, tempString2, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString2, tablerow_mid);
				if (portNMPPointer->enabled == true)
					fprintf(nipper->reportFile, "Enabled%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Disabled%s", tablerow_mid);
				fprintf(nipper->reportFile, "%d%s", portNMPPointer->vlan, tablerow_mid);
				switch (portNMPPointer->speed)
				{
					case port_speed_auto:
						fprintf(nipper->reportFile, "Auto%s", tablerow_mid);
						break;
					case port_speed_10:
						fprintf(nipper->reportFile, "10Mb%s", tablerow_mid);
						break;
					case port_speed_100:
						fprintf(nipper->reportFile, "100Mb%s", tablerow_mid);
						break;
					case port_speed_1000:
						fprintf(nipper->reportFile, "1000Mb%s", tablerow_mid);
						break;
					case port_speed_auto_10_100:
						fprintf(nipper->reportFile, "10/100Mb%s", tablerow_mid);
						break;
				}
				switch (portNMPPointer->duplex)
				{
					case port_duplex_auto:
						fprintf(nipper->reportFile, "Auto%s", tablerow_mid);
						break;
					case port_duplex_half:
						fprintf(nipper->reportFile, "Half%s", tablerow_mid);
						break;
					case port_duplex_full:
						fprintf(nipper->reportFile, "Full%s", tablerow_mid);
						break;
				}
				if (portNMPPointer->portSecurity == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				if (portNMPPointer->cdp == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				switch (portNMPPointer->trunk)
				{
					case port_trunk_auto:
						fprintf(nipper->reportFile, "Auto%s", tablerow_mid);
						break;
					case port_trunk_nonegotiate:
						fprintf(nipper->reportFile, "No Negotiate%s", tablerow_mid);
						break;
					case port_trunk_on:
						fprintf(nipper->reportFile, "On%s", tablerow_mid);
						break;
					case port_trunk_off:
						fprintf(nipper->reportFile, "Off%s", tablerow_mid);
						break;
					case port_trunk_desirable:
						fprintf(nipper->reportFile, "Desirable%s", tablerow_mid);
						break;
				}
				if (portNMPPointer->vtp == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				switch (portNMPPointer->spantreeGuard)
				{
					case port_guard_none:
						fprintf(nipper->reportFile, "Off%s", tablerow_end);
						break;
					case port_guard_default:
						fprintf(nipper->reportFile, "On%s", tablerow_end);
						break;
					case port_guard_loop:
						fprintf(nipper->reportFile, "On (loop)%s", tablerow_end);
						break;
					case port_guard_root:
						fprintf(nipper->reportFile, "On (root)%s", tablerow_end);
						break;
				}
				portNMPPointer = portNMPPointer->next;
			}
			output_table(nipper, false, tempString, &settings_ports_heading);
			moduleNMPPointer = moduleNMPPointer->next;
		}
		addAbbreviation("VLAN", false);
		addAbbreviation("STP", false);
	}

	// End of Config Section
	output_endsection(nipper, section_end);

	// Output end of file stuff
	output_fileend(nipper);
}

