/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


void cleanupPIX(struct nipperConfig *nipper)
{
	// Pointers
	struct interfacePIXConfig *interfacePIXPointer = 0;
	struct accessListPIX *accessPIXPointer = 0;
	struct accessListPIXPermission *permissionPIXPointer = 0;
	struct objectGroupList *objectGroupPointer = 0;
	struct objectList *objectPointer = 0;
	struct namePIXList *namePIXPointer = 0;
	struct enablePIXPassword *enablePIXPointer = 0;
	struct usernamePIX *usernamePIXPointer = 0;
	struct fixupList *fixupPIXPointer = 0;
	struct icmpAccess *icmpPIXPointer = 0;
	struct staticPIX *staticPIXPointer = 0;
	struct routePIX *routePIXPointer = 0;

	// Cleanup Interfaces
	while (nipper->pix->interface != 0)
	{
		interfacePIXPointer = nipper->pix->interface->next;
		free(nipper->pix->interface);
		nipper->pix->interface = interfacePIXPointer;
	}

	// Cleanup Static NAT / PAT
	while (nipper->pix->staticNAT != 0)
	{
		staticPIXPointer = nipper->pix->staticNAT->next;
		free(nipper->pix->staticNAT);
		nipper->pix->staticNAT = staticPIXPointer;
	}

	// Cleanup SSH
	if (nipper->pix->ssh != 0)
		free(nipper->pix->ssh);

	// Cleanup access-lists
	while (nipper->pix->access != 0)
	{
		while (nipper->pix->access->permission != 0)
		{
			permissionPIXPointer = nipper->pix->access->permission->next;
			free(nipper->pix->access->permission);
			nipper->pix->access->permission = permissionPIXPointer;
		}
		accessPIXPointer = nipper->pix->access->next;
		free(nipper->pix->access);
		nipper->pix->access = accessPIXPointer;
	}

	// Cleanup Onjects
	while (nipper->pix->objectGroup != 0)
	{
		while (nipper->pix->objectGroup->object != 0)
		{
			objectPointer = nipper->pix->objectGroup->object->next;
			free(nipper->pix->objectGroup->object);
			nipper->pix->objectGroup->object = objectPointer;
		}
		objectGroupPointer = nipper->pix->objectGroup->next;
		free(nipper->pix->objectGroup);
		nipper->pix->objectGroup = objectGroupPointer;
	}

	// Free route
	while (nipper->pix->route != 0)
	{
		routePIXPointer = nipper->pix->route->next;
		free (nipper->pix->route);
		nipper->pix->route = routePIXPointer;
	}

	// Free name list
	while (nipper->pix->nameList != 0)
	{
		namePIXPointer = nipper->pix->nameList->next;
		free (nipper->pix->nameList);
		nipper->pix->nameList = namePIXPointer;
	}

	// Free Enable Passwords
	while (nipper->pix->enable != 0)
	{
		enablePIXPointer = nipper->pix->enable->next;
		free (nipper->pix->enable);
		nipper->pix->enable = enablePIXPointer;
	}

	// Free Usernames
	while (nipper->pix->username != 0)
	{
		usernamePIXPointer = nipper->pix->username->next;
		free (nipper->pix->username);
		nipper->pix->username = usernamePIXPointer;
	}

	// Fixup
	while (nipper->pix->fixup != 0)
	{
		fixupPIXPointer = nipper->pix->fixup->next;
		free (nipper->pix->fixup);
		nipper->pix->fixup = fixupPIXPointer;
	}

	// ICMP Access
	while (nipper->pix->icmp != 0)
	{
		icmpPIXPointer = nipper->pix->icmp->next;
		free (nipper->pix->icmp);
		nipper->pix->icmp = icmpPIXPointer;
	}

	// Cleanup PIX config
	free(nipper->pix);
}

