/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by the following authors:                   *
 *                                                                         *
 *       Ian Ventura-Whiting (fizz@titania.co.uk)                          *
 *       Benjamin Allen                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


#define access_old 0
#define access_std 1
#define access_ext 2

#define access_type_network 0
#define access_type_interface 1
#define access_type_object 2
#define access_type_protocol 3
#define access_type_host 4
#define access_type_any 5

#define access_oper_eq 0
#define access_oper_neq 1
#define access_oper_lt 2
#define access_oper_gt 3
#define access_oper_range 4
#define access_oper_object 5


// Access List Permission
struct accessListPIXPermission
{
	int permission;					// perm_allow or perm_deny
	char protocol[32];
	int protocolType;				// access_type_protocol...
	char source[64];
	char sourceMask[16];
	int sourceType;					// access_type_interface...
	char sourceService[32];
	int sourceServiceOp;			// access_oper_eq...
	char dest[64];
	char destMask[16];
	int destType;					// access_type_interface...
	char destService[32];
	int destServiceOp;				// access_oper_eq...
	int inactive;					// true or false
	char timeRange[32];
	int log;						// true or false
	struct accessListPIXPermission *next;
};

// PIX access List
struct accessListPIX
{
	char name[32];
	char remark[64];
	int type;						// access_old...
	struct accessListPIXPermission *permission;
	struct accessListPIX *next;
};


// Process Access List
void processPIXAccessList(char *line, struct nipperConfig *nipper)
{
	// Variables...
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];
	struct accessListPIX *accessPIXPointer = 0;
	struct accessListPIXPermission *permissionPIXPointer = 0;
	struct objectGroupList *objectGroupPointer = 0;
	struct ciscoCommand command;
	int tempInt = 0;

	// Init
	command = splitLine(line);
	memset(tempString, 0, nipper->maxSize);

	// Access Lists standard, extended, remark, ethertype
	if (command.parts > 3)
	{

		// If old, standard or extended access list...
		if ((strcmp(command.part[2], "permit") == 0) || (strcmp(command.part[2], "deny") == 0) || (strcmp(command.part[2], "standard") == 0) || (strcmp(command.part[2], "extended") == 0))
		{
			// Debug
			if (nipper->debugMode == true)
			{
				printf("Access-List Line: %s\n", line);
			}

			// Are there any access-lists?
			if (nipper->pix->access == 0)
			{
				// Create Access-List structure
				nipper->pix->access = malloc(sizeof(struct accessListPIX));
				accessPIXPointer = nipper->pix->access;
				memset(accessPIXPointer, 0, sizeof(struct accessListPIX));
				strncpy(accessPIXPointer->name, command.part[1], sizeof(accessPIXPointer->name) - 1);
			}
			else
			{
				// Search for access-list
				accessPIXPointer = nipper->pix->access;
				while ((accessPIXPointer->next != 0) && (strcmp(accessPIXPointer->name, command.part[1]) != 0))
					accessPIXPointer = accessPIXPointer->next;

				// If access-list does not exist, create...
				if (strcmp(accessPIXPointer->name, command.part[1]) != 0)
				{
					// Create Access-List structure
					accessPIXPointer->next = malloc(sizeof(struct accessListPIX));
					accessPIXPointer = accessPIXPointer->next;
					memset(accessPIXPointer, 0, sizeof(struct accessListPIX));
					strncpy(accessPIXPointer->name, command.part[1], sizeof(accessPIXPointer->name) - 1);
				}
			}

			// Establish what type of access list it is
			if (strcmp(command.part[2], "standard") == 0)
			{
				accessPIXPointer->type = access_std;
				nipper->pix->standardAccessList++;
				tempInt = 3;
			}
			else if (strcmp(command.part[2], "extended") == 0)
			{
				accessPIXPointer->type = access_ext;
				nipper->pix->extendedAccessList++;
				tempInt = 3;
			}
			else
			{
				accessPIXPointer->type = access_old;
				nipper->pix->oldAccessList++;
				tempInt = 2;
			}

			// Does a permission already exist?
			if (accessPIXPointer->permission == 0)
			{
				// Create Permission structure
				accessPIXPointer->permission = malloc(sizeof(struct accessListPIXPermission));
				permissionPIXPointer = accessPIXPointer->permission;
			}
			else
			{
				// Search
				permissionPIXPointer = accessPIXPointer->permission;
				while (permissionPIXPointer->next != 0)
					permissionPIXPointer = permissionPIXPointer->next;

				// Create Permission structure
				permissionPIXPointer->next = malloc(sizeof(struct accessListPIXPermission));
				permissionPIXPointer = permissionPIXPointer->next;
			}

			// Init Permission struct
			memset(permissionPIXPointer, 0, sizeof(struct accessListPIXPermission));
			permissionPIXPointer->protocolType = access_type_protocol;
			strcpy(permissionPIXPointer->sourceService, "any");
			permissionPIXPointer->sourceServiceOp = access_oper_eq;
			strcpy(permissionPIXPointer->destService, "any");
			permissionPIXPointer->destServiceOp = access_oper_eq;
			permissionPIXPointer->inactive = false;

			// Permit or Deny?
			if (strcmp(command.part[tempInt], "permit") == 0)
			{
				permissionPIXPointer->log = false;
				permissionPIXPointer->permission = perm_permit;
			}
			else
			{
				permissionPIXPointer->log = true;
				permissionPIXPointer->permission = perm_deny;
			}
			tempInt++;

			// If access list is old type or extended...
			if (accessPIXPointer->type != access_std)
			{
				// Protocol
				if (strcmp(command.part[tempInt], "object-group") == 0)
				{
					permissionPIXPointer->protocolType = access_type_object;
					tempInt++;
				}
				strncpy(permissionPIXPointer->protocol, command.part[tempInt], sizeof(permissionPIXPointer->protocol) - 1);
				tempInt++;
			}

			// Source
			if (strcmp(command.part[tempInt], "interface") == 0)
			{
				permissionPIXPointer->sourceType = access_type_interface;
				tempInt++;
				strncpy(permissionPIXPointer->source, command.part[tempInt], sizeof(permissionPIXPointer->source) - 1);
			}
			else if (strcmp(command.part[tempInt], "object-group") == 0)
			{
				permissionPIXPointer->sourceType = access_type_object;
				tempInt++;
				strncpy(permissionPIXPointer->source, command.part[tempInt], sizeof(permissionPIXPointer->source) - 1);
			}
			else if (strcmp(command.part[tempInt], "any") == 0)
			{
				permissionPIXPointer->sourceType = access_type_any;
				strcpy(permissionPIXPointer->source, "any");
			}
			else if (strcmp(command.part[tempInt], "host") == 0)
			{
				permissionPIXPointer->sourceType = access_type_host;
				tempInt++;
				strncpy(permissionPIXPointer->source, command.part[tempInt], sizeof(permissionPIXPointer->source) - 1);
			}
			else
			{
				permissionPIXPointer->sourceType = access_type_network;
				strncpy(permissionPIXPointer->source, command.part[tempInt], sizeof(permissionPIXPointer->source) - 1);
				tempInt++;
				strncpy(permissionPIXPointer->sourceMask, command.part[tempInt], sizeof(permissionPIXPointer->sourceMask) - 1);
			}
			tempInt++;

			// Is there more
			if ((command.parts > tempInt) && (accessPIXPointer->type != access_std))
			{
				// Source Service
				if (strcmp(command.part[tempInt], "object-group") == 0)
				{
					// Check to make sure this is a service and not the destination
					if (nipper->pix->objectGroup != 0)
					{
						objectGroupPointer = nipper->pix->objectGroup;
						while ((objectGroupPointer->next != 0) && (strcmp(objectGroupPointer->name, command.part[tempInt + 1]) != 0))
							objectGroupPointer = objectGroupPointer->next;

						if ((strcmp(objectGroupPointer->name, command.part[tempInt + 1]) == 0) && (objectGroupPointer->type == object_type_service))
						{
							permissionPIXPointer->sourceServiceOp = access_oper_object;
							tempInt++;
							strncpy(permissionPIXPointer->sourceService, command.part[tempInt], sizeof(permissionPIXPointer->sourceService) - 1);
							tempInt++;
						}
					}
				}
				else if (strcmp(command.part[tempInt], "range") == 0)
				{
					permissionPIXPointer->sourceServiceOp = access_oper_range;
					tempInt++;
					sprintf(tempString, "%s - %s", command.part[tempInt], command.part[tempInt + 1]);
					strncpy(permissionPIXPointer->sourceService, tempString, sizeof(permissionPIXPointer->sourceService) - 1);
					tempInt = tempInt + 2;
				}
				else if (strcmp(command.part[tempInt], "eq") == 0)
				{
					permissionPIXPointer->sourceServiceOp = access_oper_eq;
					tempInt++;
					strncpy(permissionPIXPointer->sourceService, command.part[tempInt], sizeof(permissionPIXPointer->sourceService) - 1);
					tempInt++;
				}
				else if (strcmp(command.part[tempInt], "neq") == 0)
				{
					permissionPIXPointer->sourceServiceOp = access_oper_neq;
					tempInt++;
					strncpy(permissionPIXPointer->sourceService, command.part[tempInt], sizeof(permissionPIXPointer->sourceService) - 1);
					tempInt++;
				}
				else if (strcmp(command.part[tempInt], "lt") == 0)
				{
					permissionPIXPointer->sourceServiceOp = access_oper_lt;
					tempInt++;
					strncpy(permissionPIXPointer->sourceService, command.part[tempInt], sizeof(permissionPIXPointer->sourceService) - 1);
					tempInt++;
				}
				else if (strcmp(command.part[tempInt], "gt") == 0)
				{
					permissionPIXPointer->sourceServiceOp = access_oper_gt;
					tempInt++;
					strncpy(permissionPIXPointer->sourceService, command.part[tempInt], sizeof(permissionPIXPointer->sourceService) - 1);
					tempInt++;
				}

				// Destination
				if (strcmp(command.part[tempInt], "interface") == 0)
				{
					permissionPIXPointer->destType = access_type_interface;
					tempInt++;
					strncpy(permissionPIXPointer->dest, command.part[tempInt], sizeof(permissionPIXPointer->dest) - 1);
				}
				else if (strcmp(command.part[tempInt], "object-group") == 0)
				{
					permissionPIXPointer->destType = access_type_object;
					tempInt++;
					strncpy(permissionPIXPointer->dest, command.part[tempInt], sizeof(permissionPIXPointer->dest) - 1);
				}
				else if (strcmp(command.part[tempInt], "any") == 0)
				{
					permissionPIXPointer->destType = access_type_any;
					strcpy(permissionPIXPointer->dest, "any");
				}
				else if (strcmp(command.part[tempInt], "host") == 0)
				{
					permissionPIXPointer->destType = access_type_host;
					tempInt++;
					strncpy(permissionPIXPointer->dest, command.part[tempInt], sizeof(permissionPIXPointer->dest) - 1);
				}
				else
				{
					permissionPIXPointer->destType = access_type_network;
					strncpy(permissionPIXPointer->dest, command.part[tempInt], sizeof(permissionPIXPointer->dest) - 1);
					tempInt++;
					strncpy(permissionPIXPointer->destMask, command.part[tempInt], sizeof(permissionPIXPointer->destMask) - 1);
				}
				tempInt++;

				// Destination Service
				if (command.parts > tempInt)
				{
					if (strcmp(command.part[tempInt], "object-group") == 0)
					{
						permissionPIXPointer->destServiceOp = access_oper_object;
						tempInt++;
						strncpy(permissionPIXPointer->destService, command.part[tempInt], sizeof(permissionPIXPointer->destService) - 1);
						tempInt++;
					}
					else if (strcmp(command.part[tempInt], "range") == 0)
					{
						permissionPIXPointer->destServiceOp = access_oper_range;
						tempInt++;
						sprintf(tempString, "%s - %s", command.part[tempInt], command.part[tempInt + 1]);
						strncpy(permissionPIXPointer->destService, tempString, sizeof(permissionPIXPointer->destService) - 1);
						tempInt = tempInt + 2;
					}
					else if (strcmp(command.part[tempInt], "eq") == 0)
					{
						permissionPIXPointer->destServiceOp = access_oper_eq;
						tempInt++;
						strncpy(permissionPIXPointer->destService, command.part[tempInt], sizeof(permissionPIXPointer->destService) - 1);
						tempInt++;
					}
					else if (strcmp(command.part[tempInt], "neq") == 0)
					{
						permissionPIXPointer->destServiceOp = access_oper_neq;
						tempInt++;
						strncpy(permissionPIXPointer->destService, command.part[tempInt], sizeof(permissionPIXPointer->destService) - 1);
						tempInt++;
					}
					else if (strcmp(command.part[tempInt], "lt") == 0)
					{
						permissionPIXPointer->destServiceOp = access_oper_lt;
						tempInt++;
						strncpy(permissionPIXPointer->destService, command.part[tempInt], sizeof(permissionPIXPointer->destService) - 1);
						tempInt++;
					}
					else if (strcmp(command.part[tempInt], "gt") == 0)
					{
						permissionPIXPointer->destServiceOp = access_oper_gt;
						tempInt++;
						strncpy(permissionPIXPointer->destService, command.part[tempInt], sizeof(permissionPIXPointer->destService) - 1);
						tempInt++;
					}
					else if ((strcmp(command.part[tempInt], "log") != 0) && (strcmp(command.part[tempInt], "inactive") != 0) && (strcmp(command.part[tempInt], "time-range") != 0))
					{
						permissionPIXPointer->destServiceOp = access_oper_eq;
						strncpy(permissionPIXPointer->destService, command.part[tempInt], sizeof(permissionPIXPointer->destService) - 1);
						tempInt++;
					}
				}

				// Options...
				while (command.parts > tempInt)
				{
					// Inactive
					if (strcmp(command.part[tempInt], "inactive") == 0)
						permissionPIXPointer->inactive = true;

					// Active Time Range
					else if (strcmp(command.part[tempInt], "time-range") == 0)
					{
						tempInt++;
						strncpy(permissionPIXPointer->timeRange, command.part[tempInt], sizeof(permissionPIXPointer->timeRange) - 1);
					}

					// Log
					else if (strcmp(command.part[tempInt], "log") == 0)
					{
						permissionPIXPointer->log = true;
						if (command.parts > tempInt + 1)
						{
							tempInt++;
							while (((strcmp(command.part[tempInt], "inactive") != 0) || (strcmp(command.part[tempInt], "time-range") != 0)) && (command.parts > tempInt))
							{
								if (strcmp(command.part[tempInt], "disable") == 0)
									permissionPIXPointer->log = false;
								tempInt++;
							}
							tempInt--;
						}
					}

					tempInt++;
				}
			}
		}

		// Access List Remark
		else if (strcmp(command.part[2], "remark") == 0)
		{
			// Debug
			if (nipper->debugMode == true)
			{
				printf("Access-List Remark Line: %s\n", line);
			}

			// If there are no Access Lists
			if (nipper->pix->access == 0)
			{
				// Create Access-List structure
				nipper->pix->access = malloc(sizeof(struct accessListPIX));
				accessPIXPointer = nipper->pix->access;
				memset(accessPIXPointer, 0, sizeof(struct accessListPIX));
				strncpy(accessPIXPointer->name, command.part[1], sizeof(accessPIXPointer->name) - 1);
			}
			else
			{

				// Search for access-list
				accessPIXPointer = nipper->pix->access;
				while ((accessPIXPointer->next != 0) && (strcmp(accessPIXPointer->name, command.part[1]) != 0))
					accessPIXPointer = accessPIXPointer->next;

				// If access-list does not exist, create...
				if (strcmp(accessPIXPointer->name, command.part[1]) == 0)
				{
					for (tempInt = 3; tempInt < command.parts; tempInt++)
					{
						sprintf(tempString2, "%s%s ",tempString, command.part[tempInt]);
						strcpy(tempString, tempString2);
					}
					tempString[strlen(tempString) - 1] = 0;
				}
			}
			strncpy(accessPIXPointer->remark, tempString, sizeof(accessPIXPointer->remark) - 1);
		}

		// Access List Ethertype
		else if (strcmp(command.part[2], "ethertype") == 0)
		{
			// Debug
			if (nipper->debugMode == true)
			{
				printf("Access-List Ethertype Line: %s\n", line);
			}
		}
	}

	// Alert Interval
	else if (strcmp(command.part[1], "alert-interval") == 0)
	{
		// Debug
		if (nipper->debugMode == true)
		{
			printf("Access-List Alert Interval Line: %s\n", line);
		}
		nipper->pix->accessListAlertInterval = atoi(command.part[2]);
	}

	// Deny Flow Max
	else if (strcmp(command.part[1], "alert-interval") == 0)
	{
		// Debug
		if (nipper->debugMode == true)
		{
			printf("Access-List Deny-Flow-Max Line: %s\n", line);
		}
		nipper->pix->accessListDenyFlowMax = atoi(command.part[2]);
	}
}

