/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


// Process Object Groups
void processObjectGroup(char *line, struct nipperConfig *nipper)
{
	// Variables...
	int tempInt = 0;
	fpos_t filePosition;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];
	struct ciscoCommand command;
	struct objectGroupList *objectGroupPointer = 0;
	struct objectList *objectPointer = 0;

	// Debug output
	if (nipper->debugMode == true)
		printf("Object Group Line: %s\n", line);

	// if first object group...
	if (nipper->pix->objectGroup == 0)
	{
		nipper->pix->objectGroup = malloc(sizeof(struct objectGroupList));
		objectGroupPointer = nipper->pix->objectGroup;
	}
	else
	{
		objectGroupPointer = nipper->pix->objectGroup;
		while (objectGroupPointer->next != 0)
			objectGroupPointer = objectGroupPointer->next;
		objectGroupPointer->next = malloc(sizeof(struct objectGroupList));
		objectGroupPointer = objectGroupPointer->next;
	}

	// Init Struct
	command = splitLine(line);
	memset(objectGroupPointer, 0, sizeof(struct objectGroupList));

	// Object Type
	if (strcmp(command.part[1], "network") == 0)
		objectGroupPointer->type = object_type_network;
	else if (strcmp(command.part[1], "protocol") == 0)
		objectGroupPointer->type = object_type_protocol;
	else if (strcmp(command.part[1], "service") == 0)
		objectGroupPointer->type = object_type_service;
	else if (strcmp(command.part[1], "icmp-type") == 0)
		objectGroupPointer->type = object_type_icmp;

	// Object Group Name
	strncpy(objectGroupPointer->name, command.part[2], sizeof(objectGroupPointer->name) - 1);

	// Service object group type...
	if ((command.parts == 4) && (objectGroupPointer->type == object_type_service))
	{
		if (strcmp(command.part[3], "tcp") == 0)
			objectGroupPointer->serviceType = object_service_tcp;
		else if (strcmp(command.part[3], "udp") == 0)
			objectGroupPointer->serviceType = object_service_udp;
		else if (strcmp(command.part[3], "tcp-udp") == 0)
			objectGroupPointer->serviceType = object_service_both;
	}

	// Loop through the interface lines
	fgetpos(nipper->input, &filePosition);
	readLine(nipper->input, line, nipper->maxSize);
	while ((feof(nipper->input) == 0) && (line[0] == ' '))
	{
		// Debug
		if (nipper->debugMode == true)
		{
			printf("Object Group Line: %s\n", line);
		}

		// Init
		command = splitLine(line);

		// Description
		if (strcmp(command.part[0], "description") == 0)
		{
			memset(tempString, 0, nipper->maxSize);
			memset(tempString2, 0, nipper->maxSize);
			for (tempInt = 1; tempInt < command.parts; tempInt++)
			{
				sprintf(tempString, "%s%s ", tempString2, command.part[tempInt]);
				strncpy(tempString2, tempString, nipper->maxSize);
			}
			tempString2[strlen(tempString2) - 1] = 0;
			strncpy(objectGroupPointer->description, tempString2, sizeof(objectGroupPointer->description) - 1);
		}
		else
		{
			// If structure doesn't exist
			if (objectGroupPointer->object == 0)
			{
				objectGroupPointer->object = malloc(sizeof(struct objectList));
				objectPointer = objectGroupPointer->object;
			}
			else
			{
				objectPointer = objectGroupPointer->object;
				while (objectPointer->next != 0)
					objectPointer = objectPointer->next;
				objectPointer->next = malloc(sizeof(struct objectList));
				objectPointer = objectPointer->next;
			}

			// Init
			memset(objectPointer, 0, sizeof(struct objectList));

			if (strcmp(command.part[0], "network-object") == 0)
			{
				if (strcmp(command.part[1], "host") == 0)
				{
					objectPointer->type = object_type_host;
					strncpy(objectPointer->object, command.part[2], sizeof(objectPointer->object) - 1);
				}
				else
				{
					objectPointer->type = object_type_network;
					if (command.parts == 2)
						strncpy(objectPointer->object, command.part[1], sizeof(objectPointer->object) - 1);
					else
					{
						strncpy(objectPointer->object, command.part[1], sizeof(objectPointer->object) - 1);
						strncpy(objectPointer->netMask, command.part[2], sizeof(objectPointer->netMask) - 1);
					}
				}
			}
			else if (strcmp(command.part[0], "group-object") == 0)
			{
				objectPointer->type = object_type_group;
				strncpy(objectPointer->object, command.part[1], sizeof(objectPointer->object) - 1);
			}
			else if (strcmp(command.part[0], "port-object") == 0)
			{
				objectPointer->type = object_type_port;
				if (strcmp(command.part[1], "eq") == 0)
					strncpy(objectPointer->object, command.part[2], sizeof(objectPointer->object) - 1);
				else if (strcmp(command.part[1], "range") == 0)
				{
					if (command.parts == 3)
						strncpy(objectPointer->object, command.part[2], sizeof(objectPointer->object) - 1);
					else
					{
						sprintf(tempString, "%s %s", command.part[2], command.part[3]);
						strncpy(objectPointer->object, tempString, sizeof(objectPointer->object) - 1);
					}
				}
				else
				{
					if (command.parts == 2)
						strncpy(objectPointer->object, command.part[1], sizeof(objectPointer->object) - 1);
					else
					{
						sprintf(tempString, "%s %s", command.part[1], command.part[2]);
						strncpy(objectPointer->object, tempString, sizeof(objectPointer->object) - 1);
					}
				}
			}
			else if (strcmp(command.part[0], "icmp-object") == 0)
			{
				objectPointer->type = object_type_icmp;
				strncpy(objectPointer->object, command.part[1], sizeof(objectPointer->object) - 1);
			}
			else if (strcmp(command.part[0], "protocol-object") == 0)
			{
				objectPointer->type = object_type_protocol;
				strncpy(objectPointer->object, command.part[1], sizeof(objectPointer->object) - 1);
			}
		}

		// Get next line
		fgetpos(nipper->input, &filePosition);
		readLine(nipper->input, line, nipper->maxSize);
	}

	// Set file position back
	fsetpos(nipper->input, &filePosition);
}


