/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


// Old PIX ACL
// -----------
void outputPIXOldACL(struct ciscoPIXConfig *configPIX, FILE *reportFile, int reportFormat, int line, int expandACL, int maxSize, int permission, char *protocol, char *source, char *sourceMask, int sourceType, char *sourceService, int sourceOper, char *dest, char *destMask, int destType, char *destService, int destOper)
{
	// Variables
	char tempString[maxSize];
	struct objectGroupList *objectGroupPointer = 0;
	struct objectList *objectPointer = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Process Source Objects
	if ((sourceType == access_type_object) && (expandACL == true))
	{
		// Find object
		if (configPIX->objectGroup != 0)
		{
			objectGroupPointer = configPIX->objectGroup;
			while ((objectGroupPointer != 0) && (strcmp(objectGroupPointer->name, source) != 0))
				objectGroupPointer = objectGroupPointer->next;

			// If Object group has been found
			if (strcmp(objectGroupPointer->name, source) == 0)
			{
				// Loop through the object group
				objectPointer = objectGroupPointer->object;
				while (objectPointer != 0)
				{
					switch (objectPointer->type)
					{
						case object_type_network:
							outputPIXOldACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, objectPointer->object, objectPointer->netMask, access_type_network, sourceService, sourceOper, dest, destMask, destType, destService, destOper);
							break;
						case object_type_group:
							outputPIXOldACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, objectPointer->object, objectPointer->netMask, access_type_object, sourceService, sourceOper, dest, destMask, destType, destService, destOper);
							break;
						case object_type_host:
							outputPIXOldACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, objectPointer->object, objectPointer->netMask, access_type_host, sourceService, sourceOper, dest, destMask, destType, destService, destOper);
							break;
					}
					objectPointer = objectPointer->next;
				}
			}
		}
	}
	else
	{

		// Process Destination Objects
		if ((destType == access_type_object) && (expandACL == true))
		{
			// Find object
			if (configPIX->objectGroup != 0)
			{
				objectGroupPointer = configPIX->objectGroup;
				while ((objectGroupPointer != 0) && (strcmp(objectGroupPointer->name, dest) != 0))
					objectGroupPointer = objectGroupPointer->next;

				// If Object group has been found
				if (strcmp(objectGroupPointer->name, dest) == 0)
				{
					// Loop through the object group
					objectPointer = objectGroupPointer->object;
					while (objectPointer != 0)
					{
						switch (objectPointer->type)
						{
							case object_type_network:
								outputPIXOldACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, source, sourceMask, sourceType, sourceService, sourceOper, objectPointer->object, objectPointer->netMask, access_type_network, destService, destOper);
								break;
							case object_type_group:
								outputPIXOldACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, source, sourceMask, sourceType, sourceService, sourceOper, objectPointer->object, objectPointer->netMask, access_type_object, destService, destOper);
								break;
							case object_type_host:
								outputPIXOldACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, source, sourceMask, sourceType, sourceService, sourceOper, objectPointer->object, objectPointer->netMask, access_type_host, destService, destOper);
								break;
						}
						objectPointer = objectPointer->next;
					}
				}
			}
		}
		else
		{

			// Line
			fprintf(reportFile, "%s%d%s", tablerow_start, line, tablerow_mid);

			// Permission
			if (permission == perm_permit)
				fprintf(reportFile, "Permit");
			else
				fprintf(reportFile, "Deny");

			// Protocol
			fprintf(reportFile, "%s%s%s", tablerow_mid, protocol, tablerow_mid);
			addAbbreviation(protocol, false);

			// Source
			if (sourceType == access_type_network)
			{
				outputFriendly(source, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s", tempString);
				outputFriendly(sourceMask, tempString, maxSize, reportFormat);
				fprintf(reportFile, " %s%s", tempString, tablerow_mid);
			}
			else
			{
				outputFriendly(source, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			}

			// Source Service
			outputFriendly(sourceService, tempString, maxSize, reportFormat);
			addAbbreviation(sourceService, false);
			switch (sourceOper)
			{
				case access_oper_neq:
					fprintf(reportFile, "NOT %s%s", tempString, tablerow_mid);
					break;
				case access_oper_lt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&lt;");
					else
						fprintf(reportFile, "<");
					fprintf(reportFile, " %s%s", tempString, tablerow_mid);
					break;
				case access_oper_gt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&gt;");
					else
						fprintf(reportFile, ">");
					fprintf(reportFile, " %s%s", tempString, tablerow_mid);
					break;
				default	:
					fprintf(reportFile, "%s%s", tempString, tablerow_mid);
					break;
			}

			// Destination
			if (destType == access_type_network)
			{
				outputFriendly(dest, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s ", tempString);
				outputFriendly(destMask, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			}
			else
			{
				outputFriendly(dest, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			}

			// Destination Services
			outputFriendly(destService, tempString, maxSize, reportFormat);
			addAbbreviation(destService, false);
			switch (destOper)
			{
				case access_oper_neq:
					fprintf(reportFile, "NOT %s%s", tempString, tablerow_end);
					break;
				case access_oper_lt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&lt;");
					else
						fprintf(reportFile, "<");
					fprintf(reportFile, " %s%s", tempString, tablerow_end);
					break;
				case access_oper_gt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&gt;");
					else
						fprintf(reportFile, ">");
					fprintf(reportFile, " %s%s", tempString, tablerow_end);
					break;
				default:
					fprintf(reportFile, "%s%s", tempString, tablerow_end);
					break;
			}
		}
	}
}


// Extended PIX ACL
// ----------------
void outputPIXExtendedACL(struct ciscoPIXConfig *configPIX, FILE *reportFile, int reportFormat, int line, int expandACL, int maxSize, int permission, char *protocol, char *source, char *sourceMask, int sourceType, char *sourceService, int sourceOper, char *dest, char *destMask, int destType, char *destService, int destOper, char *timeRange, int log, int inactive)
{
	// Variables
	char tempString[maxSize];
	struct objectGroupList *objectGroupPointer = 0;
	struct objectList *objectPointer = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Process Source Objects
	if ((sourceType == access_type_object) && (expandACL == true))
	{
		// Find object
		if (configPIX->objectGroup != 0)
		{
			objectGroupPointer = configPIX->objectGroup;
			while ((objectGroupPointer->next != 0) && (strcmp(objectGroupPointer->name, source) != 0))
				objectGroupPointer = objectGroupPointer->next;

			// If Object group has been found
			if (strcmp(objectGroupPointer->name, source) == 0)
			{
				// Loop through the object group
				objectPointer = objectGroupPointer->object;
				while (objectPointer != 0)
				{
					switch (objectPointer->type)
					{
						case object_type_network:
							outputPIXExtendedACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, objectPointer->object, objectPointer->netMask, access_type_network, sourceService, sourceOper, dest, destMask, destType, destService, destOper, timeRange, log, inactive);
							break;
						case object_type_group:
							outputPIXExtendedACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, objectPointer->object, objectPointer->netMask, access_type_object, sourceService, sourceOper, dest, destMask, destType, destService, destOper, timeRange, log, inactive);
							break;
						case object_type_host:
							outputPIXExtendedACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, objectPointer->object, objectPointer->netMask, access_type_host, sourceService, sourceOper, dest, destMask, destType, destService, destOper, timeRange, log, inactive);
							break;
					}
					objectPointer = objectPointer->next;
				}
			}
		}
	}
	else
	{

		// Process Destination Objects
		if ((destType == access_type_object) && (expandACL == true))
		{
			// Find object
			if (configPIX->objectGroup != 0)
			{
				objectGroupPointer = configPIX->objectGroup;
				while ((objectGroupPointer->next != 0) && (strcmp(objectGroupPointer->name, dest) != 0))
					objectGroupPointer = objectGroupPointer->next;

				// If Object group has been found
				if (strcmp(objectGroupPointer->name, dest) == 0)
				{
					// Loop through the object group
					objectPointer = objectGroupPointer->object;
					while (objectPointer != 0)
					{
						switch (objectPointer->type)
						{
							case object_type_network:
								outputPIXExtendedACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, source, sourceMask, sourceType, sourceService, sourceOper, objectPointer->object, objectPointer->netMask, access_type_network, destService, destOper, timeRange, log, inactive);
								break;
							case object_type_group:
								outputPIXExtendedACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, source, sourceMask, sourceType, sourceService, sourceOper, objectPointer->object, objectPointer->netMask, access_type_object, destService, destOper, timeRange, log, inactive);
								break;
							case object_type_host:
								outputPIXExtendedACL(configPIX, reportFile, reportFormat, line, expandACL, maxSize, permission, protocol, source, sourceMask, sourceType, sourceService, sourceOper, objectPointer->object, objectPointer->netMask, access_type_host, destService, destOper, timeRange, log, inactive);
								break;
						}
						objectPointer = objectPointer->next;
					}
				}
			}
		}
		else
		{

			// ACL Line
			fprintf(reportFile, "%s%d%s", tablerow_start, line, tablerow_mid);

			// Permission
			if (permission == perm_permit)
				fprintf(reportFile, "Permit");
			else
				fprintf(reportFile, "Deny");

			// Protocol
			fprintf(reportFile, "%s%s%s", tablerow_mid, protocol, tablerow_mid);
			addAbbreviation(protocol, false);

			// Source
			outputFriendly(source, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);

			// Source Service
			outputFriendly(sourceService, tempString, maxSize, reportFormat);
			addAbbreviation(sourceService, false);
			switch (sourceOper)
			{
				case access_oper_neq:
					fprintf(reportFile, "NOT %s", tempString);
					break;

				case access_oper_lt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&lt;");
					else
						fprintf(reportFile, "<");
					fprintf(reportFile, " %s", tempString);
					break;

				case access_oper_gt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&gt;");
					else
						fprintf(reportFile, ">");
					fprintf(reportFile, " %s", tempString);
					break;

				default:
					fprintf(reportFile, "%s", tempString);
					break;
			}

			// Destination
			outputFriendly(dest, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_mid);

			// Destination Service
			outputFriendly(destService, tempString, maxSize, reportFormat);
			addAbbreviation(destService, false);
			switch (destOper)
			{
				case access_oper_neq:
					fprintf(reportFile, "NOT %s", tempString);
					break;

				case access_oper_lt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&lt;");
					else
						fprintf(reportFile, "<");
					fprintf(reportFile, " %s", tempString);
					break;

				case access_oper_gt:
					if (reportFormat == output_html)
						fprintf(reportFile, "&gt;");
					else
						fprintf(reportFile, ">");
					fprintf(reportFile, " %s", tempString);
					break;

				default:
					fprintf(reportFile, "%s", tempString);
					break;
			}

			// Time
			fprintf(reportFile, "%s%s%s", tablerow_mid, timeRange, tablerow_mid);

			// Log
			if (log == true)
				fprintf(reportFile, "Yes");
			else
				fprintf(reportFile, "No");

			// Active
			if (inactive == false)
				fprintf(reportFile, "%sYes%s", tablerow_mid, tablerow_end);
			else
				fprintf(reportFile, "%sNo%s", tablerow_mid, tablerow_end);
		}
	}
}
