/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


void generatePIXReport(struct nipperConfig *nipper)
{
	// Variables
	struct interfacePIXConfig *interfacePIXConfigPointer = 0;
	struct accessListPIX *accessPointer = 0;
	struct accessListPIXPermission *permissionPointer = 0;
	struct objectGroupList *objectGroupPointer = 0;
	struct objectList *objectPointer = 0;
	struct namePIXList *namePIXPointer = 0;
	struct enablePIXPassword *enablePIXPointer = 0;
	struct usernamePIX *usernamePIXPointer = 0;
	struct staticPIX *staticPIXPointer = 0;
	struct fixupList *fixupPIXPointer = 0;
	struct icmpAccess *icmpPIXPointer = 0;
	struct routePIX *routePIXPointer = 0;
	int tempInt = 0;
	int lineCount = 0;
	int issues = 0;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];

	// Counter Variables...
	int accessListIssues = 0;
	int denyIssue = false;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (nipper->outputFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (nipper->outputFile[0] == 0)
		nipper->reportFile = stdout;
	else
	{
		nipper->reportFile = fopen(nipper->outputFile, "w");
		if (nipper->reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, nipper->outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(nipper->hostname, nipper->friendlyHostname, nipper->maxSize, nipper->outputFormat);

	// Output file heading (if required)
	output_filestart(nipper);

	// -------------------------
	// Security Section Contents
	// -------------------------
	if (nipper->fullReport == true)
	{
		output_parseSectionIndex(report_security_title, nipper);

		// Start of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "<ul>\n");
				break;
		}
		output_parseIndex("Introduction", nipper, section_security);

		// Timeout
		if (nipper->pix->ssh != 0)
		{
			if (nipper->pix->ssh->timeout * 60 > nipper->connectionTimeout)
				nipper->insecureTimeouts++;
		}
		if (nipper->insecureTimeouts > 0)
		{
			output_parseIndex(report_timeout_title, nipper, section_security);
			issues++;
		}

		// Access Lists
		accessPointer = nipper->pix->access;
		while (accessPointer != 0)
		{
			permissionPointer = accessPointer->permission;
			while (permissionPointer != 0)
			{
				// If permit
				if (permissionPointer->permission == perm_permit)
				{
					// Standard Access List...
					if (accessPointer->type == access_std)
					{
						// From any source...
						if ((strcmp(permissionPointer->source, "any") == 0) && (nipper->strictACL == true))
							accessListIssues++;
					}
					// Extended Access List / old...
					else
					{
						if ((((strcmp(permissionPointer->source, "any") == 0) && (nipper->strictACL == true)) || (strcmp(permissionPointer->dest, "any") == 0) || (strcmp(permissionPointer->destService, "any") == 0)) && (strcmp(permissionPointer->protocol, "icmp") != 0))
							accessListIssues++;
					}
				}
				// Deny & Log (Standard ACL)...
				if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type == access_std))
				{
					if (((strcmp(permissionPointer->source, "any") != 0) && (nipper->strictACL == true)) || (permissionPointer->log == false) || (permissionPointer->permission != perm_deny))
						accessListIssues++;
				}
				// Deny & Log (Extended / Old ACL)...
				else if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type != access_std))
				{
					if (((strcmp(permissionPointer->source, "any") != 0) && (nipper->strictACL == true)) || (strcmp(permissionPointer->dest, "any") != 0) || (strcmp(permissionPointer->destService, "any") != 0) || (permissionPointer->log == false) || (permissionPointer->permission != perm_deny))
						accessListIssues++;
				}

				permissionPointer = permissionPointer->next;
			}
			accessPointer = accessPointer->next;
		}
		if (nipper->pix->access == 0)
			accessListIssues = -1;
		if (accessListIssues != 0)
		{
			output_parseIndex(report_accesslist_title, nipper, section_security);
			issues++;
		}

		// SSH Protocol Version
		if (nipper->pix->ssh != 0)
		{
			if (nipper->pix->ssh->version < 2)
			{
				output_parseIndex(report_sshproto_title, nipper, section_security);
				issues++;
			}
		}

		// End of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "</ul>\n");
				break;
			case output_text:
				fprintf(nipper->reportFile, "\n");
				break;
		}
	}

	// ------------------------------
	// Configuration Section Contents
	// ------------------------------
	output_parseSectionIndex(report_config_title, nipper);

	// Start of configuration index
	switch (nipper->outputFormat)
	{
		case output_html:
			fprintf(nipper->reportFile, "<ul>\n");
			break;
	}
	output_parseIndex("Introduction", nipper, section_config);

	output_parseIndex(settings_general_title, nipper, section_config);
	if (((nipper->pix->enable != 0) || (nipper->pix->password[0] != 0)) && (nipper->passwords == true))
		output_parseIndex(settings_users_title, nipper, section_config);
	if (nipper->pix->ssh != 0)
		output_parseIndex(settings_ssh_title, nipper, section_config);
	if (nipper->pix->interface != 0)
		output_parseIndex(settings_inter_title, nipper, section_config);
	if (nipper->pix->staticNAT !=0)
		output_parseIndex(settings_natpat_title, nipper, section_config);
	if (nipper->pix->route !=0)
		output_parseIndex(settings_routing_title, nipper, section_config);
	if (nipper->pix->access != 0)
		output_parseIndex(settings_accesslists_title, nipper, section_config);
	if (nipper->pix->icmp != 0)
		output_parseIndex(settings_icmpaccess_title, nipper, section_config);
	if (nipper->pix->fixup != 0)
		output_parseIndex(settings_protoinspect_title, nipper, section_config);
	if (nipper->pix->objectGroup != 0)
		output_parseIndex(settings_objects_title, nipper, section_config);
	if (nipper->pix->nameList != 0)
		output_parseIndex(settings_namemap_title, nipper, section_config);

	// End of index
	output_abbrevContents(nipper);
	output_endsection(nipper, subsection_end);

	// Security Report section
	if (nipper->fullReport == true)
	{
		// Output report introduction
		output_parseTitle(report_security_title, nipper, rate_none, section_main_sec);
		output_parseTitle("Introduction", nipper, rate_none, section_security);
		output_parseText(report_security_intro, nipper, section_none, rate_none, issues, 0);
		output_endsection(nipper, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		if (nipper->insecureTimeouts > 0)
		{
			output_parseTitle(report_timeout_title, nipper, rate_med, section_security);
			output_parseText(report_timeout_obs1, nipper, section_obs, rate_none, nipper->insecureTimeouts, nipper->tableNum);
			if ((nipper->connectionTimeout % 60) == 0)
			{
				numToWord(nipper->reportFile, (nipper->connectionTimeout / 60));
				fprintf(nipper->reportFile, " minutes");
			}
			else
			{
				numToWord(nipper->reportFile, nipper->connectionTimeout / 60);
				fprintf(nipper->reportFile, " minutes and ");
				numToWord(nipper->reportFile, nipper->connectionTimeout % 60);
				fprintf(nipper->reportFile, " seconds");
			}
			output_parseText(report_timeout_obs2, nipper, section_none, rate_none, nipper->insecureTimeouts, nipper->tableNum);
			if (nipper->insecureTimeouts > 1)
			{
				output_table(nipper, true, report_timeout_table, &report_timeout_heading);
				if (nipper->pix->ssh != 0)
				{
					if ((nipper->pix->ssh->timeout * 60) > nipper->connectionTimeout)
					{
						fprintf(nipper->reportFile, "%sSSH Session Timeout%s", tablerow_start, tablerow_mid);
						numToWord(nipper->reportFile, nipper->pix->ssh->timeout);
						fprintf(nipper->reportFile, " minutes%s", tablerow_end);
					}
				}
				output_table(nipper, false, report_timeout_table, &report_timeout_heading);
			}
			else
			{
				if (nipper->pix->ssh != 0)
				{
					if ((nipper->pix->ssh->timeout * 60) > nipper->connectionTimeout)
					{
						fprintf(nipper->reportFile, "The SSH session timeout was ");
						numToWord(nipper->reportFile, nipper->pix->ssh->timeout);
						fprintf(nipper->reportFile, " minutes.");
					}
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_timeout_impact, nipper, section_imp, rate_high, 0, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_timeout_ease, nipper, section_ease, rate_difficult, 0, 0);
			output_endsection(nipper, section_ease);
			output_parseText(report_timeout_rec1, nipper, section_rec, rate_none, 0, 0);
			if ((nipper->connectionTimeout % 60) == 0)
			{
				numToWord(nipper->reportFile, (nipper->connectionTimeout / 60));
				fprintf(nipper->reportFile, " minutes");
			}
			else
			{
				numToWord(nipper->reportFile, nipper->connectionTimeout / 60);
				fprintf(nipper->reportFile, " minutes and ");
				numToWord(nipper->reportFile, (nipper->connectionTimeout % 60));
				fprintf(nipper->reportFile, " seconds");
			}
			output_parseText(report_timeout_rec2, nipper, section_none, rate_none, 0, 0);
			output_endsection(nipper, section_rec);
		}

		// Access Lists
		if (accessListIssues != 0)
		{
			denyIssue = false;
			output_parseTitle(report_accesslist_title, nipper, rate_med, section_security);
			output_parseText(report_accesslist_obs1, nipper, section_obs, rate_none, accessListIssues, nipper->tableNum);
			// One Access List issue...
			if (accessListIssues == 1)
			{
				accessPointer = nipper->pix->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->type == access_std)
							{
								// From any source...
								if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue5);
							}
							// Extended Access List / old...
							else
							{
								// From any source to any destination and service...
								if ((strcmp(permissionPointer->dest, "any") == 0) && (nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue1);

								// To and destination and service...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue2);

								// From any source to any service...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue3);

								// From any source to any destination...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue4);

								// From any source...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue5);

								// To any destination...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue6);

								// To any service...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue7);
							}
						}
						// Deny & Log (Standard ACL)...
						if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type == access_std))
						{
							fprintf(nipper->reportFile, "%s %s %s", report_accesslist_std, accessPointer->name, report_accesslist_issue8);
							denyIssue = true;
						}
						// Deny & Log (Extended ACL)...
						else if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type == access_ext))
						{
							fprintf(nipper->reportFile, "%s %s %s", report_accesslist_ext, accessPointer->name, report_accesslist_issue8);
							denyIssue = true;
						}
						// Deny & Log (Old ACL)...
						else if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type == access_old))
						{
							fprintf(nipper->reportFile, "%s %s %s", report_accesslist_old, accessPointer->name, report_accesslist_issue8);
							denyIssue = true;
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
				output_newPara(nipper);
			}
			// More than one issue...
			else if (accessListIssues > 1)
			{
				output_table(nipper, true, report_accesslist_table1, &report_accesslists_heading);
				accessPointer = nipper->pix->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->type == access_std)
							{
								// From any source...
								if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue5, tablerow_end);
								}
							}
							// Extended Access List / old...
							else
							{
								// Access from any source to any destination and service
								if ((nipper->strictACL == true) && (strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue1, tablerow_end);
								}

								// Access to any destination and service
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue2, tablerow_end);
								}

								// Access from any source to any service
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue3, tablerow_end);
								}

								// Access from any source to any destination
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue4, tablerow_end);
								}

								// Access from any source
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue5, tablerow_end);
								}

								// To any destination
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue6, tablerow_end);
								}

								// To any service
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, lineCount, tablerow_mid);
									fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue7, tablerow_end);
								}
							}
						}
						// Deny & Log (Standard ACL)...
						if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type == access_std))
						{
							outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s%s %s", tablerow_start, tempString, tablerow_mid, tablerow_mid);
							fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue8, tablerow_end);
							denyIssue = true;
						}
						// Deny & Log (Extended / Old ACL)...
						else if ((nipper->denyLog == true) && (permissionPointer->next == 0) && (accessPointer->type != access_std))
						{
							outputFriendly(accessPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s%s %s", tablerow_start, tempString, tablerow_mid, tablerow_mid);
							fprintf(nipper->reportFile, "%s%s", report_accesslist_tabissue8, tablerow_end);
							denyIssue = true;
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
				output_table(nipper, false, report_accesslist_table1, &report_accesslists_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_accesslist_impact, nipper, section_imp, rate_med, 0, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_accesslist_ease, nipper, section_ease, rate_noease, 0, 0);
			output_endsection(nipper, section_ease);
			if (denyIssue == true)
				output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, 2, 0);
			else
				output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, 1, 0);
			output_endsection(nipper, section_rec);
		}

		// SSH Protocol Version
		if (nipper->pix->ssh != 0)
		{
			if (nipper->pix->ssh->version < 2)
			{
				output_parseTitle(report_sshproto_title, nipper, rate_lowmed, section_security);
				output_parseText(report_sshproto_obs, nipper, section_obs, rate_none, 0, 0);
				output_endsection(nipper, section_obs);
				output_parseText(report_sshproto_impact, nipper, section_imp, rate_med, 0, 0);
				output_endsection(nipper, section_imp);
				output_parseText(report_sshproto_ease, nipper, section_ease, rate_difficult, 0, 0);
				output_endsection(nipper, section_ease);
				output_parseText(report_sshproto_rec1, nipper, section_rec, rate_none, 0, 0);
				output_endsection(nipper, section_rec);
			}
		}

		// End Section
		output_endsection(nipper, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	output_parseTitle(report_config_title, nipper, rate_none, section_main_con);
	output_parseTitle("Introduction", nipper, rate_none, section_config);
	output_parseText(report_config_intro, nipper, section_none, rate_none, 0, 0);
	output_endsection(nipper, subsection_end);

	// General Device Settings
	output_parseTitle(settings_general_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_general_table, &settings_general_heading);
	fprintf(nipper->reportFile, "%sHostname%s%s %s", tablerow_start, tablerow_mid, nipper->friendlyHostname, tablerow_end);
	if (nipper->domainName[0] != 0)
	{
		outputFriendly(nipper->domainName, tempString, nipper->maxSize, nipper->outputFormat);
		fprintf(nipper->reportFile, "%sDomain Name%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
	}
	if ((nipper->version[0] != 0) && (nipper->deviceType == type_pix_firewall))
	{
		addAbbreviation("PIX", false);
		fprintf(nipper->reportFile, "%sPIX Version%s%s %s", tablerow_start, tablerow_mid, nipper->version, tablerow_end);
	}
	else if ((nipper->version[0] != 0) && (nipper->deviceType == type_asa_firewall))
	{
		addAbbreviation("ASA", false);
		fprintf(nipper->reportFile, "%sASA Version%s%s %s", tablerow_start, tablerow_mid, nipper->version, tablerow_end);
	}
	else if ((nipper->version[0] != 0) && (nipper->deviceType == type_fwsm_firewall))
	{
		addAbbreviation("FWSM", false);
		fprintf(nipper->reportFile, "%sFWSM Version%s%s %s", tablerow_start, tablerow_mid, nipper->version, tablerow_end);
	}
	if (nipper->pix->transparent == true)
		fprintf(nipper->reportFile, "%sTransparent Firewall%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sTransparent Firewall%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(nipper, false, settings_general_table, &settings_general_heading);
	output_endsection(nipper, subsection_end);

	// Users / Passwords / Authentication
	if ((nipper->pix->enable != 0) || (nipper->pix->password[0] != 0) || (nipper->pix->username != 0))
	{
		output_parseTitle(settings_users_title, nipper, rate_none, section_config);
		// Login Password
		if (nipper->pix->password[0] != 0)
		{
			if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "    <text>");
			fprintf(nipper->reportFile, "%s %s.", settings_logon_password, nipper->pix->password);
			if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "</text>\n");
			output_newPara(nipper);
		}

		// Enable Password
		if (nipper->pix->enable != 0)
		{
			output_table(nipper, true, settings_enable_table, &report_enablepix_heading);
			enablePIXPointer = nipper->pix->enable;
			while (enablePIXPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%d", tablerow_start, enablePIXPointer->level);
				fprintf(nipper->reportFile, "%s%s", tablerow_mid, enablePIXPointer->passwordEncrypted);
				fprintf(nipper->reportFile, "%s", tablerow_end);
				enablePIXPointer = enablePIXPointer->next;
			}
			output_table(nipper, false, settings_enable_table, &report_enablepix_heading);
		}

		// Usernames
		if (nipper->pix->username != 0)
		{
			output_table(nipper, true, settings_users_table, &report_userspix_heading);
			usernamePIXPointer = nipper->pix->username;
			while (usernamePIXPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, usernamePIXPointer->username, tablerow_mid);
				fprintf(nipper->reportFile, "%d%s", usernamePIXPointer->level, tablerow_mid);
				switch (usernamePIXPointer->encryption)
				{
					case encrypt_none:
						fprintf(nipper->reportFile, "N/A");
						break;
					case encrypt_pix:
						fprintf(nipper->reportFile, "Standard");
						break;
					case encrypt_mschap:
						fprintf(nipper->reportFile, "MS-CHAP");
						addAbbreviation("MS-CHAP", false);
						break;
					case encrypt_ntenc:
						fprintf(nipper->reportFile, "NT Encrypted");
						addAbbreviation("NT", false);
						break;
				}
				if (nipper->passwords == true)
				{
					if (usernamePIXPointer->passwordEncrypted[0] == 0)
						fprintf(nipper->reportFile, "%sNone%s", tablerow_mid, tablerow_end);
					else
						fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, usernamePIXPointer->passwordEncrypted, tablerow_end);
				}
				else
					fprintf(nipper->reportFile, "%s", tablerow_end);
				usernamePIXPointer = usernamePIXPointer->next;
			}
			output_table(nipper, false, settings_users_table, &report_userspix_heading);
		}
		output_endsection(nipper, subsection_end);
	}

	// SSH
	if (nipper->pix->ssh != 0)
	{
		output_parseTitle(settings_ssh_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_ssh_table, &settings_general_heading);
		if (nipper->pix->ssh->version == 0)
			fprintf(nipper->reportFile, "%sProtocol Version%s1 and 2%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sProtocol Version%s%d%s", tablerow_start, tablerow_mid, nipper->pix->ssh->version, tablerow_end);
		fprintf(nipper->reportFile, "%sSession Timeout%s", tablerow_start, tablerow_mid);
		numToWord(nipper->reportFile, nipper->pix->ssh->timeout);
		fprintf(nipper->reportFile, " minutes%s", tablerow_end);
		if (nipper->pix->ssh->ipAddress[0] == 0)
			fprintf(nipper->reportFile, "%sManagement Host%sAny%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sManagement Host%s%s %s%s", tablerow_start, tablerow_mid, nipper->pix->ssh->ipAddress, nipper->pix->ssh->netMask, tablerow_end);
		fprintf(nipper->reportFile, "%sManagement Interface%s%s%s", tablerow_start, tablerow_mid, nipper->pix->ssh->interface, tablerow_end);
		output_table(nipper, false, settings_ssh_table, &settings_general_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("SSH", false);
	}

	// Interfaces
	if (nipper->pix->interface !=0)
	{
		output_parseTitle(settings_inter_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_inter_table, &settings_interpix_heading);
		interfacePIXConfigPointer = nipper->pix->interface;
		while (interfacePIXConfigPointer != 0)
		{
			fprintf(nipper->reportFile, "%s%s%s", tablerow_start, interfacePIXConfigPointer->interface, tablerow_mid);
			outputFriendly(interfacePIXConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			if (interfacePIXConfigPointer->shutdown == true)
				fprintf(nipper->reportFile, "Yes");
			else
				fprintf(nipper->reportFile, "No");
			fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, interfacePIXConfigPointer->ipAddress, tablerow_mid);
			fprintf(nipper->reportFile, "%s%s", interfacePIXConfigPointer->netMask, tablerow_mid);
			fprintf(nipper->reportFile, "%d%s", interfacePIXConfigPointer->security, tablerow_mid);
			switch (interfacePIXConfigPointer->netSpeed)
			{
				case int_netspeed_auto:
					fprintf(nipper->reportFile, "Auto");
					break;
				case int_netspeed_100full:
					fprintf(nipper->reportFile, "100Mb Full Duplex");
					break;
			}
			fprintf(nipper->reportFile, "%s%d%s", tablerow_mid, interfacePIXConfigPointer->mtu, tablerow_mid);
			if (interfacePIXConfigPointer->in[0] == 0)
				fprintf(nipper->reportFile, " %s", tablerow_mid);
			else
			{
				outputFriendly(interfacePIXConfigPointer->in, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			}
			if (interfacePIXConfigPointer->out[0] == 0)
				fprintf(nipper->reportFile, " ");
			else
			{
				outputFriendly(interfacePIXConfigPointer->out, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s", tempString);
			}
			fprintf(nipper->reportFile, "%s", tablerow_end);
			interfacePIXConfigPointer = interfacePIXConfigPointer->next;
		}
		output_table(nipper, false, settings_inter_table, &settings_interpix_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("ACL", false);
		addAbbreviation("MTU", false);
		addAbbreviation("IP", false);
	}

	// NAT / PAT
	if (nipper->pix->staticNAT !=0)
	{
		output_parseTitle(settings_natpat_title, nipper, rate_none, section_config);
		output_parseText(settings_natpat_desc, nipper, section_none, rate_none, 1, 0);
		output_table(nipper, true, settings_nat_static_table, &settings_nat_static_heading);
		staticPIXPointer = nipper->pix->staticNAT;
		while (staticPIXPointer != 0)
		{
			outputFriendly(staticPIXPointer->interfaceReal, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
			if (staticPIXPointer->interface == true)
				fprintf(nipper->reportFile, "Interface");
			else if (staticPIXPointer->accessList[0] != 0)
			{
				outputFriendly(staticPIXPointer->accessList, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "ACL: %s", tempString);
				addAbbreviation("ACL", false);
			}
			else if (staticPIXPointer->ipNetMaskReal[0] != 0)
				fprintf(nipper->reportFile, "%s %s", staticPIXPointer->ipReal, staticPIXPointer->ipNetMaskReal);
			else
				fprintf(nipper->reportFile, "%s", staticPIXPointer->ipReal);
			outputFriendly(staticPIXPointer->interfaceMapped, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_mid);
			outputFriendly(staticPIXPointer->ipMapped, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			if (staticPIXPointer->dns == true)
				fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "No%s", tablerow_mid);
			if (staticPIXPointer->randomseq == true)
				fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "No%s", tablerow_mid);
			if (staticPIXPointer->nailed == true)
				fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "No%s", tablerow_mid);
			if (staticPIXPointer->tcpMaxConnections == 0)
				fprintf(nipper->reportFile, "Unlimited%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d%s", staticPIXPointer->tcpMaxConnections, tablerow_mid);
			if (staticPIXPointer->embrionicConnections == 0)
				fprintf(nipper->reportFile, "Unlimited%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d%s", staticPIXPointer->embrionicConnections, tablerow_mid);
			if (staticPIXPointer->udpMaxConnections == 0)
				fprintf(nipper->reportFile, "Unlimited%s", tablerow_end);
			else
				fprintf(nipper->reportFile, "%d%s", staticPIXPointer->udpMaxConnections, tablerow_end);

			staticPIXPointer = staticPIXPointer->next;
		}
		output_table(nipper, false, settings_nat_static_table, &settings_nat_static_heading);
		output_endsection(nipper, subsection_end);
	}

	// Routing
	if (nipper->pix->route != 0)
	{
		output_parseTitle(settings_routing_title, nipper, rate_none, section_config);
		output_parseText(settings_routing_desc, nipper, section_none, rate_none, 0, 0);
		output_table(nipper, true, settings_static_route_table, &settings_route_heading);
		routePIXPointer = nipper->pix->route;
		while (routePIXPointer != 0)
		{
			outputFriendly(routePIXPointer->interface, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile,"%s%s%s", tablerow_start, tempString, tablerow_mid);
			outputFriendly(routePIXPointer->ipAddress, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			outputFriendly(routePIXPointer->netMask, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			outputFriendly(routePIXPointer->gateway, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			fprintf(nipper->reportFile, "%d%s", routePIXPointer->metric, tablerow_mid);
			if (routePIXPointer->track == 0)
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d%s", routePIXPointer->track, tablerow_mid);
			if (routePIXPointer->tunneled == true)
				fprintf(nipper->reportFile, "Yes%s", tablerow_end);
			else
				fprintf(nipper->reportFile, "No%s", tablerow_end);
			routePIXPointer = routePIXPointer->next;
		}
		output_table(nipper, false, settings_static_route_table, &settings_route_heading);
		output_endsection(nipper, subsection_end);
	}

	// Access Lists
	if (nipper->pix->access != 0)
	{
		output_parseTitle(settings_accesslists_title, nipper, rate_none, section_config);
		// Intro text
		output_parseText(settings_accesslists1, nipper, section_none, rate_none, 0, 0);
		if (nipper->outputFormat == output_xml)
			fprintf(nipper->reportFile, "</text>\n");
		output_newPara(nipper);
		output_parseText(settings_accesslists3, nipper, section_none, rate_none, 0, 0);
		// General ACL Settings
		output_table(nipper, true, settings_accesslist_table, &settings_general_heading);
		fprintf(nipper->reportFile, "%sAlert Interval%s%d seconds%s", tablerow_start, tablerow_mid, nipper->pix->accessListAlertInterval, tablerow_end);
		fprintf(nipper->reportFile, "%sMaximum Deny Flows%s%d%s", tablerow_start, tablerow_mid, nipper->pix->accessListDenyFlowMax, tablerow_end);
		output_table(nipper, false, settings_accesslist_table, &settings_general_heading);
		// Old Access Lists
		if (nipper->pix->oldAccessList > 0)
		{
			accessPointer = nipper->pix->access;
			while (accessPointer != 0)
			{
				if (accessPointer->type == access_old)
				{
					if (accessPointer->remark[0] == 0)
						sprintf(tempString, "%s %s", settings_accesslists_table, accessPointer->name);
					else
						sprintf(tempString, "%s %s (%s)", settings_accesslists_table, accessPointer->name, accessPointer->remark);
					outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
					output_table(nipper, true, tempString2, &settings_accesspix_heading);
					permissionPointer = accessPointer->permission;
					tempInt = 1;
					while (permissionPointer != 0)
					{
						outputPIXOldACL(nipper->pix, nipper->reportFile, nipper->outputFormat, tempInt, nipper->expandACL, nipper->maxSize, permissionPointer->permission, permissionPointer->protocol, permissionPointer->source, permissionPointer->sourceMask, permissionPointer->sourceType, permissionPointer->sourceService, permissionPointer->sourceServiceOp, permissionPointer->dest, permissionPointer->destMask, permissionPointer->destType, permissionPointer->destService, permissionPointer->destServiceOp);
						permissionPointer = permissionPointer->next;
						tempInt++;
					}
					output_table(nipper, false, tempString2, &settings_accesspix_heading);
				}
				accessPointer = accessPointer->next;
			}
		}
		// Standard Access Lists
		if (nipper->pix->standardAccessList > 0)
		{
			accessPointer = nipper->pix->access;
			while (accessPointer != 0)
			{
				if (accessPointer->type == access_std)
				{
					if (accessPointer->remark[0] == 0)
						sprintf(tempString, "%s %s", settings_accessstd_table, accessPointer->name);
					else
						sprintf(tempString, "%s %s (%s)", settings_accessstd_table, accessPointer->name, accessPointer->remark);
					outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
					output_table(nipper, true, tempString2, &settings_accessstdpix_heading);
					permissionPointer = accessPointer->permission;
					tempInt = 1;
					while (permissionPointer != 0)
					{
						fprintf(nipper->reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
						tempInt++;
						if (permissionPointer->permission == perm_permit)
							fprintf(nipper->reportFile, "Permit%s", tablerow_mid);
						else
							fprintf(nipper->reportFile, "Deny%s", tablerow_mid);
						outputFriendly(permissionPointer->source, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
						permissionPointer = permissionPointer->next;
					}
					output_table(nipper, false, tempString2, &settings_accessstdpix_heading);
				}
				accessPointer = accessPointer->next;
			}
		}
		// Extended Access Lists
		if (nipper->pix->extendedAccessList > 0)
		{
			accessPointer = nipper->pix->access;
			while (accessPointer != 0)
			{
				if (accessPointer->type == access_ext)
				{
					if (accessPointer->remark[0] == 0)
						sprintf(tempString, "%s %s", settings_accessext_table, accessPointer->name);
					else
						sprintf(tempString, "%s %s (%s)", settings_accessext_table, accessPointer->name, accessPointer->remark);
					outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
					output_table(nipper, true, tempString2, &settings_accessextpix_heading);
					permissionPointer = accessPointer->permission;
					tempInt = 1;
					while (permissionPointer != 0)
					{
						outputPIXExtendedACL(nipper->pix, nipper->reportFile, nipper->outputFormat, tempInt, nipper->expandACL, nipper->maxSize, permissionPointer->permission, permissionPointer->protocol, permissionPointer->source, permissionPointer->sourceMask, permissionPointer->sourceType, permissionPointer->sourceService, permissionPointer->sourceServiceOp, permissionPointer->dest, permissionPointer->destMask, permissionPointer->destType, permissionPointer->destService, permissionPointer->destServiceOp, permissionPointer->timeRange, permissionPointer->log, permissionPointer->inactive);
						permissionPointer = permissionPointer->next;
						tempInt++;
					}
					output_table(nipper, false, tempString2, &settings_accessextpix_heading);
				}
				accessPointer = accessPointer->next;
			}
		}
		output_endsection(nipper, subsection_end);
	}

	// ICMP Access
	if (nipper->pix->icmp != 0)
	{
		output_parseTitle(settings_icmpaccess_title, nipper, rate_none, section_config);
		// Intro
		output_parseText(settings_icmpaccess_desc, nipper, section_none, rate_none, 0, 0);
		output_table(nipper, true, settings_icmpaccess_table, &settings_icmpaccess_heading);
		icmpPIXPointer = nipper->pix->icmp;
		while (icmpPIXPointer != 0)
		{
			fprintf(nipper->reportFile, "%s%s%s", tablerow_start, icmpPIXPointer->source, tablerow_mid);
			outputFriendly(icmpPIXPointer->interface, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s", tempString);
			if (icmpPIXPointer->icmpType[0] != 0)
				fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, icmpPIXPointer->icmpType, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sany%s", tablerow_mid, tablerow_end);
			icmpPIXPointer = icmpPIXPointer->next;
		}
		output_table(nipper, false, settings_icmpaccess_table, &settings_icmpaccess_heading);
		output_endsection(nipper, subsection_end);
	}

	// Protocol Analysis
	if (nipper->pix->fixup != 0)
	{
		output_parseTitle(settings_protoinspect_title, nipper, rate_none, section_config);
		// Intro
		output_parseText(settings_protoinspect_desc, nipper, section_none, rate_none, 0, 0);
		if (nipper->pix->fixup != 0)
		{
			output_table(nipper, true, settings_protoinspect_table, &settings_protoinspect_heading);
			fixupPIXPointer = nipper->pix->fixup;
			while (fixupPIXPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, fixupPIXPointer->protocol, tablerow_mid);
				addAbbreviation(fixupPIXPointer->protocol, false);
				if (fixupPIXPointer->enabled == true)
					fprintf(nipper->reportFile, "Yes");
				else
					fprintf(nipper->reportFile, "No");
				fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, fixupPIXPointer->options, tablerow_end);
				fixupPIXPointer = fixupPIXPointer->next;
			}
			output_table(nipper, false, settings_protoinspect_table, &settings_protoinspect_heading);
		}
		output_endsection(nipper, subsection_end);
	}

	// Group Objects
	if (nipper->pix->objectGroup != 0)
	{
		output_parseTitle(settings_objects_title, nipper, rate_none, section_config);
		// Intro
		output_parseText(settings_objects_desc, nipper, section_none, rate_none, 0, 0);
		output_list(nipper, &settings_objects_heading);
		objectGroupPointer = nipper->pix->objectGroup;
		while (objectGroupPointer != 0)
		{
			if (objectGroupPointer->type == object_type_network)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "Network %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "Network %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			else if (objectGroupPointer->type == object_type_protocol)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "Protocol %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "Protocol %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			else if (objectGroupPointer->type == object_type_service)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "Service %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "Service %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			else if (objectGroupPointer->type == object_type_icmp)
			{
				if (objectGroupPointer->description[0] == 0)
					sprintf(tempString, "ICMP %s %s", settings_objects_table, objectGroupPointer->name);
				else
					sprintf(tempString, "ICMP %s %s (%s)", settings_objects_table, objectGroupPointer->name, objectGroupPointer->description);
			}
			outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
			output_table(nipper, true, tempString2, &settings_object_heading);
			objectPointer = objectGroupPointer->object;
			while (objectPointer != 0)
			{
				switch (objectPointer->type)
				{
					case object_type_group:
						outputFriendly(objectPointer->object, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%sObject Group%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
						break;
					case object_type_port:
						fprintf(nipper->reportFile, "%sPort%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						addAbbreviation(objectPointer->object, false);
						break;
					case object_type_service:
						fprintf(nipper->reportFile, "%sService%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						addAbbreviation(objectPointer->object, false);
						break;
					case object_type_network:
						fprintf(nipper->reportFile, "%sNetwork%s%s %s%s", tablerow_start, tablerow_mid, objectPointer->object, objectPointer->netMask, tablerow_end);
						break;
					case object_type_host:
						fprintf(nipper->reportFile, "%sHost%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						break;
					case object_type_protocol:
						fprintf(nipper->reportFile, "%sProtocol%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						break;
					case object_type_icmp:
						fprintf(nipper->reportFile, "%sICMP%s%s%s", tablerow_start, tablerow_mid, objectPointer->object, tablerow_end);
						addAbbreviation("ICMP", false);
						break;
				}
				objectPointer = objectPointer->next;
			}
			output_table(nipper, false, tempString2, &settings_object_heading);
			objectGroupPointer = objectGroupPointer->next;
		}
		output_endsection(nipper, subsection_end);
	}

	// IP Address - Name Mappings
	if (nipper->pix->nameList != 0)
	{
		output_parseTitle(settings_namemap_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_namemap_table, &settings_namemap_heading);
		namePIXPointer = nipper->pix->nameList;
		while (namePIXPointer != 0)
		{
			fprintf(nipper->reportFile, "%s%s%s%s%s", tablerow_start, namePIXPointer->address, tablerow_mid, namePIXPointer->name, tablerow_end);
			namePIXPointer = namePIXPointer->next;
		}
		output_table(nipper, false, settings_namemap_table, &settings_namemap_heading);
		output_endsection(nipper, subsection_end);
	}

	// End of Configuration Section
	output_endsection(nipper, section_end);

	// Output end of file stuff
	output_fileend(nipper);
}
