/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

void generateSOSReport(struct nipperConfig *nipper)
{
	// Variables
	int tempInt = 0;
	int sourceInt = 0;
	int destInt = 0;
	int serviceInt = 0;
	int issues = 0;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];
	struct adminUsersSOS *userSOSPointer = 0;
	struct authServerSOS *authServerPointer = 0;
	struct interfaceSOS *interfaceSOSPointer = 0;
	struct policySOS *policySOSPointer = 0;
	struct zonePolicyRuleSOS *ruleSOSPointer = 0;
	struct listSOS *listSOSSourcePointer = 0;
	struct listSOS *listSOSDestPointer = 0;
	struct listSOS *listSOSServicePointer = 0;
	struct addressSOS *addressSOSPointer = 0;
	struct mappingSOS *mappingSOSPointer = 0;

	// Issue Counters
	int denyIssue = false;
	int rejectIssue = false;
	int policyIssues = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (nipper->outputFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (nipper->outputFile[0] == 0)
		nipper->reportFile = stdout;
	else
	{
		nipper->reportFile = fopen(nipper->outputFile, "w");
		if (nipper->reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, nipper->outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(nipper->hostname, nipper->friendlyHostname, nipper->maxSize, nipper->outputFormat);

	// Output file heading (if required)
	output_filestart(nipper);

	// -------------------------
	// Security Section Contents
	// -------------------------
	if (nipper->fullReport == true)
	{
		output_parseSectionIndex(report_security_title, nipper);

		// Start of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "<ul>\n");
				break;
		}
		output_parseIndex("Introduction", nipper, section_security);

		// Policy Issues
		policySOSPointer = nipper->sos->policies;
		while (policySOSPointer != 0)
		{
			ruleSOSPointer = policySOSPointer->policyRules;
			while (ruleSOSPointer != 0)
			{
				if ((ruleSOSPointer->permit == sos_permit_reject) && (ruleSOSPointer->disabled == false))
				{
					rejectIssue = true;
					policyIssues++;
				}
				else if ((ruleSOSPointer->permit == sos_permit_permit) && (ruleSOSPointer->disabled == false))
				{
					tempInt = 0;
					if (nipper->strictACL == true)
					{
						listSOSSourcePointer = ruleSOSPointer->source;
						while (listSOSSourcePointer != 0)
						{
							if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
								tempInt++;
							listSOSSourcePointer = listSOSSourcePointer->next;
						}
					}
					listSOSSourcePointer = ruleSOSPointer->dest;
					while (listSOSSourcePointer != 0)
					{
						if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
							tempInt++;
						listSOSSourcePointer = listSOSSourcePointer->next;
					}
					listSOSSourcePointer = ruleSOSPointer->service;
					while (listSOSSourcePointer != 0)
					{
						if (strcasecmp(listSOSSourcePointer->name, "ANY") == 0)
							tempInt++;
						listSOSSourcePointer = listSOSSourcePointer->next;
					}
					if (tempInt > 0)
						policyIssues++;
				}
				if ((ruleSOSPointer->next == 0) && (nipper->denyLog == true))
				{
					if ((ruleSOSPointer->permit != sos_permit_deny) || (ruleSOSPointer->disabled == true) || (ruleSOSPointer->log == false) || (strcasecmp(ruleSOSPointer->source->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->dest->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->service->name, "ANY") != 0))
					{
						tempInt++;
						denyIssue = true;
					}
				}
				ruleSOSPointer = ruleSOSPointer->next;
			}
			policySOSPointer = policySOSPointer->next;
		}
		if (policyIssues > 0)
		{
			output_parseIndex(report_policy_title, nipper, section_security);
			issues++;
		}

		// Timeout
		if (nipper->sos->admin != 0)
		{
			if ((nipper->sos->admin->authTimeout * 60) > nipper->connectionTimeout)
				nipper->insecureTimeouts++;
			authServerPointer = nipper->sos->authServer;
			while (authServerPointer != 0)
			{
				if ((authServerPointer->timeout * 60 ) > nipper->connectionTimeout)
					nipper->insecureTimeouts++;
				authServerPointer = authServerPointer->next;
			}
		}
		if (nipper->insecureTimeouts > 0)
		{
			output_parseIndex(report_timeout_title, nipper, section_security);
			issues++;
		}

		// Administrative HTTP Redirect
		if (nipper->sos->admin != 0)
		{
			if (nipper->sos->admin->httpRedirect != true)
			{
				output_parseIndex(report_httpred_title, nipper, section_security);
				issues++;
			}
		}

		// Management IP
		if (nipper->sos->admin != 0)
		{
			if (strcmp(nipper->sos->admin->adminIP, "0.0.0.0") == 0)
			{
				output_parseIndex(report_mngrip_title, nipper, section_security);
				issues++;
			}
		}

		// End of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "</ul>\n");
				break;
			case output_text:
				fprintf(nipper->reportFile, "\n");
				break;
		}
	}

	// ------------------------------
	// Configuration Section Contents
	// ------------------------------
	output_parseSectionIndex(report_config_title, nipper);

	// Start of configuration index
	switch (nipper->outputFormat)
	{
		case output_html:
			fprintf(nipper->reportFile, "<ul>\n");
			break;
	}
	output_parseIndex("Introduction", nipper, section_config);

	output_parseIndex(settings_general_title, nipper, section_config);
	if (nipper->sos->admin != 0)
		output_parseIndex(settings_admin_title, nipper, section_config);
	output_parseIndex(settings_services_title, nipper, section_config);
	if (nipper->sos->authServer != 0)
		output_parseIndex(settings_authservers_title, nipper, section_config);
	if (nipper->sos->sshEnabled == true)
		output_parseIndex(settings_ssh_title, nipper, section_config);
	if (nipper->sos->interface !=0)
		output_parseIndex(settings_inter_title, nipper, section_config);
	if (nipper->sos->policies != 0)
		output_parseIndex(settings_policies_title, nipper, section_config);
	if (nipper->sos->nameList != 0)
		output_parseIndex(settings_namemap_title, nipper, section_config);

	// End of index
	output_abbrevContents(nipper);
	output_endsection(nipper, subsection_end);

	// Security Report section
	if (nipper->fullReport == true)
	{
		// Introduction
		output_parseTitle(report_security_title, nipper, rate_none, section_main_sec);
		output_parseTitle("Introduction", nipper, rate_none, section_security);
		output_parseText(report_security_intro, nipper, section_none, rate_none, issues, 0);
		output_endsection(nipper, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		// Policy Lists
		if (policyIssues > 0)
		{
			output_parseTitle(report_policy_title, nipper, rate_med, section_security);
			output_parseText(report_policy_obs1, nipper, section_obs, rate_none, policyIssues, nipper->tableNum);
			if (policyIssues > 1)
			{
				output_table(nipper, true, report_policy_table, &report_policy_heading);
			}
			policySOSPointer = nipper->sos->policies;
			while (policySOSPointer != 0)
			{
				ruleSOSPointer = policySOSPointer->policyRules;
				while (ruleSOSPointer != 0)
				{
					// Rejects Access...
					if ((ruleSOSPointer->permit == sos_permit_reject) && (ruleSOSPointer->disabled == false))
					{
						if (policyIssues > 1)
						{
							fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
							if (policySOSPointer->global == true)
								fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
							else
							{
								outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
								fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
								outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
								fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
							}
							fprintf(nipper->reportFile, "%s%s", report_policy_table8, tablerow_end);
						}
						else
						{
							if (policySOSPointer->global == true)
								sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs9);
							else
								sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs9);
						}
					}
					// ACL issues...
					else if ((ruleSOSPointer->permit == sos_permit_permit) && (ruleSOSPointer->disabled == false))
					{
						// Identify if there is an issue
						destInt = 0;
						sourceInt = 0;
						serviceInt = 0;
						if (nipper->strictACL == true)
						{
							listSOSSourcePointer = ruleSOSPointer->source;
							while (listSOSSourcePointer != 0)
							{
								if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
									sourceInt++;
								listSOSSourcePointer = listSOSSourcePointer->next;
							}
						}
						listSOSSourcePointer = ruleSOSPointer->dest;
						while (listSOSSourcePointer != 0)
						{
							if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
								destInt++;
							listSOSSourcePointer = listSOSSourcePointer->next;
						}
						listSOSSourcePointer = ruleSOSPointer->service;
						while (listSOSSourcePointer != 0)
						{
							if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
								serviceInt++;
							listSOSSourcePointer = listSOSSourcePointer->next;
						}

						// Output issue
						if ((sourceInt > 0) && (destInt > 0) && (serviceInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table4, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs5);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs5);
							}
						}
						else if ((sourceInt > 0) && (destInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table2, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs3);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs3);
							}
						}
						else if ((sourceInt > 0) && (serviceInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table3, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs4);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs4);
							}
						}
						else if (sourceInt > 0)
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table1, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs2);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs2);
							}
						}
						else if ((destInt > 0) && (serviceInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table6, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs7);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs7);
							}
						}
						else if (destInt > 0)
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table5, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs6);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs6);
							}
						}
						else if (serviceInt > 0)
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table7, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs8);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs8);
							}
						}
						if ((policyIssues == 1) && ((sourceInt > 0) || (destInt > 0) || (serviceInt > 0)))
						{
							outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s", tempString2);
							output_newPara(nipper);
						}
					}
					// Deny all and log...
					if ((ruleSOSPointer->next == 0) && (nipper->denyLog == true))
					{
						if ((ruleSOSPointer->permit != sos_permit_deny) || (ruleSOSPointer->disabled == true) || (ruleSOSPointer->log == false) || (strcasecmp(ruleSOSPointer->source->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->dest->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->service->name, "ANY") != 0))
						{
							if (policyIssues > 1)
							{
								fprintf(nipper->reportFile, "%s %s", tablerow_start, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(nipper->reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(nipper->reportFile, "%s%s", report_policy_table9, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "The Global policy list %s", report_policy_obs10);
								else
									sprintf(tempString, "The zone %s to zone %s policy list %s", policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs10);
							}
						}
					}
					ruleSOSPointer = ruleSOSPointer->next;
				}
				policySOSPointer = policySOSPointer->next;
			}
			if (policyIssues > 1)
			{
				output_table(nipper, false, report_policy_table, &report_policy_heading);
			}
			output_endsection(nipper, section_obs);
			if (rejectIssue == true)
				output_parseText(report_policy_impact, nipper, section_imp, rate_med, 2, 0);
			else
				output_parseText(report_policy_impact, nipper, section_imp, rate_med, 1, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_policy_ease, nipper, section_ease, rate_noease, 0, 0);
			output_endsection(nipper, section_ease);
			if (denyIssue == true)
				output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, 2, 0);
			else
				output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, 1, 0);
		}

		// Timeout
		if (nipper->insecureTimeouts > 0)
		{
			output_parseTitle(report_timeout_title, nipper, rate_med, section_security);
			output_parseText(report_timeout_obs1, nipper, section_obs, rate_none, nipper->insecureTimeouts, nipper->tableNum);
			if ((nipper->connectionTimeout % 60) == 0)
			{
				numToWord(nipper->reportFile, (nipper->connectionTimeout / 60));
				fprintf(nipper->reportFile, " minutes");
			}
			else
			{
				numToWord(nipper->reportFile, nipper->connectionTimeout / 60);
				fprintf(nipper->reportFile, " minutes and ");
				numToWord(nipper->reportFile, nipper->connectionTimeout % 60);
				fprintf(nipper->reportFile, " seconds");
			}
			output_parseText(report_timeout_obs2, nipper, section_none, rate_none, nipper->insecureTimeouts, nipper->tableNum);
			if (nipper->insecureTimeouts > 1)
			{
				output_table(nipper, true, report_timeout_table, &report_timeout_heading);
				if (nipper->sos->admin != 0)
				{
					if ((nipper->sos->admin->authTimeout * 60) > nipper->connectionTimeout)
					{
						fprintf(nipper->reportFile, "%sWeb Administration%s", tablerow_start, tablerow_mid);
						numToWord(nipper->reportFile, nipper->sos->admin->authTimeout);
						fprintf(nipper->reportFile, " minutes%s", tablerow_end);
					}
				}
				authServerPointer = nipper->sos->authServer;
				while (authServerPointer != 0)
				{
					if ((authServerPointer->timeout * 60 ) > nipper->connectionTimeout)
					{
						switch (authServerPointer->type)
						{
							case authtype_builtin:
								fprintf(nipper->reportFile, "%sBuiltin Authentication%s", tablerow_start, tablerow_mid);
								break;
							case authtype_radius:
								fprintf(nipper->reportFile, "%sRADIUS Authentication%s", tablerow_start, tablerow_mid);
								break;
							case authtype_ldap:
								fprintf(nipper->reportFile, "%sLDAP Authentication%s", tablerow_start, tablerow_mid);
								break;
							case authtype_securid:
								fprintf(nipper->reportFile, "%sSecurId Authentication%s", tablerow_start, tablerow_mid);
								break;
						}
						numToWord(nipper->reportFile, authServerPointer->timeout);
						fprintf(nipper->reportFile, " minutes%s", tablerow_end);
					}
					authServerPointer = authServerPointer->next;
				}
				output_table(nipper, false, report_timeout_table, &report_timeout_heading);
			}
			else
			{
				if (nipper->sos->admin != 0)
				{
					if ((nipper->sos->admin->authTimeout * 60) > nipper->connectionTimeout)
					{
						fprintf(nipper->reportFile, "Web Administration had a timeout of ");
						numToWord(nipper->reportFile, nipper->sos->admin->authTimeout);
						fprintf(nipper->reportFile, " minutes.");
					}
				}
				authServerPointer = nipper->sos->authServer;
				while (authServerPointer != 0)
				{
					if ((authServerPointer->timeout * 60 ) > nipper->connectionTimeout)
					{
						switch (authServerPointer->type)
						{
							case authtype_builtin:
								fprintf(nipper->reportFile, "Builtin Authentication had a timeout of ");
								break;
							case authtype_radius:
								fprintf(nipper->reportFile, "RADIUS Authentication had a timeout of ");
								break;
							case authtype_ldap:
								fprintf(nipper->reportFile, "LDAP Authentication had a timeout of ");
								break;
							case authtype_securid:
								fprintf(nipper->reportFile, "SecurId Authentication had a timeout of ");
								break;
						}
						numToWord(nipper->reportFile, authServerPointer->timeout);
						fprintf(nipper->reportFile, " minutes.");
					}
					authServerPointer = authServerPointer->next;
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_timeout_impact, nipper, section_imp, rate_high, 0, 0);
			output_endsection(nipper, section_imp);
			output_parseText(report_timeout_ease, nipper, section_ease, rate_difficult, 0, 0);
			output_endsection(nipper, section_ease);
			output_parseText(report_timeout_rec1, nipper, section_rec, rate_none, 0, 0);
			if ((nipper->connectionTimeout % 60) == 0)
			{
				numToWord(nipper->reportFile, (nipper->connectionTimeout / 60));
				fprintf(nipper->reportFile, " minutes");
			}
			else
			{
				numToWord(nipper->reportFile, nipper->connectionTimeout / 60);
				fprintf(nipper->reportFile, " minutes and ");
				numToWord(nipper->reportFile, (nipper->connectionTimeout % 60));
				fprintf(nipper->reportFile, " seconds");
			}
			output_parseText(report_timeout_rec2, nipper, section_none, rate_none, 0, 0);
			output_endsection(nipper, section_rec);
		}

		// Administrative HTTP Redirect
		if (nipper->sos->admin != 0)
		{
			if (nipper->sos->admin->httpRedirect != true)
			{
				output_parseTitle(report_httpred_title, nipper, rate_med, section_security);
				output_parseText(report_httpred_obs, nipper, section_obs, rate_none, 0, 0);
				output_endsection(nipper, section_obs);
				output_parseText(report_httpred_impact, nipper, section_imp, rate_med, 0, 0);
				output_endsection(nipper, section_imp);
				output_parseText(report_httpred_ease, nipper, section_ease, rate_trivial, 0, 0);
				output_endsection(nipper, section_ease);
				output_parseText(report_httpred_rec, nipper, section_rec, rate_none, 0, 0);
				output_endsection(nipper, section_rec);
			}
		}

		// Management IP
		if (nipper->sos->admin != 0)
		{
			if (strcmp(nipper->sos->admin->adminIP, "0.0.0.0") == 0)
			{
				output_parseTitle(report_mngrip_title, nipper, rate_med, section_security);
				output_parseText(report_mngrip_obs, nipper, section_obs, rate_none, 0, 0);
				output_endsection(nipper, section_obs);
				output_parseText(report_mngrip_impact, nipper, section_imp, rate_med, 0, 0);
				output_endsection(nipper, section_imp);
				output_parseText(report_mngrip_ease, nipper, section_ease, rate_noease, 0, 0);
				output_endsection(nipper, section_ease);
				output_parseText(report_mngrip_rec, nipper, section_rec, rate_none, 0, 0);
				output_endsection(nipper, section_rec);
			}
		}

		// End of security issues
		output_endsection(nipper, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	output_parseTitle(report_config_title, nipper, rate_none, section_main_con);
	output_parseTitle("Introduction", nipper, rate_none, section_config);
	output_parseText(report_config_intro, nipper, section_none, rate_none, 0, 0);
	output_endsection(nipper, subsection_end);

	// General Settings
	output_parseTitle(settings_general_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_general_table, &settings_general_heading);
	fprintf(nipper->reportFile, "%sHostname%s%s%s", tablerow_start, tablerow_mid, nipper->friendlyHostname, tablerow_end);
	output_table(nipper, false, settings_general_table, &settings_general_heading);

	// Administrative Settings
	if (nipper->sos->admin != 0)
	{
		output_parseTitle(settings_admin_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_admin_table, &settings_general_heading);
		if (nipper->sos->admin->name[0] != 0)
		{
			outputFriendly(nipper->sos->admin->name, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sAdministrative User%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		if ((nipper->sos->admin->password[0] != 0) && (nipper->passwords == true))
		{
			outputFriendly(nipper->sos->admin->password, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sAdministrative Password%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		if (nipper->sos->admin->restrictLength == 0)
			fprintf(nipper->reportFile, "%sAdmin Password Length%sAny Length (upto 31 characters)%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sAdmin Password Length%s%d%s", tablerow_start, tablerow_mid, nipper->sos->admin->restrictLength, tablerow_end);
		fprintf(nipper->reportFile, "%sAdmin Login Attempts%s%d%s", tablerow_start, tablerow_mid, nipper->sos->admin->accessAttempts, tablerow_end);
		if (nipper->sos->admin->privilege == adminpriv_default)
			fprintf(nipper->reportFile, "%sAdmin Privilages%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->sos->admin->privilege == adminpriv_rw)
			fprintf(nipper->reportFile, "%sAdmin Privilages%sRead / Write%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->sos->admin->privilege == adminpriv_ext)
			fprintf(nipper->reportFile, "%sAdmin Privilages%sGet Externally%s", tablerow_start, tablerow_mid, tablerow_end);
		if (strcmp(nipper->sos->admin->adminIP, "0.0.0.0") == 0)
			fprintf(nipper->reportFile, "%sAdmin Management IP%sAny%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sAdmin Management IP%s%s %s%s", tablerow_start, tablerow_mid, nipper->sos->admin->adminIP, nipper->sos->admin->adminNetMask, tablerow_end);
		addAbbreviation("IP", false);
		if (nipper->sos->admin->consoleOnly == true)
			fprintf(nipper->reportFile, "%sConsole Only Administration%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sConsole Only Administration%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->sos->admin->authServer[0] != 0)
			fprintf(nipper->reportFile, "%sAuthentication Server%s%s%s", tablerow_start, tablerow_mid, nipper->sos->admin->authServer, tablerow_end);
		if (nipper->sos->admin->authTimeout == 0)
			fprintf(nipper->reportFile, "%sAdministration Timeout%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sAdministration Timeout%s%d mins%s", tablerow_start, tablerow_mid, nipper->sos->admin->authTimeout, tablerow_end);
		if (nipper->sos->admin->httpRedirect == true)
			fprintf(nipper->reportFile, "%sHTTP Redirection%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->sos->admin->httpRedirect == false)
			fprintf(nipper->reportFile, "%sHTTP Redirection%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sHTTP Redirection%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
		addAbbreviation("HTTP", false);
		if (nipper->sos->admin->mailAlert == true)
			fprintf(nipper->reportFile, "%sMail Alerts%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sMail Alerts%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->sos->admin->email[0] != 0)
			fprintf(nipper->reportFile, "%sMail Alerts Address%s%s%s", tablerow_start, tablerow_mid, nipper->sos->admin->email, tablerow_end);
		if (nipper->sos->admin->email1[0] != 0)
			fprintf(nipper->reportFile, "%sMail Alerts Address%s%s%s", tablerow_start, tablerow_mid, nipper->sos->admin->email1, tablerow_end);
		if (nipper->sos->admin->email2[0] != 0)
			fprintf(nipper->reportFile, "%sMail Alerts Address%s%s%s", tablerow_start, tablerow_mid, nipper->sos->admin->email2, tablerow_end);
		if (nipper->sos->admin->emailServer[0] != 0)
			fprintf(nipper->reportFile, "%sMail Server%s%s%s", tablerow_start, tablerow_mid, nipper->sos->admin->emailServer, tablerow_end);
		if (nipper->sos->admin->adminFormat == adminfmt_default)
			fprintf(nipper->reportFile, "%sConfiguration Format%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (nipper->sos->admin->adminFormat == adminfmt_dos)
		{
			addAbbreviation("DOS", false);
			fprintf(nipper->reportFile, "%sConfiguration Format%sDOS%s", tablerow_start, tablerow_mid, tablerow_end);
		}
		else if (nipper->sos->admin->adminFormat == adminfmt_unix)
			fprintf(nipper->reportFile, "%sConfiguration Format%sUnix%s", tablerow_start, tablerow_mid, tablerow_end);

		output_table(nipper, false, settings_admin_table, &settings_general_heading);

		if (nipper->sos->admin->users != 0)
		{
			output_table(nipper, true, settings_admins_table, &settings_admins_heading);
			userSOSPointer = nipper->sos->admin->users;
			while (userSOSPointer != 0)
			{
				outputFriendly(userSOSPointer->username, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				if (nipper->passwords == true)
					fprintf(nipper->reportFile, "%s%s", userSOSPointer->password, tablerow_mid);
				switch (userSOSPointer->privilege)
				{
					case adminpriv_default:
						fprintf(nipper->reportFile, "Default%s", tablerow_mid);
						break;
					case adminpriv_all:
						fprintf(nipper->reportFile, "All%s", tablerow_mid);
						break;
					case adminpriv_read:
						fprintf(nipper->reportFile, "Read Only%s", tablerow_mid);
						break;
				}
				if (userSOSPointer->trustee == trustee_none)
					fprintf(nipper->reportFile, "None%s", tablerow_end);
				else if (userSOSPointer->trustee == trustee_modem)
					fprintf(nipper->reportFile, "Modem%s", tablerow_end);
				else if (userSOSPointer->trustee == trustee_inter)
					fprintf(nipper->reportFile, "Interface%s", tablerow_end);
				userSOSPointer = userSOSPointer->next;
			}
			output_table(nipper, false, settings_admins_table, &settings_admins_heading);
		}
		output_endsection(nipper, subsection_end);
	}

	// Services
	output_parseTitle(settings_services_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_services_table, &report_services_heading);
	if (nipper->sos->sshEnabled == true)
		fprintf(nipper->reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(nipper, false, settings_services_table, &report_services_heading);
	addAbbreviation("SSH", false);

	// Authentication Servers
	if (nipper->sos->authServer != 0)
	{
		output_parseTitle(settings_authservers_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_authservers_table, &settings_authservers_heading);
		authServerPointer = nipper->sos->authServer;
		while (authServerPointer != 0)
		{
			fprintf(nipper->reportFile, "%s%d%s", tablerow_start, authServerPointer->id, tablerow_mid);
			outputFriendly(authServerPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			switch (authServerPointer->type)
			{
				case authtype_builtin:
					fprintf(nipper->reportFile, "Built-in%s", tablerow_mid);
					break;
				case authtype_radius:
					fprintf(nipper->reportFile, "RADIUS%s", tablerow_mid);
					addAbbreviation("RADIUS", false);
					break;
				case authtype_ldap:
					fprintf(nipper->reportFile, "LDAP%s", tablerow_mid);
					addAbbreviation("LDAP", false);
					break;
				case authtype_securid:
					fprintf(nipper->reportFile, "SecurID%s", tablerow_mid);
					break;
			}
			outputFriendly(authServerPointer->serverName, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			if (authServerPointer->backup2[0] == 0)
			{
				outputFriendly(authServerPointer->backup1, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			}
			else
			{
				outputFriendly(authServerPointer->backup1, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s, ", tempString);
				outputFriendly(authServerPointer->backup2, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			}
			if (authServerPointer->timeout == 0)
				fprintf(nipper->reportFile, "None%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d mins%s", authServerPointer->timeout, tablerow_mid);
			if (authServerPointer->forcedTimeout == 0)
				fprintf(nipper->reportFile, "None%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d mins%s", authServerPointer->forcedTimeout, tablerow_mid);
			outputFriendly(authServerPointer->sourceInterface, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			if (nipper->passwords == true)
			{
				outputFriendly(authServerPointer->radiusSecret, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s", tempString);
			}
			fprintf(nipper->reportFile, "%s", tablerow_end);
			authServerPointer = authServerPointer->next;
		}
		output_table(nipper, false, settings_authservers_table, &settings_authservers_heading);
		output_endsection(nipper, subsection_end);
	}

	// SSH
	if (nipper->sos->sshEnabled == true)
	{
		output_parseTitle(settings_ssh_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_ssh_table, &settings_general_heading);
		fprintf(nipper->reportFile, "%sSSH Service%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		fprintf(nipper->reportFile, "%sSSH Protocol Version%s%d%s", tablerow_start, tablerow_mid, nipper->sos->sshVersion, tablerow_end);
		fprintf(nipper->reportFile, "%sSSH V1 Key Generation Time%s%d mins%s", tablerow_start, tablerow_mid, nipper->sos->sshKeyGenTime, tablerow_end);
		fprintf(nipper->reportFile, "%sSSH V2 Public Key%s%s%s", tablerow_start, tablerow_mid, nipper->sos->sshPublicKey, tablerow_end);
		output_table(nipper, false, settings_ssh_table, &settings_general_heading);
	}

	// Interfaces
	if (nipper->sos->interface !=0)
	{
		output_parseTitle(settings_inter_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_inter_table, &settings_interSOS_heading);
		interfaceSOSPointer = nipper->sos->interface;
		while (interfaceSOSPointer != 0)
		{
			outputFriendly(interfaceSOSPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
			if (interfaceSOSPointer->enabled == true)
				fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "No%s", tablerow_mid);
			outputFriendly(interfaceSOSPointer->ipAddress, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			outputFriendly(interfaceSOSPointer->zone, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
			if (interfaceSOSPointer->manageIdent == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageIdent == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageMtrace == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageMtrace == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageNsmgmt == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageNsmgmt == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->managePing == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->managePing == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageSnmp == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageSnmp == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageSsh == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageSsh == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageSsl == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageSsl == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageTelnet == true)
				fprintf(nipper->reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageTelnet == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageWeb == true)
				fprintf(nipper->reportFile, "On%s", tablerow_end);
			else if (interfaceSOSPointer->manageWeb == false)
				fprintf(nipper->reportFile, "Off%s", tablerow_end);
			else
				fprintf(nipper->reportFile, "-%s", tablerow_end);
			interfaceSOSPointer = interfaceSOSPointer->next;
		}
		output_table(nipper, false, settings_inter_table, &settings_interSOS_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("SNMP", false);
		addAbbreviation("SSL", false);
	}

	// Policies
	if (nipper->sos->policies !=0)
	{
		output_parseTitle(settings_policies_title, nipper, rate_none, section_config);
		output_parseText(settings_policies_text, nipper, section_none, rate_none, 0, 0);
		policySOSPointer = nipper->sos->policies;
		while (policySOSPointer != 0)
		{
			if (policySOSPointer->global == true)
				sprintf(tempString2, "Global zone policy list");
			else if (policySOSPointer->name[0] == 0)
				sprintf(tempString2, "Zone %s to zone %s policy list", policySOSPointer->fromZone, policySOSPointer->toZone);
			else
				sprintf(tempString2, "%s policy list (zone %s to zone %s)", policySOSPointer->name, policySOSPointer->fromZone, policySOSPointer->toZone);
			outputFriendly(tempString2, tempString, nipper->maxSize, nipper->outputFormat);
			output_table(nipper, true, tempString, &settings_policies_heading);
			ruleSOSPointer = policySOSPointer->policyRules;
			while (ruleSOSPointer != 0)
			{
				listSOSSourcePointer = ruleSOSPointer->source;
				while (listSOSSourcePointer != 0)
				{
					listSOSDestPointer = ruleSOSPointer->dest;
					while (listSOSDestPointer != 0)
					{
						listSOSServicePointer = ruleSOSPointer->service;
						while (listSOSServicePointer != 0)
						{
							fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
							switch (ruleSOSPointer->permit)
							{
								case sos_permit_permit:
									fprintf(nipper->reportFile, "Permit%s", tablerow_mid);
									break;
								case sos_permit_deny:
									fprintf(nipper->reportFile, "Deny%s", tablerow_mid);
									break;
								case sos_permit_reject:
									fprintf(nipper->reportFile, "Reject%s", tablerow_mid);
									break;
							}
							if (ruleSOSPointer->disabled == true)
								fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
							else
								fprintf(nipper->reportFile, "No%s", tablerow_mid);
							fprintf(nipper->reportFile, "%s%s", listSOSSourcePointer->name, tablerow_mid);
							fprintf(nipper->reportFile, "%s%s", listSOSDestPointer->name, tablerow_mid);
							fprintf(nipper->reportFile, "%s%s", listSOSServicePointer->name, tablerow_mid);
							addAbbreviation(listSOSServicePointer->name, false);
							if (ruleSOSPointer->log == true)
								fprintf(nipper->reportFile, "Yes%s", tablerow_end);
							else
								fprintf(nipper->reportFile, "No%s", tablerow_end);
							listSOSServicePointer = listSOSServicePointer->next;
						}
						listSOSDestPointer = listSOSDestPointer->next;
					}
					listSOSSourcePointer = listSOSSourcePointer->next;
				}
				ruleSOSPointer = ruleSOSPointer->next;
			}
			output_table(nipper, false, tempString, &settings_policies_heading);
			policySOSPointer = policySOSPointer->next;
		}
	}

	// Name Lists
	if (nipper->sos->nameList != 0)
	{
		output_parseTitle(settings_namemap_title, nipper, rate_none, section_config);
		addressSOSPointer = nipper->sos->nameList;
		while (addressSOSPointer != 0)
		{
			sprintf(tempString2, "Zone %s name mappings", addressSOSPointer->zone);
			outputFriendly(tempString2, tempString, nipper->maxSize, nipper->outputFormat);
			output_table(nipper, true, tempString, &settings_namemapSOS_heading);
			mappingSOSPointer = addressSOSPointer->mapping;
			while (mappingSOSPointer != 0)
			{
				outputFriendly(mappingSOSPointer->name, tempString2, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString2, tablerow_mid);
				outputFriendly(mappingSOSPointer->address, tempString2, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString2, tablerow_mid);
				outputFriendly(mappingSOSPointer->netMask, tempString2, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString2, tablerow_mid);
				outputFriendly(mappingSOSPointer->comment, tempString2, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString2, tablerow_end);
				mappingSOSPointer = mappingSOSPointer->next;
			}
			output_table(nipper, false, tempString, &settings_namemapSOS_heading);
			addressSOSPointer = addressSOSPointer->next;
		}
	}

	// End of configuration section
	output_endsection(nipper, section_end);

	// Output end of file stuff
	output_fileend(nipper);
}

