/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


void processNameMappings(struct nipperConfig *nipper)
{
	// Variables...
	char line[nipper->maxSize];				// Line from configuration file

	// Pointers...
	struct namePIXList *namePIXPointer = 0;
	struct staticPIX *staticPIXPointer = 0;
	struct routePIX *routePIXPointer = 0;
	struct accessListPIX *accessPIXPointer = 0;
	struct accessListPIXPermission *permissionPIXPointer = 0;
	struct objectGroupList *objectGroupPointer = 0;
	struct objectList *objectPointer = 0;
	struct interfacePIXConfig *interfacePIXPointer = 0;
	struct policySOS *policySOSPointer = 0;
	struct zonePolicyRuleSOS *ruleSOSPointer = 0;
	struct addressSOS *addressSOSPointer = 0;
	struct listSOS *listSOSPointer = 0;
	struct mappingSOS *mappingSOSPointer = 0;


	switch (nipper->deviceType)
	{

		// PIX Devices...
		case type_pix_firewall:
		case type_asa_firewall:
		case type_fwsm_firewall:

			if (nipper->pix->nameList != 0)
			{
				// Convert Interfaces
				interfacePIXPointer = nipper->pix->interface;
				while (interfacePIXPointer != 0)
				{
					// Search for a match
					namePIXPointer = nipper->pix->nameList;
					while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, interfacePIXPointer->ipAddress) != 0))
						namePIXPointer = namePIXPointer->next;

					// If match has been found
					if (strcmp(namePIXPointer->name, interfacePIXPointer->ipAddress) == 0)
					{
						strncpy(interfacePIXPointer->ipAddress, namePIXPointer->address, sizeof(interfacePIXPointer->ipAddress) - 1);
						interfacePIXPointer->ipAddress[strlen(namePIXPointer->address)] = 0;
					}

					// Next Interface
					interfacePIXPointer = interfacePIXPointer->next;
				}

				// Static NAT (Mapped IP)
				staticPIXPointer = nipper->pix->staticNAT;
				while (staticPIXPointer != 0)
				{
					// Search for a match
					namePIXPointer = nipper->pix->nameList;
					while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, staticPIXPointer->ipMapped) != 0))
						namePIXPointer = namePIXPointer->next;

					// If match has been found
					if (strcmp(namePIXPointer->name, staticPIXPointer->ipMapped) == 0)
					{
						strncpy(staticPIXPointer->ipMapped, namePIXPointer->address, sizeof(staticPIXPointer->ipMapped) - 1);
						staticPIXPointer->ipMapped[strlen(namePIXPointer->address)] = 0;
					}

					// Next
					staticPIXPointer = staticPIXPointer->next;
				}

				// Static NAT (Real IP)
				staticPIXPointer = nipper->pix->staticNAT;
				while (staticPIXPointer != 0)
				{
					// Search for a match
					namePIXPointer = nipper->pix->nameList;
					while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, staticPIXPointer->ipReal) != 0))
						namePIXPointer = namePIXPointer->next;

					// If match has been found
					if (strcmp(namePIXPointer->name, staticPIXPointer->ipReal) == 0)
					{
						strncpy(staticPIXPointer->ipReal, namePIXPointer->address, sizeof(staticPIXPointer->ipReal) - 1);
						staticPIXPointer->ipReal[strlen(namePIXPointer->address)] = 0;
					}

					// Next
					staticPIXPointer = staticPIXPointer->next;
				}

				// Static Routing (IP Address)
				routePIXPointer = nipper->pix->route;
				while (routePIXPointer != 0)
				{
					// Search for a match
					namePIXPointer = nipper->pix->nameList;
					while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, routePIXPointer->ipAddress) != 0))
						namePIXPointer = namePIXPointer->next;

					// If match has been found
					if (strcmp(namePIXPointer->name, routePIXPointer->ipAddress) == 0)
					{
						strncpy(routePIXPointer->ipAddress, namePIXPointer->address, sizeof(routePIXPointer->ipAddress) - 1);
						routePIXPointer->ipAddress[strlen(namePIXPointer->address)] = 0;
					}

					// Next
					routePIXPointer = routePIXPointer->next;
				}

				// Static Routing (Gateway)
				routePIXPointer = nipper->pix->route;
				while (routePIXPointer != 0)
				{
					// Search for a match
					namePIXPointer = nipper->pix->nameList;
					while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, routePIXPointer->gateway) != 0))
						namePIXPointer = namePIXPointer->next;

					// If match has been found
					if (strcmp(namePIXPointer->name, routePIXPointer->gateway) == 0)
					{
						strncpy(routePIXPointer->gateway, namePIXPointer->address, sizeof(routePIXPointer->gateway) - 1);
						routePIXPointer->gateway[strlen(namePIXPointer->address)] = 0;
					}

					// Next
					routePIXPointer = routePIXPointer->next;
				}

				// ACL (Source)
				accessPIXPointer = nipper->pix->access;
				while (accessPIXPointer != 0)
				{
					permissionPIXPointer = accessPIXPointer->permission;
					while (permissionPIXPointer != 0)
					{
						// Search for a match
						namePIXPointer = nipper->pix->nameList;
						while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, permissionPIXPointer->source) != 0))
							namePIXPointer = namePIXPointer->next;

						// If match has been found
						if (strcmp(namePIXPointer->name, permissionPIXPointer->source) == 0)
						{
							strncpy(permissionPIXPointer->source, namePIXPointer->address, sizeof(permissionPIXPointer->source) - 1);
							permissionPIXPointer->source[strlen(namePIXPointer->address)] = 0;
						}

						// Next
						permissionPIXPointer = permissionPIXPointer->next;
					}

					// Next
					accessPIXPointer = accessPIXPointer->next;
				}

				// ACL (Destination)
				accessPIXPointer = nipper->pix->access;
				while (accessPIXPointer != 0)
				{
					permissionPIXPointer = accessPIXPointer->permission;
					while (permissionPIXPointer != 0)
					{
						// Search for a match
						namePIXPointer = nipper->pix->nameList;
						while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, permissionPIXPointer->dest) != 0))
							namePIXPointer = namePIXPointer->next;

						// If match has been found
						if (strcmp(namePIXPointer->name, permissionPIXPointer->dest) == 0)
						{
							strncpy(permissionPIXPointer->dest, namePIXPointer->address, sizeof(permissionPIXPointer->dest) - 1);
							permissionPIXPointer->dest[strlen(namePIXPointer->address)] = 0;
						}

						// Next
						permissionPIXPointer = permissionPIXPointer->next;
					}

					// Next
					accessPIXPointer = accessPIXPointer->next;
				}

				// Object Groups
				objectGroupPointer = nipper->pix->objectGroup;
				while (objectGroupPointer != 0)
				{
					if (objectGroupPointer->type == object_type_network)
					{
						objectPointer = objectGroupPointer->object;
						while (objectPointer != 0)
						{
							if ((objectPointer->type == object_type_host) || (objectPointer->type == object_type_network))
							{
								// Search for a match
								namePIXPointer = nipper->pix->nameList;
								while ((namePIXPointer->next != 0) && (strcmp(namePIXPointer->name, objectPointer->object) != 0))
									namePIXPointer = namePIXPointer->next;

								// If match has been found
								if (strcmp(namePIXPointer->name, objectPointer->object) == 0)
								{
									strncpy(objectPointer->object, namePIXPointer->address, sizeof(objectPointer->object) - 1);
									objectPointer->object[strlen(namePIXPointer->address)] = 0;
								}
							}

							// Next
							objectPointer = objectPointer->next;
						}
					}

					// Next
					objectGroupPointer = objectGroupPointer->next;
				}
			}
			break;


		// ScreenOS Devices...
		case type_sos_firewall:
			if (nipper->sos->nameList != 0)
			{
				// Convert Policy Rules
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while (ruleSOSPointer != 0)
					{

						// Source Address
						// Find relevent zone name list
						addressSOSPointer = nipper->sos->nameList;
						while ((strcmp(policySOSPointer->fromZone, addressSOSPointer->zone) != 0) && (addressSOSPointer->next != 0))
							addressSOSPointer = addressSOSPointer->next;
						// If zone found
						if (strcmp(policySOSPointer->fromZone, addressSOSPointer->zone) == 0)
						{
							listSOSPointer = ruleSOSPointer->source;
							while (listSOSPointer != 0)
							{
								mappingSOSPointer = addressSOSPointer->mapping;
								while ((mappingSOSPointer->next != 0) && (strcmp(listSOSPointer->name, mappingSOSPointer->name) != 0))
									mappingSOSPointer = mappingSOSPointer->next;

								// If found
								if (strcmp(listSOSPointer->name, mappingSOSPointer->name) == 0)
								{
									if (mappingSOSPointer->netMask[0] == 0)
										sprintf(line, "%s", mappingSOSPointer->address);
									else
										sprintf(line, "%s %s", mappingSOSPointer->address, mappingSOSPointer->netMask);
									strncpy(listSOSPointer->name, line, sizeof(listSOSPointer->name) - 1);
								}
								listSOSPointer = listSOSPointer->next;
							}
						}

						// Destination Address
						// Find relevent zone name list
						addressSOSPointer = nipper->sos->nameList;
						while ((strcmp(policySOSPointer->toZone, addressSOSPointer->zone) != 0) && (addressSOSPointer->next != 0))
							addressSOSPointer = addressSOSPointer->next;
						// If zone found
						if (strcmp(policySOSPointer->toZone, addressSOSPointer->zone) == 0)
						{
							listSOSPointer = ruleSOSPointer->dest;
							while (listSOSPointer != 0)
							{
								mappingSOSPointer = addressSOSPointer->mapping;
								while ((mappingSOSPointer->next != 0) && (strcmp(listSOSPointer->name, mappingSOSPointer->name) != 0))
									mappingSOSPointer = mappingSOSPointer->next;

								// If found
								if (strcmp(listSOSPointer->name, mappingSOSPointer->name) == 0)
								{
									if (mappingSOSPointer->netMask[0] == 0)
										sprintf(line, "%s", mappingSOSPointer->address);
									else
										sprintf(line, "%s %s", mappingSOSPointer->address, mappingSOSPointer->netMask);
									strncpy(listSOSPointer->name, line, sizeof(listSOSPointer->name) - 1);
								}
								listSOSPointer = listSOSPointer->next;
							}
						}
						ruleSOSPointer = ruleSOSPointer->next;
					}
					policySOSPointer = policySOSPointer->next;
				}
			}
			break;
	}
}
