/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by the following authors:                   *
 *                                                                         *
 *       Ian Ventura-Whiting (fizz@titania.co.uk)                          *
 *       David Croft                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/


void generateIOSReport(struct nipperConfig *nipper)
{
	// Variables
	struct accessList *accessPointer = 0;
	struct accessListPermission *permissionPointer = 0;
	struct ntpHost *ntpHostPointer = 0;
	struct ntpKey *ntpKeyPointer = 0;
	struct ntpAccessGroup *ntpGroupPointer = 0;
	struct enablePassword *enablePointer = 0;
	struct usernameList *usernamePointer = 0;
	struct privilageLevels *privilagePointer = 0;
	struct snmpCommunity *communityPointer = 0;
	struct snmpView *snmpViewPointer = 0;
	struct snmpViewItem *snmpViewItemPointer = 0;
	struct snmpGroup *snmpGroupPointer = 0;
	struct snmpUser *snmpUserPointer = 0;
	struct snmpTrap *snmpTrapPointer = 0;
	struct snmpHost *snmpHostPointer = 0;
	struct host *hostPointer = 0;
	struct aaaGroupServer *aaaGroupPointer = 0;
	struct aaaServer *aaaServerPointer = 0;
	struct netInterface *interfacePointer = 0;
	struct interfaceConfig *interfaceConfigPointer = 0;
	struct bgpNeighbor *neighborPointer = 0;
	struct ospfNetworkConfig *ospfNetworkPointer = 0;
	struct ospfAreaConfig *ospfAreaPointer = 0;
	struct ospfNeighbor *ospfNeighborPointer = 0;
	struct ospfConfig *ospfPointer = 0;
	struct ospfInterStruct *ospfInterPointer = 0;
	struct bgpConfig *bgpPointer = 0;
	struct bgpInterStruct *bgpInterPointer = 0;
	struct eigrpConfig *eigrpPointer = 0;
	struct eigrpAutonStruct *eigrpAutonPointer = 0;
	struct eigrpNetwork *eigrpNetworkPointer = 0;
	struct eigrpNeighbor *eigrpNeighborPointer = 0;
	struct keyChain *keyChainPointer = 0;
	struct keyConfig *keyPointer = 0;
	struct lineConfig *linePointer = 0;
	struct bannerConfig *bannerPointer = 0;
	struct staticRoute *routePointer = 0;
	struct vulnerability *vulnPointer = 0;
	struct host *dnsPointer = 0;
	struct routeMapConfig *routeMapPointer = 0;
	struct natConfig *natPointer = 0;
	struct vrrpStruct *vrrpPointer = 0;
	int tempInt = 0;
	int tempInt2 = 0;
	int lineCount = 0;
	int issues = 0;
	char tempString[nipper->maxSize];
	char tempString2[nipper->maxSize];

	// Counter Variables...
	int accessListIssues = 0;
	int ipDirectedIssues = 0;
	int auxIssues = 0;
	int ospfAuthNoneIssues = 0;
	int ospfAuthClearIssues = 0;
	int ospfAuthMD5 = 0;
	int redirectsIssues = 0;
	int enableIssues = 0;
	int proxyArpIssues = 0;
	int unreachIssues = 0;
	int maskReplyIssues = 0;
	int mopIssues = 0;
	int httpReport = 0;
	int uRPFIssues = 0;
	int sshIssue = 0;
	int telnetIssue = 0;
	int snmpIssue = 0;
	int ripIssues = 0;
	int loggingIssue = 0;
	int eigrpIssues = 0;
	int switchportIssues = 0;
	int switchportModeIssues = 0;
	int vrrpIssues = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (nipper->outputFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (nipper->outputFile[0] == 0)
		nipper->reportFile = stdout;
	else
	{
		nipper->reportFile = fopen(nipper->outputFile, "w");
		if (nipper->reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, nipper->outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(nipper->hostname, nipper->friendlyHostname, nipper->maxSize, nipper->outputFormat);

	// Output file heading (if required)
	output_filestart(nipper);

	// -------------------------
	// Security Section Contents
	// -------------------------
	// Report section?
	if (nipper->fullReport == true)
	{
		output_parseSectionIndex(report_security_title, nipper);

		// Start of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "<ul>\n");
				break;
		}
		output_parseIndex("Introduction", nipper, section_security);

		// Software Version
		if ((nipper->versionMajor < 12) || ((nipper->versionMajor == 12) && (((nipper->deviceType != type_ios_catalyst) && (nipper->versionMinor < 4)) || ((nipper->deviceType == type_ios_catalyst) && (nipper->versionMinor < 2)))))
		{
			issues++;
			output_parseIndex(report_osver_title, nipper, section_security);
			nipper->vulnIssues = 0;
			nipper->vulnIssueDos = false;
			nipper->vulnIssueRemote = false;
			vulnPointer = &report_vuln_ios;
			while (vulnPointer->next != 0)
			{
				checkVulnVersion(nipper, vulnPointer);
				vulnPointer = vulnPointer->next;
			}
		}

		// Simple Passwords
		if (nipper->simplePasswords > 1)
		{
			output_parseIndex(report_dict_title1, nipper, section_security);
			issues++;
		}
		else if (nipper->simplePasswords > 0)
		{
			output_parseIndex(report_dict_title2, nipper, section_security);
			issues++;
		}

		// Password Strength
		if (nipper->passwordStrengths > 1)
		{
			output_parseIndex(report_strength_title1, nipper, section_security);
			issues++;
		}
		else if (nipper->passwordStrengths > 0)
		{
			output_parseIndex(report_strength_title2, nipper, section_security);
			issues++;
		}

		// Auto configuration
		if ((nipper->ios->bootNetwork == true) || (nipper->ios->serviceConfig == true))
		{
			output_parseIndex(report_configauto_title, nipper, section_security);
			issues++;
		}

		// IP directed broadcasts
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipDirectBroadcast == true) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
				ipDirectedIssues++;
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (ipDirectedIssues > 0)
		{
			output_parseIndex(report_ipdirected_title, nipper, section_security);
			issues++;
		}

		// BGP Route Dampening
		if (nipper->ios->bgp != 0)
		{
			if (nipper->ios->bgp->dampening != true)
			{
				output_parseIndex(report_bgpdamp_title, nipper, section_security);
				issues++;
			}
		}

		// OSPF Authentication
		ospfPointer = nipper->ios->ospf;
		while (ospfPointer !=0)
		{
			ospfAreaPointer = nipper->ios->ospf->ospfArea;
			if (ospfAreaPointer == 0)
				ospfAuthNoneIssues++;
			while (ospfAreaPointer != 0)
			{
				if (ospfAreaPointer->areaAuth == auth_none)
					ospfAuthNoneIssues++;
				else if (ospfAreaPointer->areaAuth == auth_clear)
					ospfAuthClearIssues++;
				else
					ospfAuthMD5++;
				ospfAreaPointer = ospfAreaPointer->next;
			}
			ospfPointer = ospfPointer->next;
		}
		if ((ospfAuthNoneIssues > 0) || (ospfAuthClearIssues > 0))
		{
			output_parseIndex(report_ospfauth_title, nipper, section_security);
			issues++;
		}

		// EIGRP
		if (nipper->ios->eigrp != 0)
		{
			tempInt2 = 0;
			eigrpPointer = nipper->ios->eigrp;
			while (eigrpPointer != 0)
			{
				tempInt2++;
				eigrpPointer = eigrpPointer->next;
			}
			interfaceConfigPointer = nipper->ios->interface;
			while (interfaceConfigPointer != 0)
			{
				if ((interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
				{
					tempInt = tempInt2;
					eigrpAutonPointer = interfaceConfigPointer->eigrp;
					while (eigrpAutonPointer != 0)
					{
						if (eigrpAutonPointer->passive == true)
							tempInt--;
						eigrpAutonPointer = eigrpAutonPointer->next;
					}
					if (tempInt > 0)
						eigrpIssues++;
				}
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			if (eigrpIssues > 0)
			{
				output_parseIndex(report_eigrpauth_title, nipper, section_security);
				issues++;
			}
		}

		// RIP
		if (nipper->ios->rip != 0)
		{
			interfaceConfigPointer = nipper->ios->interface;
			while (interfaceConfigPointer != 0)
			{
				if (((interfaceConfigPointer->ripKeyChain[0] == 0) || (interfaceConfigPointer->ripAuthMode != rip_auth_md5) || (interfaceConfigPointer->ripSendVersion != rip_version2) || (interfaceConfigPointer->ripReceiveVersion != rip_version2)) && (interfaceConfigPointer->ripPassive == false) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					ripIssues++;
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			if (ripIssues > 0)
			{
				output_parseIndex(report_rip_title, nipper, section_security);
				issues++;
			}
		}

		// VRRP
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			vrrpPointer = interfaceConfigPointer->vrrp;
			while (vrrpPointer != 0)
			{
				if ((interfaceConfigPointer->shutdown == false) && (vrrpPointer->shutdown == false) && (vrrpPointer->authentication != vrrp_auth_md5))
					vrrpIssues++;
				vrrpPointer = vrrpPointer->next;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (vrrpIssues > 0)
		{
			output_parseIndex(report_vrrp_title, nipper, section_security);
			issues++;
		}

		// TCP Keep Alives
		if (nipper->ios->keepAlivesIn == false)
		{
			output_parseIndex(report_keepalive_title, nipper, section_security);
			issues++;
		}

		// Connection Timeout
		linePointer = nipper->ios->line;
		while (linePointer != 0)
		{
			if ((linePointer->login != login_disabled) && ((linePointer->timeout == 0) || (linePointer->timeout > nipper->connectionTimeout)) && ((linePointer->absoluteTimeout == 0) || (linePointer->absoluteTimeout > nipper->connectionTimeout)) && ((linePointer->sessionTimeout == 0) || (linePointer->sessionTimeout > nipper->connectionTimeout)) && ((linePointer->execTimeout == 0) || (linePointer->execTimeout > nipper->connectionTimeout)))
				nipper->insecureTimeouts++;
			linePointer = linePointer->next;
		}
		if (nipper->insecureTimeouts > 0)
		{
			output_parseIndex(report_timeout_title, nipper, section_security);
			issues++;
		}

		// Aux port
		linePointer = nipper->ios->line;
		while (linePointer != 0)
		{
			if ((linePointer->callback == false) && (linePointer->login != login_disabled) && (linePointer->exec == true) && (linePointer->lineType == line_aux))
				auxIssues++;
			linePointer = linePointer->next;
		}
		if (auxIssues > 0)
		{
			output_parseIndex(report_aux_title, nipper, section_security);
			issues++;
		}

		// Source Routing
		if ((nipper->ios->sourceRouting != false) && (nipper->deviceType == type_ios_router))
		{
			output_parseIndex(report_source_title, nipper, section_security);
			issues++;
		}

		// Finger
		if (nipper->ios->finger != false)
		{
			output_parseIndex(report_finger_title, nipper, section_security);
			issues++;
		}

		// HTTP
		httpReport = false;
		if (nipper->ios->http == 0)
			httpReport = true;
		else if (nipper->ios->http->enabled == true)
			httpReport = true;
		if (httpReport == true)
		{
			output_parseIndex(report_http_title, nipper, section_security);
			issues++;
		}

		// SNMP
		if (nipper->ios->snmp != 0)
		{
			if (nipper->ios->snmp->enabled == true)
			{
				if (nipper->ios->snmp->community != 0)
					snmpIssue = 1;
				snmpGroupPointer = nipper->ios->snmp->group;
				while (snmpGroupPointer != 0)
				{
					if (snmpGroupPointer->version == snmp1)
						snmpIssue = 1;
					else if ((snmpGroupPointer->version == snmp2c) && (snmpIssue != 1))
						snmpIssue = 2;
					else if (((snmpGroupPointer->version == snmp3_no_auth) || (snmpGroupPointer->version == snmp3_auth)) && (snmpIssue == 0))
						snmpIssue = -1;
					snmpGroupPointer = snmpGroupPointer->next;
				}
			}
		}
		if (snmpIssue != 0)
		{
			output_parseIndex(report_snmp_title, nipper, section_security);
			issues++;
		}

		// Telnet
		telnetIssue = false;
		linePointer = nipper->ios->line;
		while (linePointer != 0)
		{
			if ((linePointer->telnet == true) && (linePointer->exec == true) && (linePointer->login != login_disabled))
				telnetIssue = true;
			linePointer = linePointer->next;
		}
		if (telnetIssue == true)
		{
			output_parseIndex(report_telnet_title, nipper, section_security);
			issues++;
		}

		// Redirects
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipRedirects == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
			{
				redirectsIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (redirectsIssues > 0)
		{
			output_parseIndex(report_redirects_title, nipper, section_security);
			issues++;
		}

		// Access Lists
		accessPointer = nipper->ios->access;
		while (accessPointer != 0)
		{
			permissionPointer = accessPointer->permission;
			while (permissionPointer != 0)
			{
				if (permissionPointer->permission == perm_permit)
				{
					// Standard Access List...
					if (accessPointer->type == ios_acl_standard)
					{
						// From any source...
						if (nipper->strictACL == true)
						{
							if (strcmp(permissionPointer->source, "any") == 0)
								accessListIssues++;
						}
					}
					// Extended Access List / old...
					else
					{
						if ((((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0)) || (strcmp(permissionPointer->dest, "any") == 0) || (strcmp(permissionPointer->destService, "any") == 0)) && (strcmp(permissionPointer->protocol, "icmp") != 0))
							accessListIssues++;
					}
				}
				// Deny & Log last...
				if ((nipper->denyLog == true) && (permissionPointer->next == 0) && ((permissionPointer->permission == perm_permit) || (permissionPointer->log == false) || (strcmp(permissionPointer->source, "any") != 0)))
				{
					if (accessPointer->type == ios_acl_standard)
						accessListIssues++;
					else if ((strcmp(permissionPointer->dest, "any") != 0) || (permissionPointer->destOperator == perm_op_none) || (permissionPointer->sourceOperator == perm_op_none) || (strcmp(permissionPointer->destService, "any") != 0) || (strcmp(permissionPointer->protocol, "ip") != 0))
						accessListIssues++;
				}
				permissionPointer = permissionPointer->next;
			}
			accessPointer = accessPointer->next;
		}
		if ((nipper->ios->access == 0) && (nipper->deviceType == type_ios_router))
			accessListIssues = -1;
		if (accessListIssues != 0)
		{
			output_parseIndex(report_accesslist_title, nipper, section_security);
			issues++;
		}

		// uRPF verification
		if ((nipper->versionMajor > 11) && (nipper->deviceType == type_ios_router) && (nipper->deviceUse == use_edge))
		{
			// If cef hasn't been enabled...
			if (nipper->ios->cef == false)
				uRPFIssues = -1;

			// Look for interfaces without uRPF on...
			else
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->uRPF == false) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						uRPFIssues++;
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
			}
		}
		if (uRPFIssues != 0)
		{
			output_parseIndex(report_urpf_title, nipper, section_security);
			issues++;
		}

		// Switch Port Mode
		if (nipper->deviceType != type_ios_router)
		{
			interfaceConfigPointer = nipper->ios->interface;
			while (interfaceConfigPointer != 0)
			{
				if ((interfaceConfigPointer->switchportMode != sp_mode_access) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Vlan", 4) != 0))
					switchportModeIssues++;
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
		}
		if (switchportModeIssues > 0)
		{
			output_parseIndex(report_trunking_title, nipper, section_security);
			issues++;
		}

		// Switch Port Security
		if (nipper->deviceType != type_ios_router)
		{
			interfaceConfigPointer = nipper->ios->interface;
			while (interfaceConfigPointer != 0)
			{
				if ((interfaceConfigPointer->switchportSecurity == false) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Vlan", 4) != 0))
					switchportIssues++;
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
		}
		if (switchportIssues > 0)
		{
			output_parseIndex(report_portsecurity_title, nipper, section_security);
			issues++;
		}

		// Logging
		if (nipper->ios->logging == 0)
			loggingIssue = -1;
		else
		{
			if (nipper->ios->logging->logging == false)
				loggingIssue = 1;
			else if ((nipper->ios->logging->logServer == 0) && (nipper->ios->logging->bufferedLogging == 0))
				loggingIssue = 2;
			else if (nipper->ios->logging->logServer == 0)
				loggingIssue = 3;
			else if (nipper->ios->logging->bufferedLogging == 0)
				loggingIssue = 4;
		}
		if (loggingIssue != 0)
		{
			if (nipper->loggingLevels == unconfigured)
				nipper->loggingLevels = true;
			issues++;
			output_parseIndex(report_logging_title, nipper, section_security);
		}

		// Proxy ARP
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->proxyArp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
			{
				proxyArpIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (proxyArpIssues > 0)
		{
			output_parseIndex(report_proxyarp_title, nipper, section_security);
			issues++;
		}

		// SSH Protocol Version
		tempInt = 0;
		linePointer = nipper->ios->line;
		while (linePointer != 0)
		{
			if ((linePointer->ssh == true) && (linePointer->exec == true) && (linePointer->login != login_disabled))
				tempInt++;
			linePointer = linePointer->next;
		}
		if (tempInt > 0)
		{
			if (nipper->ios->ssh != 0)
			{
				if (nipper->ios->ssh->protocolVersion != ssh_ver_2)
					sshIssue = 2;
			}
			else
				sshIssue = 2;
			if ((nipper->versionMajor == 12) && (nipper->versionMinor < 2))
				sshIssue = 1;
		}
		if (sshIssue != 0)
		{
			output_parseIndex(report_sshproto_title, nipper, section_security);
			issues++;
		}

		// CDP
		if (nipper->ios->cdp != false)
		{
			output_parseIndex(report_cdp_title, nipper, section_security);
			issues++;
		}

		// IP Classless Routing
		if ((nipper->ios->classlessRouting == true) && (nipper->deviceType == type_ios_router))
		{
			output_parseIndex(report_classless_title, nipper, section_security);
			issues++;
		}

		// Minimum Password Length
		if ((nipper->versionMajor == 12) && (nipper->versionMinor > 2) && (nipper->ios->passwordMinLength < nipper->passwordLength))
		{
			output_parseIndex(report_passlength_title, nipper, section_security);
			issues++;
		}

		// Bootp
		if (nipper->ios->bootp == true)
		{
			output_parseIndex(report_bootp_title, nipper, section_security);
			issues++;
		}

		// Small Servers
		if ((nipper->ios->udpSmallServers != false) || (nipper->ios->tcpSmallServers != false))
		{
			output_parseIndex(report_small_title, nipper, section_security);
			issues++;
		}

		// IP unreachables
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipUnreachables == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
			{
				unreachIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (unreachIssues > 0)
		{
			output_parseIndex(report_icmpunreach_title, nipper, section_security);
			issues++;
		}

		// IP Mask Reply
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->ipMaskReply == true) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->ipAddress, "None", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
			{
				maskReplyIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (maskReplyIssues > 0)
		{
			output_parseIndex(report_maskreply_title, nipper, section_security);
			issues++;
		}

		// Enable Secret Passwords
		enablePointer = nipper->ios->enable;
		while (enablePointer != 0)
		{
			if (enablePointer->secret == false)
				enableIssues++;
			enablePointer = enablePointer->next;
		}
		if (enableIssues > 0)
		{
			output_parseIndex(report_enablesec_title, nipper, section_security);
			issues++;
		}

		// Password Encryption Service
		if (nipper->ios->servicePasswordEncryption == false)
		{
			output_parseIndex(report_passenc_title, nipper, section_security);
			issues++;
		}

		// Banners
		if (nipper->ios->banner == 0)
		{
			output_parseIndex(report_banner_title, nipper, section_security);
			issues++;
		}

		// Domain Lookup
		if (nipper->ios->domainLookup == true)
		{
			output_parseIndex(report_lookup_title, nipper, section_security);
			issues++;
		}

		// PAD
		if (nipper->ios->pad == true)
		{
			output_parseIndex(report_pad_title, nipper, section_security);
			issues++;
		}

		// MOP
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->mop == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
			{
				mopIssues++;
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (mopIssues > 0)
		{
			output_parseIndex(report_mop_title, nipper, section_security);
			issues++;
		}

		// End of security index
		switch (nipper->outputFormat)
		{
			case output_html:
				fprintf(nipper->reportFile, "</ul>\n");
				break;
			case output_text:
				fprintf(nipper->reportFile, "\n");
				break;
		}
	}

	// ------------------------------
	// Configuration Section Contents
	// ------------------------------
	output_parseSectionIndex(report_config_title, nipper);

	// Start of configuration index
	switch (nipper->outputFormat)
	{
		case output_html:
			fprintf(nipper->reportFile, "<ul>\n");
			break;
	}
	output_parseIndex("Introduction", nipper, section_config);

	// In Configuration Index
	output_parseIndex(settings_general_title, nipper, section_config);
	output_parseIndex(settings_dns_title, nipper, section_config);
	output_parseIndex(settings_timezone_title, nipper, section_config);
	if (((nipper->ios->enable != 0) || (nipper->ios->username != 0)) && (nipper->passwords == true))
		output_parseIndex(settings_users_title, nipper, section_config);
	output_parseIndex(settings_services_title, nipper, section_config);
	if (nipper->ios->logging != 0)
		output_parseIndex(settings_logging_title, nipper, section_config);
	if (nipper->ios->snmp != 0)
		output_parseIndex(settings_snmp_title, nipper, section_config);
	if (nipper->ios->ssh != 0)
		output_parseIndex(settings_ssh_title, nipper, section_config);
	if (nipper->ios->ntp != 0)
		output_parseIndex(settings_ntp_title, nipper, section_config);
	if (nipper->ios->ftp != 0)
		output_parseIndex(settings_ftp_title, nipper, section_config);
	if (nipper->ios->http != 0)
		output_parseIndex(settings_http_title, nipper, section_config);
	if (nipper->ios->tacacs != 0)
		output_parseIndex(settings_tacacs_title, nipper, section_config);
	if ((nipper->ios->aaa !=0) && (nipper->ios->aaa->groupServer !=0))
		output_parseIndex(settings_aaa_title, nipper, section_config);
	vrrpIssues = 0;
	interfaceConfigPointer = nipper->ios->interface;
	while (interfaceConfigPointer != 0)
	{
		if (interfaceConfigPointer->vrrp != 0)
			vrrpIssues++;
		interfaceConfigPointer = interfaceConfigPointer->next;
	}
	if ((nipper->ios->bgp != 0) || (vrrpIssues != 0) || (nipper->ios->eigrp != 0) || (nipper->ios->isis != 0) || (nipper->ios->rip != 0) || (nipper->ios->ospf != 0) || (nipper->ios->route != 0) || (nipper->ios->routeMap != 0))
		output_parseIndex(settings_routing_title, nipper, section_config);
	if ((nipper->passwords == true) && (nipper->ios->chain != 0))
		output_parseIndex(settings_keys_title, nipper, section_config);
	if (nipper->ios->line != 0)
		output_parseIndex(settings_lines_title, nipper, section_config);
	if (nipper->ios->interface != 0)
		output_parseIndex(settings_inter_title, nipper, section_config);
	if (nipper->deviceType != type_ios_router)
		output_parseIndex(settings_vtp_title, nipper, section_config);
	if ((nipper->ios->interface !=0) && (nipper->deviceType != type_ios_router))
		output_parseIndex(settings_switchport_title, nipper, section_config);
	if (nipper->ios->nat !=0)
		output_parseIndex(settings_nat_title, nipper, section_config);
	if (nipper->ios->access !=0)
		output_parseIndex(settings_accesslists_title, nipper, section_config);

	// End of index
	output_abbrevContents(nipper);
	output_endsection(nipper, subsection_end);

	// Security Report...
	if (nipper->fullReport == true)
	{
		// Output report introduction
		output_parseTitle(report_security_title, nipper, rate_none, section_main_sec);
		output_parseTitle("Introduction", nipper, rate_none, section_security);
		output_parseText(report_security_intro, nipper, section_none, rate_none, issues, 0);
		output_endsection(nipper, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		// Software Version
		if ((nipper->versionMajor < 12) || ((nipper->versionMajor == 12) && (((nipper->deviceType != type_ios_catalyst) && (nipper->versionMinor < 4)) || ((nipper->deviceType == type_ios_catalyst) && (nipper->versionMinor < 2)))))
		{
			output_parseTitle(report_osver_title, nipper, rate_high, section_security);
			output_parseText(report_osver_obs1, nipper, section_obs, rate_none, nipper->vulnIssues, nipper->tableNum);
			if (nipper->vulnIssues > 1)
			{
				output_table(nipper, true, report_osver_table, &report_osver_heading);
				vulnPointer = &report_vuln_ios;
				while (vulnPointer->next != 0)
				{
					if (vulnPointer->include == true)
					{
						outputFriendly(vulnPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						fprintf(nipper->reportFile, "%s%s%s%s", vulnPointer->cve, tablerow_mid, vulnPointer->bid, tablerow_end);
					}
					vulnPointer = vulnPointer->next;
				}
				output_table(nipper, false, report_osver_table, &report_osver_heading);
				output_parseText(report_osver_obs2, nipper, section_none, rate_none, nipper->vulnIssues, nipper->tableNum -1);
			}
			else if (nipper->vulnIssues > 0)
			{
				output_parseText(report_osver_obs3, nipper, section_none, rate_none, nipper->vulnIssues, nipper->tableNum -1);
				vulnPointer = &report_vuln_ios;
				while (vulnPointer->next != 0)
				{
					if (vulnPointer->include == true)
					{
						outputFriendly(vulnPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "\"%s\" (", tempString);
						if (vulnPointer->cve[0] != 0)
							fprintf(nipper->reportFile, "CVE reference %s", vulnPointer->cve);
						if ((vulnPointer->cve[0] != 0) && (vulnPointer->bid[0] != 0))
							fprintf(nipper->reportFile, " and ");
						if (vulnPointer->bid[0] != 0)
							fprintf(nipper->reportFile, " Bugtraq ID %s", vulnPointer->bid);
						fprintf(nipper->reportFile, ").");
					}
					vulnPointer = vulnPointer->next;
				}
				output_parseText(report_osver_obs2, nipper, section_none, rate_none, nipper->vulnIssues, nipper->tableNum -1);
			}
			output_endsection(nipper, section_obs);
			if (nipper->vulnIssues > 0)
			{
				output_parseText(report_osver_impact1, nipper, section_imp, rate_high, nipper->vulnIssues, nipper->tableNum -1);
				if ((nipper->vulnIssueDos == true) && (nipper->vulnIssueRemote == true))
					output_parseText(report_osver_impact2, nipper, section_none, rate_none, nipper->vulnIssues, nipper->tableNum -1);
				else if (nipper->vulnIssueDos == true)
					output_parseText(report_osver_impact3, nipper, section_none, rate_none, nipper->vulnIssues, nipper->tableNum -1);
				else if (nipper->vulnIssueRemote == true)
					output_parseText(report_osver_impact4, nipper, section_none, rate_none, nipper->vulnIssues, nipper->tableNum -1);
				addAbbreviation("CVE", false);
				addAbbreviation("BID", false);
			}
			else
				output_parseText(report_osver_impact5, nipper, section_imp, rate_high, nipper->vulnIssues, nipper->tableNum -1);
			output_endsection(nipper, section_imp);
			output_parseText(report_osver_ease, nipper, section_ease, rate_trivial, nipper->vulnIssues, nipper->tableNum -1);
			output_endsection(nipper, section_ease);
			output_parseText(report_osver_rec, nipper, section_rec, rate_none, nipper->vulnIssues, nipper->tableNum -1);
			output_endsection(nipper, section_rec);
		}

		// Dictionary-based Passwords
		if (nipper->simplePasswords > 0)
		{
			if (nipper->simplePasswords > 1)
				output_parseTitle(report_dict_title1, nipper, rate_high, section_security);
			else
				output_parseTitle(report_dict_title2, nipper, rate_high, section_security);
			output_parseText(report_dict_obs, nipper, section_obs, rate_none, nipper->simplePasswords, nipper->tableNum);
			if (nipper->simplePasswords > 1)
			{
				output_table(nipper, true, report_dict_table, &report_dict_heading);
				output_IOSTablePasswords(nipper, true);
				output_table(nipper, false, report_dict_table, &report_dict_heading);
			}
			else
				output_IOSLinePasswords(nipper, true);
			output_endsection(nipper, section_obs);
			output_parseText(report_dict_impact, nipper, section_imp, rate_high, nipper->simplePasswords, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_dict_ease, nipper, section_ease, rate_trivial, nipper->simplePasswords, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_dict_rec, nipper, section_rec, rate_none, nipper->simplePasswords, nipper->tableNum);
			numToWord(nipper->reportFile, nipper->passwordLength);
			fprintf(nipper->reportFile, " characters in length and contain");
			if (nipper->passwordUppers == true)
				tempInt = 1;
			else
				tempInt = 0;
			if (nipper->passwordLowers == true)
				tempInt++;
			if (nipper->passwordEither == true)
				tempInt++;
			if (nipper->passwordNumbers == true)
				tempInt++;
			if (nipper->passwordSpecials == true)
				tempInt++;
			if (nipper->passwordUppers == true)
			{
				fprintf(nipper->reportFile, " uppercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordLowers == true)
			{
				fprintf(nipper->reportFile, " lowercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordEither == true)
			{
				fprintf(nipper->reportFile, " either uppercase or lowercase characters");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordNumbers == true)
			{
				fprintf(nipper->reportFile, " numbers");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordSpecials == true)
			{
				fprintf(nipper->reportFile, " special");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (((nipper->passwordNumbers == true) && (nipper->passwordSpecials == false)) || ((nipper->passwordEither == false) && (nipper->passwordNumbers == false) && (nipper->passwordSpecials == false)))
				fprintf(nipper->reportFile, ".");
			else
				fprintf(nipper->reportFile, "characters.");
			if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "</text>\n");
			output_newPara(nipper);
			output_endsection(nipper, section_rec);
		}

		// Weak Passwords
		if (nipper->passwordStrengths > 0)
		{
			if (nipper->passwordStrengths > 1)
				output_parseTitle(report_strength_title1, nipper, rate_high, section_security);
			else
				output_parseTitle(report_strength_title2, nipper, rate_high, section_security);
			output_parseText(report_strength_obs, nipper, section_obs, rate_none, nipper->passwordStrengths, nipper->tableNum);
			if (nipper->passwordStrengths > 1)
			{
				output_table(nipper, true, report_strength_table, &report_dict_heading);
				output_IOSTablePasswords(nipper, false);
				output_table(nipper, false, report_strength_table, &report_dict_heading);
			}
			else
				output_IOSLinePasswords(nipper, false);
			output_endsection(nipper, section_obs);
			output_parseText(report_strength_impact, nipper, section_imp, rate_high, nipper->passwordStrengths, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_strength_ease, nipper, section_ease, rate_trivial, nipper->passwordStrengths, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_strength_rec, nipper, section_rec, rate_trivial, nipper->passwordStrengths, nipper->tableNum);
			numToWord(nipper->reportFile, nipper->passwordLength);
			fprintf(nipper->reportFile, " characters in length and contain");
			if (nipper->passwordUppers == true)
				tempInt = 1;
			else
				tempInt = 0;
			if (nipper->passwordLowers == true)
				tempInt++;
			if (nipper->passwordEither == true)
				tempInt++;
			if (nipper->passwordNumbers == true)
				tempInt++;
			if (nipper->passwordSpecials == true)
				tempInt++;
			if (nipper->passwordUppers == true)
			{
				fprintf(nipper->reportFile, " uppercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordLowers == true)
			{
				fprintf(nipper->reportFile, " lowercase");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordEither == true)
			{
				fprintf(nipper->reportFile, " either uppercase or lowercase characters");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordNumbers == true)
			{
				fprintf(nipper->reportFile, " numbers");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (nipper->passwordSpecials == true)
			{
				fprintf(nipper->reportFile, " special");
				if (tempInt > 2)
					fprintf(nipper->reportFile, ",");
				else if (tempInt == 2)
					fprintf(nipper->reportFile, " and");
				tempInt--;
			}
			if (((nipper->passwordNumbers == true) && (nipper->passwordSpecials == false)) || ((nipper->passwordEither == false) && (nipper->passwordNumbers == false) && (nipper->passwordSpecials == false)))
				fprintf(nipper->reportFile, ".");
			else
				fprintf(nipper->reportFile, "characters.");
			if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "</text>\n");
			output_newPara(nipper);
			output_endsection(nipper, section_rec);
		}

		// Configuration Auto-Loading
		if ((nipper->ios->bootNetwork == true) || (nipper->ios->serviceConfig == true))
		{
			output_parseTitle(report_configauto_title, nipper, rate_high, section_security);
			output_parseText(report_configauto_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_configauto_impact, nipper, section_imp, rate_high, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_configauto_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_configauto_rec, nipper, section_rec, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Directed Broadcasts
		if (ipDirectedIssues > 0)
		{
			output_parseTitle(report_ipdirected_title, nipper, rate_high, section_security);
			output_parseText(report_ipdirected_obs, nipper, section_obs, rate_none, ipDirectedIssues, nipper->tableNum);
			if (ipDirectedIssues == 1)
			{
				
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipDirectBroadcast == true) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						fprintf(nipper->reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				fprintf(nipper->reportFile, ".");
				output_newPara(nipper);
			}
			else
			{
				output_table(nipper, true, report_ipdirected_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipDirectBroadcast == true) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_ipdirected_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_ipdirected_impact, nipper, section_imp, rate_high, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_ipdirected_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_ipdirected_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// BGP Route Dampening
		if (nipper->ios->bgp != 0)
		{
			if (nipper->ios->bgp->dampening != true)
			{
				output_parseTitle(report_bgpdamp_title, nipper, rate_medhigh, section_security);
				output_parseText(report_bgpdamp_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
				output_endsection(nipper, section_obs);
				output_parseText(report_bgpdamp_impact, nipper, section_imp, rate_high, 0, nipper->tableNum);
				output_endsection(nipper, section_imp);
				output_parseText(report_bgpdamp_ease, nipper, section_ease, rate_moderate, 0, nipper->tableNum);
				output_endsection(nipper, section_ease);
				output_parseText(report_bgpdamp_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
				output_endsection(nipper, section_rec);
			}
		}

		// OSPF Authentication
		if ((ospfAuthNoneIssues > 0) || (ospfAuthClearIssues > 0))
		{
			output_parseTitle(report_ospfauth_title, nipper, rate_medhigh, section_security);
			output_parseText(report_ospfauth_obs1, nipper, section_obs, rate_none, 0, nipper->tableNum);
			if (ospfAuthMD5 > 0)
				output_parseText(report_ospfauth_obs2, nipper, section_none, rate_none, ospfAuthNoneIssues, nipper->tableNum);
			if ((ospfAuthNoneIssues > 0) && (ospfAuthClearIssues > 0))
				output_parseText(report_ospfauth_obs3, nipper, section_none, rate_none, ospfAuthNoneIssues + ospfAuthClearIssues, nipper->tableNum);
			else if (ospfAuthNoneIssues > 0)
				output_parseText(report_ospfauth_obs4, nipper, section_none, rate_none, ospfAuthNoneIssues, nipper->tableNum);
			else
				output_parseText(report_ospfauth_obs5, nipper, section_none, rate_none, ospfAuthClearIssues, nipper->tableNum);
			if (ospfAuthNoneIssues + ospfAuthClearIssues > 1)
			{
				output_table(nipper, true, report_ospfauth_table, &report_ospfauth_heading);
				ospfPointer = nipper->ios->ospf;
				while (ospfPointer != 0)
				{
					ospfAreaPointer = ospfPointer->ospfArea;
					if (ospfAreaPointer == 0)
						fprintf(nipper->reportFile, "%s%d%sN/A%sNo Authentication%s", tablerow_start, ospfPointer->processID, tablerow_mid, tablerow_mid, tablerow_end);
					while (ospfAreaPointer != 0)
					{
						if (ospfAreaPointer->areaAuth == auth_none)
							fprintf(nipper->reportFile, "%s%d%s%s%sNo Authentication%s", tablerow_start, ospfPointer->processID, tablerow_mid, ospfAreaPointer->areaID, tablerow_mid, tablerow_end);
						else if (ospfAreaPointer->areaAuth == auth_clear)
							fprintf(nipper->reportFile, "%s%d%s%s%sClear-text Authentication%s", tablerow_start, ospfPointer->processID, tablerow_mid, ospfAreaPointer->areaID, tablerow_mid, tablerow_end);
						ospfAreaPointer = ospfAreaPointer->next;
					}
					ospfPointer = ospfPointer->next;
				}
				output_table(nipper, false, report_ospfauth_table, &report_ospfauth_heading);
			}
			else
			{
				ospfPointer = nipper->ios->ospf;
				while (ospfPointer != 0)
				{
					ospfAreaPointer = ospfPointer->ospfArea;
					if (ospfAreaPointer == 0)
						fprintf(nipper->reportFile, "%d.", ospfPointer->processID);
					while (ospfAreaPointer != 0)
					{
						if (ospfAreaPointer->areaAuth == auth_none)
							fprintf(nipper->reportFile, "%d area %s.", ospfPointer->processID, ospfAreaPointer->areaID);
						else if (ospfAreaPointer->areaAuth == auth_clear)
							fprintf(nipper->reportFile, "%d area %s.", ospfPointer->processID, ospfAreaPointer->areaID);
						ospfAreaPointer = ospfAreaPointer->next;
					}
					ospfPointer = ospfPointer->next;
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_ospfauth_impact, nipper, section_imp, rate_high, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_ospfauth_ease, nipper, section_ease, rate_moderate, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_ospfauth_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// EIGRP Authentication
		if (eigrpIssues > 0)
		{
			output_parseTitle(report_eigrpauth_title, nipper, rate_medhigh, section_security);
			output_parseText(report_eigrpauth_obs1, nipper, section_obs, rate_none, eigrpIssues, nipper->tableNum);
			tempInt2 = 0;
			eigrpPointer = nipper->ios->eigrp;
			while (eigrpPointer != 0)
			{
				tempInt2++;
				eigrpPointer = eigrpPointer->next;
			}
			if (eigrpIssues > 1)
			{
				output_table(nipper, true, report_eigrpauth_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						tempInt = tempInt2;
						eigrpAutonPointer = interfaceConfigPointer->eigrp;
						while (eigrpAutonPointer != 0)
						{
							if (eigrpAutonPointer->passive == true)
								tempInt--;
							eigrpAutonPointer = eigrpAutonPointer->next;
						}
						if (tempInt > 0)
						{
							outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
							outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
						}
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_eigrpauth_table, &report_interface_heading);
			}
			else
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						tempInt = tempInt2;
						eigrpAutonPointer = interfaceConfigPointer->eigrp;
						while (eigrpAutonPointer != 0)
						{
							if (eigrpAutonPointer->passive == true)
								tempInt--;
							eigrpAutonPointer = eigrpAutonPointer->next;
						}
						if (tempInt > 0)
						{
							outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s", tempString);
							output_parseText(report_eigrpauth_obs2, nipper, section_none, rate_none, eigrpIssues, 0);
						}
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_eigrpauth_impact, nipper, section_imp, rate_high, eigrpIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_eigrpauth_ease, nipper, section_ease, rate_moderate, eigrpIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_eigrpauth_rec, nipper, section_rec, rate_none, eigrpIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// RIP
		if (ripIssues > 0)
		{
			output_parseTitle(report_rip_title, nipper, rate_medhigh, section_security);
			output_parseText(report_rip_obs, nipper, section_obs, rate_none, ripIssues, nipper->tableNum);
			if (ripIssues == 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if (((interfaceConfigPointer->ripKeyChain[0] == 0) || (interfaceConfigPointer->ripAuthMode != rip_auth_md5) || (interfaceConfigPointer->ripSendVersion != rip_version2) || (interfaceConfigPointer->ripReceiveVersion != rip_version2)) && (interfaceConfigPointer->ripPassive == false) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						fprintf(nipper->reportFile, "%s.", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_newPara(nipper);
			}
			else
			{
				output_table(nipper, true, report_rip_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if (((interfaceConfigPointer->ripKeyChain[0] == 0) || (interfaceConfigPointer->ripAuthMode != rip_auth_md5) || (interfaceConfigPointer->ripSendVersion != rip_version2) || (interfaceConfigPointer->ripReceiveVersion != rip_version2)) && (interfaceConfigPointer->ripPassive == false) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						if ((interfaceConfigPointer->ripKeyChain[0] == 0) && (interfaceConfigPointer->ripAuthMode != rip_auth_md5))
							fprintf(nipper->reportFile, "%s%s", report_rip_tab_desc1, tablerow_end);
						else if (interfaceConfigPointer->ripKeyChain[0] == 0)
							fprintf(nipper->reportFile, "%s%s", report_rip_tab_desc2, tablerow_end);
						else if (interfaceConfigPointer->ripAuthMode != rip_auth_md5)
							fprintf(nipper->reportFile, "%s%s", report_rip_tab_desc3, tablerow_end);
						else if ((interfaceConfigPointer->ripSendVersion != rip_version2) && (interfaceConfigPointer->ripReceiveVersion != rip_version2))
							fprintf(nipper->reportFile, "%s %s", report_rip_tab_desc4, tablerow_end);
						else if (interfaceConfigPointer->ripSendVersion != rip_version2)
							fprintf(nipper->reportFile, "%s %s", report_rip_tab_desc5, tablerow_end);
						else
							fprintf(nipper->reportFile, "%s %s", report_rip_tab_desc6, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_rip_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_rip_impact, nipper, section_imp, rate_high, ripIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_rip_ease, nipper, section_ease, rate_moderate, ripIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_rip_rec, nipper, section_rec, rate_none, ripIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// VRRP
		if (vrrpIssues > 0)
		{
			output_parseTitle(report_vrrp_title, nipper, rate_medhigh, section_security);
			output_parseText(report_vrrp_obs, nipper, section_obs, rate_none, vrrpIssues, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_vrrp_impact, nipper, section_imp, rate_high, vrrpIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_vrrp_ease, nipper, section_ease, rate_moderate, vrrpIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_vrrp_rec, nipper, section_rec, rate_none, vrrpIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// TCP Keep Alives
		if (nipper->ios->keepAlivesIn == false)
		{
			output_parseTitle(report_keepalive_title, nipper, rate_medhigh, section_security);
			output_parseText(report_keepalive_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_keepalive_impact, nipper, section_imp, rate_high, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_keepalive_ease, nipper, section_ease, rate_moderate, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_keepalive_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Timeout
		if (nipper->insecureTimeouts > 0)
			nipper->tableNum = output_IOSTimeout(nipper);

		// Auxiliary Port
		if (auxIssues > 0)
		{
			output_parseTitle(report_aux_title, nipper, rate_med, section_security);
			output_parseText(report_aux_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_aux_impact, nipper, section_imp, rate_high, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_aux_ease, nipper, section_ease, rate_difficult, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_aux_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// IP Source Routing
		if ((nipper->ios->sourceRouting != false) && (nipper->deviceType == type_ios_router))
		{
			output_parseTitle(report_source_title, nipper, rate_med, section_security);
			output_parseText(report_source_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_source_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_source_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_source_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Finger
		if (nipper->ios->finger != false)
		{
			output_parseTitle(report_finger_title, nipper, rate_med, section_security);
			output_parseText(report_finger_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_finger_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_finger_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_finger_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// HTTP
		if (httpReport == true)
		{
			output_parseTitle(report_http_title, nipper, rate_med, section_security);
			output_parseText(report_http_obs1, nipper, section_obs, rate_none, 0, nipper->tableNum);
			if (nipper->ios->http == 0)
				output_parseText(report_http_obs2, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if ((nipper->ios->http->authentication == http_auth_enable) && (nipper->ios->http->accessClass == 0))
				output_parseText(report_http_obs3, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->ios->http->accessClass == 0)
				output_parseText(report_http_obs4, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->ios->http->authentication != http_auth_enable)
				output_parseText(report_http_obs5, nipper, section_none, rate_none, 0, nipper->tableNum);
			else
				output_parseText(report_http_obs6, nipper, section_none, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_http_impact1, nipper, section_imp, rate_med, 0, nipper->tableNum);
			if (nipper->ios->http != 0)
			{
				if (nipper->ios->http->authentication == http_auth_enable)
					output_parseText(report_http_impact2, nipper, section_none, rate_med, 0, nipper->tableNum);
			}
			else
			{
				if (nipper->outputFormat == output_xml)
					fprintf(nipper->reportFile, "</text>\n");
			}
			if (nipper->ios->http != 0)
			{
				if (nipper->ios->http->accessClass[0] != 0)
					output_parseText(report_http_impact3, nipper, section_none, rate_med, 0, nipper->tableNum);
				else
				{
					if (nipper->outputFormat == output_xml)
						fprintf(nipper->reportFile, "</text>\n");
				}
			}
			output_newPara(nipper);
			output_endsection(nipper, section_imp);
			output_parseText(report_http_ease1, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			if (nipper->ios->http != 0)
			{
				if (nipper->ios->http->accessClass[0] != 0)
					output_parseText(report_http_ease2, nipper, section_none, rate_none, 0, nipper->tableNum);
				else
				{
					if (nipper->outputFormat == output_xml)
						fprintf(nipper->reportFile, "</text>\n");
				}
			}
			else
			{
				if (nipper->outputFormat == output_xml)
					fprintf(nipper->reportFile, "</text>\n");
			}
			output_endsection(nipper, section_ease);
			output_newPara(nipper);
			output_parseText(report_http_rec1, nipper, section_rec, rate_none, 0, nipper->tableNum);
			if (nipper->ios->http == 0)
				output_parseText(report_http_rec2, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->ios->http->accessClass[0] != 0)
				output_parseText(report_http_rec2, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->outputFormat == output_xml)
				fprintf(nipper->reportFile, "</text>\n");
			if (nipper->ios->http == 0)
				output_parseText(report_http_rec3, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->ios->http->authentication == http_auth_enable)
				output_parseText(report_http_rec3, nipper, section_none, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// SNMP
		if (snmpIssue != 0)
		{
			output_parseTitle(report_snmp_title, nipper, rate_med, section_security);
			output_parseText(settings_snmp_desc, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_parseText(report_snmp_obs, nipper, section_none, rate_none, snmpIssue, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_snmp_impact, nipper, section_imp, rate_med, snmpIssue, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_snmp_ease, nipper, section_ease, rate_trivial, snmpIssue, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_snmp_rec, nipper, section_rec, rate_none, snmpIssue, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Telnet
		if (telnetIssue == true)
		{
			output_parseTitle(report_telnet_title, nipper, rate_med, section_security);
			output_parseText(report_telnet_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_telnet_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_telnet_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_telnet_rec1, nipper, section_rec, rate_none, 0, nipper->tableNum);
			tempInt = 0;
			linePointer = nipper->ios->line;
			while (linePointer != 0)
			{
				if ((linePointer->telnet == true) && (linePointer->exec == true) && (linePointer->login != login_disabled) && (linePointer->aclIn == 0))
					tempInt++;
				linePointer = linePointer->next;
			}
			if (tempInt > 0)
				output_parseText(report_telnet_rec2, nipper, section_none, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// ICMP Redirects
		if (redirectsIssues > 0)
		{
			output_parseTitle(report_redirects_title, nipper, rate_med, section_security);
			output_parseText(report_redirects_obs, nipper, section_obs, rate_none, redirectsIssues, nipper->tableNum);
			if (redirectsIssues == 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipRedirects == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						fprintf(nipper->reportFile, "%s.", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_newPara(nipper);
			}
			else
			{
				output_table(nipper, true, report_redirects_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipRedirects == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_redirects_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_redirects_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_redirects_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_redirects_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Access Lists
		if (accessListIssues != 0)
		{
			// Init...
			int denyIssue = false;
			// Output...
			output_parseTitle(report_accesslist_title, nipper, rate_med, section_security);
			output_parseText(report_accesslist_obs1, nipper, section_obs, rate_none, accessListIssues, nipper->tableNum);
			// One Access List issue...
			if (accessListIssues == 1)
			{
				accessPointer = nipper->ios->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->type == ios_acl_standard)
							{
								// From any source...
								if ((strcmp(permissionPointer->source, "any") == 0) && (nipper->strictACL == true))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue5);
									output_newPara(nipper);
								}
							}
							// Extended Access List...
							else
							{
								// To any destination and service from any source...
								if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue1);
									output_newPara(nipper);
								}

								// To any destination and service...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue2);
									output_newPara(nipper);
								}

								// From any service to and servie...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue3);
									output_newPara(nipper);
								}

								// From any source to any destination...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue4);
									output_newPara(nipper);
								}

								// Access from any source...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue5);
									output_newPara(nipper);
								}

								// To any destination...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue6);
									output_newPara(nipper);
								}

								// To any servie...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "Line %d in ACL %s %s", lineCount, accessPointer->name, report_accesslist_issue7);
									output_newPara(nipper);
								}
							}
						}
						// Deny & Log last...
						if ((nipper->denyLog == true) && (permissionPointer->next == 0) && ((permissionPointer->permission == perm_permit) || (permissionPointer->log == false) || (strcmp(permissionPointer->source, "any") != 0)))
						{
							if (accessPointer->type == ios_acl_standard)
							{
								fprintf(nipper->reportFile, "%s %s %s", report_accesslist_std, accessPointer->name, report_accesslist_issue8);
								output_newPara(nipper);
								denyIssue = true;
							}
							else if ((strcmp(permissionPointer->dest, "any") != 0) || (permissionPointer->destOperator == perm_op_none) || (permissionPointer->sourceOperator == perm_op_none) || (strcmp(permissionPointer->destService, "any") != 0) || (strcmp(permissionPointer->protocol, "ip") != 0))
							{
								fprintf(nipper->reportFile, "%s %s %s", report_accesslist_ext, accessPointer->name, report_accesslist_issue8);
								output_newPara(nipper);
								denyIssue = true;
							}
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
			}

			// More than one issue...
			else if (accessListIssues != -1)
			{
				output_table(nipper, true, report_accesslist_table1, &report_accesslist_heading);
				accessPointer = nipper->ios->access;
				while (accessPointer != 0)
				{
					permissionPointer = accessPointer->permission;
					lineCount = 1;
					while (permissionPointer != 0)
					{
						if (permissionPointer->permission == perm_permit)
						{
							// Standard Access List...
							if (accessPointer->type == ios_acl_standard)
							{
								// From any source...
								if ((strcmp(permissionPointer->source, "any") == 0) && (nipper->strictACL == true))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sStandard%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue5, tablerow_end);
								}
							}
							// Extended Access List...
							else
							{
								// To any destination and service from any source...
								if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue1, tablerow_end);
								}

								// To any destination and service...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue2, tablerow_end);
								}

								// From any service to and servie...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue3, tablerow_end);
								}

								// From any source to any destination...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue4, tablerow_end);
								}

								// Access from any source...
								else if ((nipper->strictACL == true) && (strcmp(permissionPointer->source, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue5, tablerow_end);
								}

								// To any destination...
								else if ((strcmp(permissionPointer->dest, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue6, tablerow_end);
								}

								// To any servie...
								else if ((strcmp(permissionPointer->destService, "any") == 0) && (strcmp(permissionPointer->protocol, "icmp") != 0))
								{
									fprintf(nipper->reportFile, "%s%s%s%d%sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, lineCount, tablerow_mid, tablerow_mid, report_accesslist_tabissue7, tablerow_end);
								}
							}
						}
						// Deny & Log last...
						if ((nipper->denyLog == true) && (permissionPointer->next == 0) && ((permissionPointer->permission == perm_permit) || (permissionPointer->log == false) || (strcmp(permissionPointer->source, "any") != 0)))
						{
							if (accessPointer->type == ios_acl_standard)
							{
								fprintf(nipper->reportFile, "%s%s%s %sStandard%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, tablerow_mid, tablerow_mid, report_accesslist_tabissue8, tablerow_end);
								denyIssue = true;
							}
							else if ((strcmp(permissionPointer->dest, "any") != 0) || (permissionPointer->destOperator == perm_op_none) || (permissionPointer->sourceOperator == perm_op_none) || (strcmp(permissionPointer->destService, "any") != 0) || (strcmp(permissionPointer->protocol, "ip") != 0))
							{
								fprintf(nipper->reportFile, "%s%s%s %sExtended%s%s%s", tablerow_start, accessPointer->name, tablerow_mid, tablerow_mid, tablerow_mid, report_accesslist_tabissue8, tablerow_end);
								denyIssue = true;
							}
						}
						permissionPointer = permissionPointer->next;
						lineCount++;
					}
					accessPointer = accessPointer->next;
				}
				output_table(nipper, false, report_accesslist_table1, &report_accesslist_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_accesslist_impact, nipper, section_imp, rate_med, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_accesslist_ease, nipper, section_ease, rate_noease, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			if (denyIssue == true)
				output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, 2, nipper->tableNum);
			else
				output_parseText(report_accesslist_rec, nipper, section_rec, rate_none, 1, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// uRPF verification
		if (uRPFIssues != 0)
		{
			output_parseTitle(report_urpf_title, nipper, rate_med, section_security);
			output_parseText(report_urpf_obs, nipper, section_obs, rate_none, uRPFIssues, nipper->tableNum);
			if (uRPFIssues > 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				output_table(nipper, true, report_urpf_table, &report_interface_heading);
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->uRPF == false) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_urpf_table, &report_interface_heading);
			}
			else if (uRPFIssues > 0)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->uRPF == false) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s.", tempString);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_urpf_impact, nipper, section_imp, rate_med, uRPFIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_urpf_ease, nipper, section_ease, rate_noease, uRPFIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_urpf_rec, nipper, section_rec, rate_none, uRPFIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Switchport Mode
		if (switchportModeIssues > 0)
		{
			output_parseTitle(report_trunking_title, nipper, rate_med, section_security);
			output_parseText(report_trunking_obs1, nipper, section_obs, rate_none, switchportModeIssues, nipper->tableNum);
			if (switchportModeIssues == 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->switchportMode != sp_mode_access) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Vlan", 4) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s", tempString);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_parseText(report_trunking_obs2, nipper, section_none, rate_none, switchportModeIssues, nipper->tableNum);
			}
			else
			{
				output_table(nipper, true, report_trunking_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->switchportMode != sp_mode_access) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Vlan", 4) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_trunking_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_trunking_impact, nipper, section_imp, rate_medhigh, switchportModeIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_trunking_ease, nipper, section_ease, rate_moderate, switchportModeIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_trunking_rec, nipper, section_rec, rate_none, switchportModeIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Switchport Security
		if (switchportIssues > 0)
		{
			output_parseTitle(report_portsecurity_title, nipper, rate_med, section_security);
			output_parseText(report_portsecurity_obs1, nipper, section_obs, rate_none, switchportIssues, nipper->tableNum);
			if (switchportIssues == 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->switchportSecurity == false) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Vlan", 4) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s", tempString);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_parseText(report_portsecurity_obs2, nipper, section_none, rate_none, switchportIssues, nipper->tableNum);
			}
			else
			{
				output_table(nipper, true, report_portsecurity_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->switchportSecurity == false) && (interfaceConfigPointer->shutdown == false) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (strncmp(interfaceConfigPointer->name, "Vlan", 4) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_portsecurity_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_portsecurity_impact, nipper, section_imp, rate_medhigh, switchportIssues, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_portsecurity_ease, nipper, section_ease, rate_moderate, switchportIssues, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_portsecurity_rec, nipper, section_rec, rate_none, switchportIssues, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Logging
		if (loggingIssue != 0)
		{
			output_parseTitle(report_logging_title, nipper, rate_med, section_security);
			output_parseText(report_logging_obs1, nipper, section_obs, rate_none, loggingIssue, nipper->tableNum);
			switch (loggingIssue)
			{
				case -1:
					output_parseText(report_logging_obs2, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
					break;
				case 1:
					output_parseText(report_logging_obs3, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
					break;
				case 2:
					output_parseText(report_logging_obs4, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
					break;
				case 3:
					output_parseText(report_logging_obs5, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
					break;
				case 4:
					output_parseText(report_logging_obs6, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
					break;
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_logging_impact, nipper, section_imp, rate_med, loggingIssue, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_logging_ease, nipper, section_ease, rate_noease, loggingIssue, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_logging_rec1, nipper, section_rec, rate_none, loggingIssue, nipper->tableNum);
			if (loggingIssue != 4)
				output_parseText(report_logging_rec2, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
			if ((nipper->versionMajor < 12) || ((nipper->versionMajor == 12) && (nipper->versionMinor < 1)))
				output_parseText(report_logging_rec3, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
			if (loggingIssue != 3)
				output_parseText(report_logging_rec4, nipper, section_none, rate_none, loggingIssue, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Proxy ARP
		if (proxyArpIssues > 0)
		{
			output_parseTitle(report_proxyarp_title, nipper, rate_lowmed, section_security);
			output_parseText(report_proxyarp_obs, nipper, section_obs, rate_none, proxyArpIssues, nipper->tableNum);
			if (proxyArpIssues > 1)
			{
				output_table(nipper, true, report_proxyarp_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->proxyArp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s %s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_proxyarp_table, &report_interface_heading);
			}
			else
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->proxyArp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						fprintf(nipper->reportFile, "%s.", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_newPara(nipper);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_proxyarp_impact, nipper, section_imp, rate_lowmed, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_proxyarp_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_proxyarp_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// SSH Protocol Version
		if (sshIssue != 0)
		{
			output_parseTitle(report_sshproto_title, nipper, rate_lowmed, section_security);
			output_parseText(report_sshproto_obs, nipper, section_obs, rate_none, sshIssue, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_sshproto_impact, nipper, section_imp, rate_med, sshIssue, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_sshproto_ease, nipper, section_ease, rate_difficult, sshIssue, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_sshproto_rec2, nipper, section_rec, rate_none, sshIssue, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// CDP
		if (nipper->ios->cdp != false)
		{
			output_parseTitle(report_cdp_title, nipper, rate_lowmed, section_security);

			// Look to see if CDP has been disabled on the network interfaces
			int cdpDisabled = true;
			interfaceConfigPointer = nipper->ios->interface;
			while ((interfaceConfigPointer != 0) && (cdpDisabled == true))
			{
				if ((interfaceConfigPointer->cdp == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0))
					cdpDisabled = false;
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			// If CDP has been disabled on all interfaces...
			if (cdpDisabled == true)
				output_parseText(report_cdp_obs, nipper, section_obs, rate_none, 1, nipper->tableNum);
			else
				output_parseText(report_cdp_obs, nipper, section_obs, rate_none, 2, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_cdp_impact, nipper, section_imp, rate_lowmed, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_cdp_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_cdp_rec1, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// IP Classless Routing
		if ((nipper->ios->classlessRouting == true) && (nipper->deviceType == type_ios_router))
		{
			output_parseTitle(report_classless_title, nipper, rate_low, section_security);
			output_parseText(report_classless_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_classless_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_classless_ease, nipper, section_ease, rate_noease, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_classless_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Minimum Password Length
		if ((nipper->versionMajor == 12) && (nipper->versionMinor > 2) && (nipper->ios->passwordMinLength < nipper->passwordLength))
		{
			output_parseTitle(report_passlength_title, nipper, rate_low, section_security);
			if (nipper->ios->passwordMinLength == 0)
				output_parseText(report_passlength_obs, nipper, section_obs, rate_none, -1, nipper->tableNum);
			else
				output_parseText(report_passlength_obs, nipper, section_obs, rate_none, nipper->ios->passwordMinLength, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_passlength_impact, nipper, section_imp, rate_low, nipper->ios->passwordMinLength, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_passlength_ease, nipper, section_ease, rate_trivial, nipper->ios->passwordMinLength, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_passlength_rec, nipper, section_rec, rate_none, nipper->ios->passwordMinLength, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Bootp
		if (nipper->ios->bootp == true)
		{
			output_parseTitle(report_bootp_title, nipper, rate_low, section_security);
			output_parseText(report_bootp_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_bootp_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_bootp_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_bootp_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// TCP & UDP Small Servers
		if ((nipper->ios->udpSmallServers != false) || (nipper->ios->tcpSmallServers != false))
		{
			// Check for IOS version...
			int olderIOS = false;
			if ((nipper->versionMajor < 11) || ((nipper->versionMajor == 11) && (nipper->versionMinor < 3)))
				olderIOS = true;
			// Report...
			output_parseTitle(report_small_title, nipper, rate_low, section_security);
			output_parseText(report_small_obs1, nipper, section_obs, rate_none, 0, nipper->tableNum);
			if (olderIOS == true)
				output_parseText(report_small_obs2, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if ((nipper->ios->udpSmallServers == true) && (nipper->ios->tcpSmallServers == true))
				output_parseText(report_small_obs3, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->ios->tcpSmallServers == true)
				output_parseText(report_small_obs4, nipper, section_none, rate_none, 0, nipper->tableNum);
			else if (nipper->ios->udpSmallServers == true)
				output_parseText(report_small_obs5, nipper, section_none, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_small_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_small_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_small_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			if ((olderIOS == true) || (nipper->ios->tcpSmallServers == true))
				output_parseText(report_small_rec_tcp, nipper, section_none, rate_none, 0, nipper->tableNum);
			if ((olderIOS == true) || (nipper->ios->udpSmallServers == true))
				output_parseText(report_small_rec_udp, nipper, section_none, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// IP Unreachables
		if (unreachIssues > 0)
		{
			output_parseTitle(report_icmpunreach_title, nipper, rate_low, section_security);
			output_parseText(report_icmpunreach_obs, nipper, section_obs, rate_none, unreachIssues, nipper->tableNum);
			if (unreachIssues == 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipUnreachables == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						fprintf(nipper->reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				fprintf(nipper->reportFile, ".");
				output_newPara(nipper);
			}
			else
			{
				output_table(nipper, true, report_icmpunreach_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipUnreachables == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_icmpunreach_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_icmpunreach_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_icmpunreach_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_icmpunreach_rec1, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// ICMP Mask Reply
		if (maskReplyIssues > 0)
		{
			output_parseTitle(report_maskreply_title, nipper, rate_low, section_security);
			output_parseText(report_maskreply_obs, nipper, section_obs, rate_none, maskReplyIssues, nipper->tableNum);
			if (maskReplyIssues == 1)
			{
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipMaskReply == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
						fprintf(nipper->reportFile, "%s", interfaceConfigPointer->name);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				fprintf(nipper->reportFile, ".");
				output_newPara(nipper);
			}
			else
			{
				output_table(nipper, true, report_maskreply_table, &report_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					if ((interfaceConfigPointer->ipMaskReply == true) && (interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, report_icmpunreach_table, &report_interface_heading);
			}
			output_endsection(nipper, section_obs);
			output_parseText(report_maskreply_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_maskreply_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_maskreply_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Enable Secret Passwords
		if (enableIssues > 0)
		{
			output_parseTitle(report_enablesec_title, nipper, rate_low, section_security);
			output_parseText(report_enablesec_obs, nipper, section_obs, rate_none, enableIssues, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_enablesec_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_enablesec_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_enablesec_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Password Encryption Service
		if (nipper->ios->servicePasswordEncryption == false)
		{
			output_parseTitle(report_passenc_title, nipper, rate_low, section_security);
			output_parseText(report_passenc_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_passenc_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_passenc_ease, nipper, section_ease, rate_trivial, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_passenc_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Banner
		if (nipper->ios->banner == 0)
		{
			output_parseTitle(report_banner_title, nipper, rate_low, section_security);
			output_parseText(report_banner_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_banner_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_banner_ease, nipper, section_ease, rate_noease, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_banner_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// Domain Lookup
		if (nipper->ios->domainLookup == true)
		{
			if (nipper->ios->dns == 0)
				tempInt = 1;
			else
				tempInt = 2;
			output_parseTitle(report_lookup_title, nipper, rate_low, section_security);
			output_parseText(report_lookup_obs, nipper, section_obs, rate_none, tempInt, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_lookup_impact, nipper, section_imp, rate_low, tempInt, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_lookup_ease, nipper, section_ease, rate_trivial, tempInt, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_lookup_rec, nipper, section_rec, rate_none, tempInt, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// PAD
		if (nipper->ios->pad == true)
		{
			output_parseTitle(report_pad_title, nipper, rate_low, section_security);
			output_parseText(report_pad_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_pad_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_pad_ease, nipper, section_ease, rate_noease, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_pad_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// MOP
		if (mopIssues > 0)
		{
			output_parseTitle(report_mop_title, nipper, rate_low, section_security);
			output_parseText(report_mop_obs, nipper, section_obs, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_obs);
			output_parseText(report_mop_impact, nipper, section_imp, rate_low, 0, nipper->tableNum);
			output_endsection(nipper, section_imp);
			output_parseText(report_mop_ease, nipper, section_ease, rate_noease, 0, nipper->tableNum);
			output_endsection(nipper, section_ease);
			output_parseText(report_mop_rec, nipper, section_rec, rate_none, 0, nipper->tableNum);
			output_endsection(nipper, section_rec);
		}

		// End Section
		output_endsection(nipper, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output report introduction
	output_parseTitle(report_config_title, nipper, rate_none, section_main_con);
	output_parseTitle("Introduction", nipper, rate_none, section_config);
	output_parseText(report_config_intro, nipper, section_none, rate_none, 0, 0);
	output_endsection(nipper, subsection_end);

	// General Settings
	output_parseTitle(settings_general_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_general_table, &settings_general_heading);
	fprintf(nipper->reportFile, "%sHostname%s%s%s", tablerow_start, tablerow_mid, nipper->friendlyHostname, tablerow_end);
	if (nipper->version[0] != 0)
		fprintf(nipper->reportFile, "%sIOS Version%s%s%s", tablerow_start, tablerow_mid, nipper->version, tablerow_end);
	if (nipper->ios->servicePasswordEncryption == false)
		fprintf(nipper->reportFile, "%sService Password Encryption%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sService Password Encryption%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if ((nipper->versionMajor == 12) && (nipper->versionMinor > 2))
	{
		if (nipper->ios->passwordMinLength == 0)
			fprintf(nipper->reportFile, "%sMinimum Password Length%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sMinimum Password Length%s%d characters%s", tablerow_start, tablerow_mid, nipper->ios->passwordMinLength, tablerow_end);
	}
	if (nipper->deviceType == type_ios_router)
	{
		if (nipper->ios->sourceRouting == false)
			fprintf(nipper->reportFile, "%sIP Source Routing%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sIP Source Routing%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	if (nipper->ios->bootp == unconfigured)
		fprintf(nipper->reportFile, "%sBOOTP%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
	else if (nipper->ios->bootp == false)
		fprintf(nipper->reportFile, "%sBOOTP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sBOOTP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->ios->serviceConfig == unconfigured)
		fprintf(nipper->reportFile, "%sService Config%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
	else if (nipper->ios->serviceConfig == false)
		fprintf(nipper->reportFile, "%sService Config%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sService Config%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->ios->keepAlivesIn == false)
		fprintf(nipper->reportFile, "%sTCP Keep Alives (In)%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sTCP Keep Alives (In)%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->ios->keepAlivesOut == false)
		fprintf(nipper->reportFile, "%sTCP Keep Alives (Out)%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sTCP Keep Alives (Out)%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->deviceType == type_ios_router)
	{
		if (nipper->ios->cef == false)
			fprintf(nipper->reportFile, "%sCisco Express Forwarding%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sCisco Express Forwarding%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		addAbbreviation("CEF", false);
	}
	if ((nipper->versionMajor > 11) || ((nipper->versionMajor == 11) && (nipper->versionMinor > 2)))
	{
		if (nipper->ios->gratuitousArps == false)
			fprintf(nipper->reportFile, "%sGratuitous ARPs%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sGratuitous ARPs%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		addAbbreviation("ARP", false);
	}
	if (nipper->deviceType == type_ios_router)
	{
		if (nipper->ios->classlessRouting == false)
			fprintf(nipper->reportFile, "%sClassless Routing%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sClassless Routing%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	output_table(nipper, false, settings_general_table, &settings_general_heading);
	output_endsection(nipper, subsection_end);
	addAbbreviation("IOS", false);
	addAbbreviation("IP", false);
	addAbbreviation("BOOTP", false);
	addAbbreviation("UTC", false);

	// DNS Settings
	output_parseTitle(settings_dns_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_dns_table, &settings_general_heading);
	if (nipper->domainName[0] != 0)
	{
		outputFriendly(nipper->domainName, tempString, nipper->maxSize, nipper->outputFormat);
		fprintf(nipper->reportFile, "%sDomain Name%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
	}
	if (nipper->ios->domainLookup == false)
		fprintf(nipper->reportFile, "%sDomain Lookup%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sDomain Lookup%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	tempInt = 1;
	dnsPointer = nipper->ios->dns;
	while (dnsPointer != 0)
	{
		fprintf(nipper->reportFile, "%sDNS Server %d%s%s%s", tablerow_start, tempInt, tablerow_mid, dnsPointer->server, tablerow_end);
		tempInt++;
		dnsPointer = dnsPointer->next;
	}
	output_table(nipper, false, settings_dns_table, &settings_general_heading);
	output_endsection(nipper, subsection_end);

	// DNS Settings
	output_parseTitle(settings_timezone_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_timezone_table, &settings_general_heading);
	fprintf(nipper->reportFile, "%sTime Zone%s%s%s", tablerow_start, tablerow_mid, nipper->ios->timeZone, tablerow_end);
	if ((nipper->ios->timeOffsetHours != 0) || (nipper->ios->timeOffsetMins != 0))
	{
		fprintf(nipper->reportFile, "%sUTC Offset%s", tablerow_start, tablerow_mid);
		if (nipper->ios->timeOffsetHours != 0)
		{
			if (nipper->ios->timeOffsetHours > 0)
				fprintf(nipper->reportFile, "+");
			fprintf(nipper->reportFile, "%d hour", nipper->ios->timeOffsetHours);
			if ((nipper->ios->timeOffsetHours > 1) || (nipper->ios->timeOffsetHours < -1))
				fprintf(nipper->reportFile, "s");
			if (nipper->ios->timeOffsetMins != 0)
				fprintf(nipper->reportFile, " ");
		}
		if (nipper->ios->timeOffsetMins != 0)
		{
			fprintf(nipper->reportFile, "%d min", nipper->ios->timeOffsetMins);
			if ((nipper->ios->timeOffsetMins > 1) || (nipper->ios->timeOffsetMins < -1))
				fprintf(nipper->reportFile, "s");
		}
		fprintf(nipper->reportFile,"%s", tablerow_end);
	}
	else
		fprintf(nipper->reportFile, "%sUTC Offset%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->ios->summerTimeZone[0] != 0)
		fprintf(nipper->reportFile, "%sSummer Time Zone%s%s%s", tablerow_start, tablerow_mid, nipper->ios->timeZone, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sSummer Time Zone%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->ios->calendarValid == true)
		fprintf(nipper->reportFile, "%sAuthorative Time Source%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sAuthorative Time Source%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(nipper, false, settings_timezone_table, &settings_general_heading);
	output_endsection(nipper, subsection_end);

	// Authentication
	if ((nipper->ios->enable != 0) || (nipper->ios->username != 0) || (nipper->ios->privilage != 0) || (nipper->ios->banner != 0))
	{
		output_parseTitle(settings_users_title, nipper, rate_none, section_config);

		// Enable Password
		if (nipper->ios->enable != 0)
		{
			output_table(nipper, true, settings_enable_table, &report_enable_heading);
			enablePointer = nipper->ios->enable;
			while (enablePointer != 0)
			{
				fprintf(nipper->reportFile, "%s%d%s", tablerow_start, enablePointer->level, tablerow_mid);
				if (nipper->passwords == true)
				{
					outputFriendly(enablePointer->password, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
				}
				switch (enablePointer->encryption)
				{
					case encrypt_type7:
						fprintf(nipper->reportFile, "Type-7%s", tablerow_end);
						break;
					case encrypt_md5:
						fprintf(nipper->reportFile, "MD5%s", tablerow_end);
						addAbbreviation("MD5", false);
						break;
					case encrypt_none:
						fprintf(nipper->reportFile, "None%s", tablerow_end);
						break;
					case encrypt_unknown:
						fprintf(nipper->reportFile, "Unknown%s", tablerow_end);
						break;
				}
				enablePointer = enablePointer->next;
			}
			output_table(nipper, false, settings_enable_table, &report_enable_heading);
		}

		// Usernames
		if (nipper->ios->username != 0)
		{
			output_table(nipper, true, settings_users_table, &report_users_heading);
			usernamePointer = nipper->ios->username;
			while (usernamePointer != 0)
			{
				outputFriendly(usernamePointer->username, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, usernamePointer->privilage, tablerow_mid);
				if (nipper->passwords == true)
				{
					outputFriendly(usernamePointer->password, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
				}
				switch (usernamePointer->encryption)
				{
					case encrypt_type7:
						fprintf(nipper->reportFile, "Type-7%s", tablerow_end);
						break;
					case encrypt_md5:
						fprintf(nipper->reportFile, "MD5%s", tablerow_end);
						addAbbreviation("MD5", false);
						break;
					case encrypt_none:
						fprintf(nipper->reportFile, "None%s", tablerow_end);
						break;
					case encrypt_unknown:
						fprintf(nipper->reportFile, "Unknown%s", tablerow_end);
						break;
				}
				usernamePointer = usernamePointer->next;
			}
			output_table(nipper, false, settings_users_table, &report_users_heading);
		}

		// Privilage
		if (nipper->ios->privilage != 0)
		{
			output_parseText(settings_privs_desc, nipper, section_none, rate_none, 0, 0);
			output_table(nipper, true, settings_privs_table, &report_privs_heading);
			privilagePointer = nipper->ios->privilage;
			while (privilagePointer != 0)
			{
				printf("%s%d%s%s%s", tablerow_start, privilagePointer->level, tablerow_mid, privilagePointer->command, tablerow_end);
				privilagePointer = privilagePointer->next;
			}
			output_table(nipper, false, settings_privs_table, &report_privs_heading);
		}

		// Banner
		if (nipper->ios->banner != 0)
		{
			output_parseText(settings_banner_desc, nipper, section_none, rate_none, 0, 0);
			bannerPointer = nipper->ios->banner;
			switch (nipper->outputFormat)
			{
				case output_html:
					fprintf(nipper->reportFile, "<quote>\n");
					break;
				case output_xml:
					fprintf(nipper->reportFile, "   <code>");
					break;
			}
			while (bannerPointer != 0)
			{
				fprintf(nipper->reportFile, "%s", bannerPointer->line);
				switch (nipper->outputFormat)
				{
					case output_html:
						fprintf(nipper->reportFile, "<br>\n");
						break;
					default:
						fprintf(nipper->reportFile, "\n");
						break;
				}
				bannerPointer = bannerPointer->next;
			}
			switch (nipper->outputFormat)
			{
				case output_html:
					fprintf(nipper->reportFile, "</quote>");
					break;
				case output_xml:
					fprintf(nipper->reportFile, "</code>\n");
					break;
			}
			output_newPara(nipper);
		}
		output_endsection(nipper, subsection_end);
	}

	// Services
	output_parseTitle(settings_services_title, nipper, rate_none, section_config);
	output_table(nipper, true, settings_services_table, &report_services_heading);
	tempInt = 0;
	linePointer = nipper->ios->line;
	while (linePointer != 0)
	{
		if (linePointer->telnet == true)
			tempInt++;
		linePointer = linePointer->next;
	}
	switch (tempInt)
	{
		case 0:
			fprintf(nipper->reportFile, "%sTelnet%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		default:
			fprintf(nipper->reportFile, "%sTelnet%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	if (nipper->versionMajor >= 12)
	{
		if (nipper->ios->line != 0)
		{
			tempInt = 0;
			linePointer = nipper->ios->line;
			while (linePointer != 0)
			{
				if (linePointer->ssh == true)
					tempInt++;
				linePointer = linePointer->next;
			}
		}
		switch (tempInt)
		{
			case 0:
				fprintf(nipper->reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			default:
				fprintf(nipper->reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		addAbbreviation("SSH", false);
	}
	if (nipper->ios->http != 0)
	{
		switch (nipper->ios->http->enabled)
		{
			case true:
				fprintf(nipper->reportFile, "%sHTTP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case false:
				fprintf(nipper->reportFile, "%sHTTP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
	}
	else
		fprintf(nipper->reportFile, "%sHTTP%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
	switch (nipper->ios->finger)
	{
		case unconfigured:
			fprintf(nipper->reportFile, "%sFinger%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case false:
			fprintf(nipper->reportFile, "%sFinger%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case true:
			fprintf(nipper->reportFile, "%sFinger%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	switch (nipper->ios->tcpSmallServers)
	{
		case false:
			fprintf(nipper->reportFile, "%sTCP Small Services%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case true:
			fprintf(nipper->reportFile, "%sTCP Small Services%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	switch (nipper->ios->udpSmallServers)
	{
		case false:
			fprintf(nipper->reportFile, "%sUDP Small Services%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
		case true:
			fprintf(nipper->reportFile, "%sUDP Small Services%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			break;
	}
	if (nipper->ios->ntp != 0)
	{
		if (nipper->ios->ntp->enabled == true)
			fprintf(nipper->reportFile, "%sNTP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sNTP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	if (nipper->ios->snmp != 0)
	{
		if (nipper->ios->snmp->enabled == true)
			fprintf(nipper->reportFile, "%sSNMP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sSNMP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	}
	if (nipper->ios->cdp == false)
		fprintf(nipper->reportFile, "%sCDP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sCDP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	if (nipper->ios->pad == true)
		fprintf(nipper->reportFile, "%sPAD%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(nipper->reportFile, "%sPAD%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(nipper, false, settings_services_table, &report_services_heading);
	output_endsection(nipper, subsection_end);
	addAbbreviation("CDP", false);
	addAbbreviation("PAD", false);
	addAbbreviation("NTP", false);
	addAbbreviation("SNMP", false);
	addAbbreviation("UDP", false);
	addAbbreviation("TCP", false);
	addAbbreviation("HTTP", false);

	// Logging
	if (nipper->ios->logging != 0)
	{
		if (nipper->loggingLevels == unconfigured)
			nipper->loggingLevels = true;
		output_parseTitle(settings_logging_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_logging_table, &settings_general_heading);
		if (nipper->ios->logging->logging == true)
			fprintf(nipper->reportFile, "%sLogging%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sLogging%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->ios->logging->logEnable == true)
			fprintf(nipper->reportFile, "%sLog Configuration Changes%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sLog Configuration Changes%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);

		// Logging to console
		if (nipper->ios->logging->consoleLogging != false)
		{
			if (nipper->ios->logging->consoleLogging == true)
				fprintf(nipper->reportFile, "%sConsole Logging%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sConsole Logging%sSystem Default%s", tablerow_start, tablerow_mid, tablerow_end);
			if (nipper->ios->logging->consoleLevel == -1)
				fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
			else
			{
				switch (nipper->ios->logging->consoleLevel)
				{
					case 0:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sEmergencies (0)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 1:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sAlerts (1)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 2:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sCritical (2)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 3:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sErrors (3)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 4:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sWarnings (4)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 5:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sNotifications (5)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 6:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sInformational (6)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 7:
						fprintf(nipper->reportFile, "%sConsole Logging Severity Level%sDebugging (7)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
				}
			}
		}
		else
			fprintf(nipper->reportFile, "%sConsole Logging%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);

		// Syslog stuff (only works if a host is configured)
		if (nipper->ios->logging->logServer != 0)
		{
			fprintf(nipper->reportFile, "%sSyslog Logging%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			switch (nipper->ios->logging->loggingTrap)
			{
				case 0:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sEmergencies (0)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 1:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sAlerts (1)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 2:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sCritical (2)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 3:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sErrors (3)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 4:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sWarnings (4)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 5:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sNotifications (5)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 6:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sInformational (6)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 7:
					fprintf(nipper->reportFile, "%sSyslog Logging Severity Level%sDebugging (7)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
			}
			outputFriendly(nipper->ios->logging->facility, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sSyslog Logging Facility%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			if (nipper->ios->logging->logCount == true)
				fprintf(nipper->reportFile, "%sSyslog Message Counting%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sSyslog Message Counting%sSystem Default%s", tablerow_start, tablerow_mid, tablerow_end);
			if (nipper->ios->logging->originId[0] != 0)
			{
				outputFriendly(nipper->ios->logging->originId, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%sSyslog Origin ID%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			}
			if (nipper->ios->logging->sourceInterface[0] != 0)
			{
				outputFriendly(nipper->ios->logging->sourceInterface, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%sSyslog Source Interface%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			}
			hostPointer = nipper->ios->logging->logServer;
			while (hostPointer != 0)
			{
				outputFriendly(hostPointer->server, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%sSyslog Logging Server%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
				hostPointer = hostPointer->next;
			}
		}
		else
			fprintf(nipper->reportFile, "%sSyslog Logging%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);

		// Logging to buffer
		if (nipper->ios->logging->bufferedLogging != false)
		{
			if (nipper->ios->logging->bufferedLogging == true)
				fprintf(nipper->reportFile, "%sBuffer Logging%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sBuffer Logging%sSystem Default%s", tablerow_start, tablerow_mid, tablerow_end);
			if (nipper->ios->logging->bufferSize == -1)
				fprintf(nipper->reportFile, "%sBuffer Size%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sBuffer Size%s%d bytes%s", tablerow_start, tablerow_mid, nipper->ios->logging->bufferSize, tablerow_end);
			if (nipper->ios->logging->bufferLevel == -1)
				fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
			else
			{
				switch (nipper->ios->logging->bufferLevel)
				{
					case 0:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sEmergencies (0)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 1:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sAlerts (1)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 2:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sCritical (2)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 3:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sErrors (3)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 4:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sWarnings (4)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 5:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sNotifications (5)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 6:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sInformational (6)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case 7:
						fprintf(nipper->reportFile, "%sBuffer Logging Severity Level%sDebugging (7)%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
				}
			}
		}
		else
			fprintf(nipper->reportFile, "%sBuffer Logging%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);

		// Logging to terminal lines
		if (nipper->ios->logging->monitorLogging == true)
		{
			fprintf(nipper->reportFile, "%sTerminal Line Logging%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			switch (nipper->ios->logging->monitorLevel)
			{
				case 0:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sEmergencies (0)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 1:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sAlerts (1)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 2:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sCritical (2)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 3:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sErrors (3)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 4:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sWarnings (4)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 5:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sNotifications (5)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 6:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sInformational (6)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case 7:
					fprintf(nipper->reportFile, "%sTerminal Line Logging Severity Level%sDebugging (7)%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
			}
		}
		else
			fprintf(nipper->reportFile, "%sTerminal Line Logging%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(nipper, false, settings_logging_table, &settings_general_heading);

		output_endsection(nipper, subsection_end);
	}

	// SNMP
	if (nipper->ios->snmp != 0)
	{
		output_parseTitle(settings_snmp_title, nipper, rate_none, section_config);
		output_parseText(settings_snmp_desc, nipper, section_none, rate_none, 0, 0);
		output_table(nipper, true, settings_snmp_table, &settings_general_heading);
		if (nipper->ios->snmp->enabled == true)
			fprintf(nipper->reportFile, "%sService enabled%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sService enabled%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->ios->snmp->contact[0] != 0)
		{
			outputFriendly(nipper->ios->snmp->contact, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sContact%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		if (nipper->ios->snmp->location[0] != 0)
		{
			outputFriendly(nipper->ios->snmp->location, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sLocation%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		fprintf(nipper->reportFile, "%sTrap Timeout%s%d seconds%s", tablerow_start, tablerow_mid, nipper->ios->snmp->trapTimeout, tablerow_end);
		if (nipper->ios->snmp->trapSource[0] != 0)
		{
			outputFriendly(nipper->ios->snmp->trapSource, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sTrap Source Interface%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		if (nipper->ios->snmp->tftpServerList[0] == 0)
			fprintf(nipper->reportFile, "%sTFTP Server List%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
		{
			outputFriendly(nipper->ios->snmp->tftpServerList, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sTFTP Server List%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		output_table(nipper, false, settings_snmp_table, &settings_general_heading);
		addAbbreviation("SNMP", false);
		addAbbreviation("TFTP", false);

		if (nipper->ios->snmp->community != 0)
		{
			output_table(nipper, true, settings_snmpstring_table, &settings_snmpstring_heading);
			communityPointer = nipper->ios->snmp->community;
			while (communityPointer != 0)
			{
				fprintf(nipper->reportFile, "%s", tablerow_start);
				if (nipper->passwords == true)
				{
					outputFriendly(communityPointer->community, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
				}
				if (communityPointer->readonly == true)
					fprintf(nipper->reportFile, "Read-Only%s%s%s%s%s", tablerow_mid, communityPointer->viewName, tablerow_mid, communityPointer->accessList, tablerow_mid);
				else
					fprintf(nipper->reportFile, "Read/Write%s%s%s%s%s", tablerow_mid, communityPointer->viewName, tablerow_mid, communityPointer->accessList, tablerow_mid);
				if (communityPointer->enabled == true)
					fprintf(nipper->reportFile, "Yes%s", tablerow_end);
				else
					fprintf(nipper->reportFile, "No%s", tablerow_end);
				communityPointer = communityPointer->next;
			}
			output_table(nipper, false, settings_snmpstring_table, &settings_snmpstring_heading);
		}

		if (nipper->ios->snmp->group != 0)
		{
			// Groups
			snmpGroupPointer = nipper->ios->snmp->group;
			output_table(nipper, true, settings_snmpgroups_table, &settings_snmpgroups_heading);
			while (snmpGroupPointer != 0)
			{
				outputFriendly(snmpGroupPointer->groupName, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				switch(snmpGroupPointer->version)
				{
					case snmp1:
						fprintf(nipper->reportFile, "1");
						break;
					case snmp2c:
						fprintf(nipper->reportFile, "2c");
						break;
					case snmp3_no_auth:
						fprintf(nipper->reportFile, "3 (No Auth)");
						break;
					case snmp3_auth:
						fprintf(nipper->reportFile, "3 (Auth)");
						break;
					case snmp3_priv:
						fprintf(nipper->reportFile, "3 (Auth and Priv)");
						break;
				}
				outputFriendly(snmpGroupPointer->readView, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_mid);
				if (snmpGroupPointer->writeView[0] != 0)
				{
					outputFriendly(snmpGroupPointer->writeView, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
				}
				else
					fprintf(nipper->reportFile, " %s", tablerow_mid);
				if (snmpGroupPointer->notifyView[0] != 0)
				{
					outputFriendly(snmpGroupPointer->notifyView, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
				}
				else
					fprintf(nipper->reportFile, " %s", tablerow_mid);
				if (snmpGroupPointer->context[0] != 0)
				{
					outputFriendly(snmpGroupPointer->context, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
				}
				else
					fprintf(nipper->reportFile, " %s", tablerow_mid);
				if (snmpGroupPointer->acl[0] != 0)
				{
					outputFriendly(snmpGroupPointer->acl, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
				}
				else
					fprintf(nipper->reportFile, " %s", tablerow_end);
				snmpGroupPointer = snmpGroupPointer->next;
			}
			output_table(nipper, false, settings_snmpgroups_table, &settings_snmpgroups_heading);

			// Group Users
			snmpGroupPointer = nipper->ios->snmp->group;
			while (snmpGroupPointer != 0)
			{
				if (snmpGroupPointer->user != 0)
				{
					switch (snmpGroupPointer->version)
					{
						case snmp1:
						case snmp2c:
						case snmp3_no_auth:
							if (snmpGroupPointer->version == snmp1)
								sprintf(tempString, "%s%s", settings_snmpgroup_table1, snmpGroupPointer->groupName);
							else if (snmpGroupPointer->version == snmp2c)
								sprintf(tempString, "%s%s", settings_snmpgroup_table2, snmpGroupPointer->groupName);
							else if (snmpGroupPointer->version == snmp3_no_auth)
								sprintf(tempString, "%s%s", settings_snmpgroup_table3, snmpGroupPointer->groupName);
							outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
							output_table(nipper, true, tempString2, &settings_snmpnoauth_heading);
							snmpUserPointer = snmpGroupPointer->user;
							while (snmpUserPointer != 0)
							{
								outputFriendly(snmpUserPointer->userName, tempString, nipper->maxSize, nipper->outputFormat);
								fprintf(nipper->reportFile, "%s%s", tablerow_start, tempString);
								if (snmpUserPointer->acl[0] == 0)
									fprintf(nipper->reportFile, " %s", tablerow_end);
								else
								{
									outputFriendly(snmpUserPointer->acl, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
								}
								snmpUserPointer = snmpUserPointer->next;
							}
							output_table(nipper, false, tempString2, &settings_snmpnoauth_heading);
							break;
						case snmp3_auth:
							sprintf(tempString, "%s%s", settings_snmpgroup_table4, snmpGroupPointer->groupName);
							outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
							output_table(nipper, true, tempString2, &settings_snmpauth_heading);
							snmpUserPointer = snmpGroupPointer->user;
							while (snmpUserPointer != 0)
							{
								outputFriendly(snmpUserPointer->userName, tempString, nipper->maxSize, nipper->outputFormat);
								fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
								if (nipper->passwords == true)
								{
									outputFriendly(snmpUserPointer->authPass, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								switch (snmpUserPointer->snmp3auth)
								{
									case snmp3_md5:
										fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
										addAbbreviation("MD5", false);
										break;
									case snmp3_sha:
										fprintf(nipper->reportFile, "SHA%s", tablerow_mid);
										addAbbreviation("SHA", false);
										break;
								}
								if (snmpUserPointer->acl[0] == 0)
									fprintf(nipper->reportFile, " %s", tablerow_end);
								else
								{
									outputFriendly(snmpUserPointer->acl, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
								}
								snmpUserPointer = snmpUserPointer->next;
							}
							output_table(nipper, false, tempString2, &settings_snmpauth_heading);
							break;
						case snmp3_priv:
							sprintf(tempString, "%s%s", settings_snmpgroup_table4, snmpGroupPointer->groupName);
							outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
							output_table(nipper, true, tempString2, &settings_snmpauth_heading);
							snmpUserPointer = snmpGroupPointer->user;
							while (snmpUserPointer != 0)
							{
								outputFriendly(snmpUserPointer->userName, tempString, nipper->maxSize, nipper->outputFormat);
								fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
								if (nipper->passwords == true)
								{
									outputFriendly(snmpUserPointer->authPass, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								switch (snmpUserPointer->snmp3auth)
								{
									case snmp3_md5:
										fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
										addAbbreviation("MD5", false);
										break;
									case snmp3_sha:
										fprintf(nipper->reportFile, "SHA%s", tablerow_mid);
										addAbbreviation("SHA", false);
										break;
								}
								if (nipper->passwords == true)
								{
									outputFriendly(snmpUserPointer->privPass, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
								}
								switch (snmpUserPointer->snmp3priv)
								{
									case snmp3_des:
										fprintf(nipper->reportFile, "DES%s", tablerow_mid);
										addAbbreviation("DES", false);
										break;
									case snmp3_3des:
										fprintf(nipper->reportFile, "3DES%s", tablerow_mid);
										addAbbreviation("3DES", false);
										break;
									case snmp3_des56:
										fprintf(nipper->reportFile, "DES 56%s", tablerow_mid);
										addAbbreviation("DES", false);
										break;
									case snmp3_aes128:
										fprintf(nipper->reportFile, "AES 128%s", tablerow_mid);
										addAbbreviation("AES", false);
										break;
									case snmp3_aes192:
										fprintf(nipper->reportFile, "AES 192%s", tablerow_mid);
										addAbbreviation("AES", false);
										break;
									case snmp3_aes256:
										fprintf(nipper->reportFile, "AES 256%s", tablerow_mid);
										addAbbreviation("AES", false);
										break;
								}
								if (snmpUserPointer->acl[0] == 0)
									fprintf(nipper->reportFile, " %s", tablerow_end);
								else
								{
									outputFriendly(snmpUserPointer->acl, tempString, nipper->maxSize, nipper->outputFormat);
									fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
								}
								snmpUserPointer = snmpUserPointer->next;
							}
							output_table(nipper, false, tempString2, &settings_snmpauth_heading);
							break;
					}
				}
				snmpGroupPointer = snmpGroupPointer->next;
			}
			addAbbreviation("OID", false);
			addAbbreviation("MIB", false);
		}

		// Views
		if (nipper->ios->snmp->view != 0)
		{
			snmpViewPointer = nipper->ios->snmp->view;
			while (snmpViewPointer != 0)
			{
				sprintf(tempString, "%s %s", settings_snmpview_table, snmpViewPointer->viewName);
				outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
				output_table(nipper, true, tempString2, &settings_snmpview_heading);
				snmpViewItemPointer = snmpViewPointer->viewItem;
				while (snmpViewItemPointer != 0)
				{
					fprintf(nipper->reportFile, "%s%s%s", tablerow_start, snmpViewItemPointer->mibId, tablerow_mid);
					if (snmpViewItemPointer->include == true)
						fprintf(nipper->reportFile, "Include%s", tablerow_end);
					else
						fprintf(nipper->reportFile, "Exclude%s", tablerow_end);
					snmpViewItemPointer = snmpViewItemPointer->next;
				}
				output_table(nipper, false, tempString2, &settings_snmpview_heading);
				snmpViewPointer = snmpViewPointer->next;
			}
			addAbbreviation("MIB", false);
		}

		if (nipper->ios->snmp->host != 0)
		{
			output_table(nipper, true, settings_snmphost_table, &settings_snmphost_heading);
			snmpHostPointer = nipper->ios->snmp->host;
			while (snmpHostPointer != 0)
			{
				outputFriendly(snmpHostPointer->host, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				switch(snmpHostPointer->version)
				{
					case snmp1:
						fprintf(nipper->reportFile, "1");
						break;
					case snmp2c:
						fprintf(nipper->reportFile, "2c");
						break;
					case snmp3_no_auth:
						fprintf(nipper->reportFile, "3 (No Auth)");
						break;
					case snmp3_auth:
						fprintf(nipper->reportFile, "3 (Auth)");
						break;
					case snmp3_priv:
						fprintf(nipper->reportFile, "3 (Auth and Priv)");
						break;
				}
				if (nipper->passwords == true)
				{
					outputFriendly(snmpHostPointer->community, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s ", tablerow_mid, tempString);
				}
				fprintf(nipper->reportFile, "%s", tablerow_end);
				snmpHostPointer = snmpHostPointer->next;
			}
			output_table(nipper, false, settings_snmphost_table, &settings_snmphost_heading);
		}

		if (nipper->ios->snmp->trap != 0)
		{
			output_table(nipper, true, settings_snmptrap_table, &settings_snmptrap_heading);
			snmpTrapPointer = nipper->ios->snmp->trap;
			while (snmpTrapPointer != 0)
			{
				outputFriendly(snmpTrapPointer->trap, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				snmpTrapPointer = snmpTrapPointer->next;
			}
			output_table(nipper, false, settings_snmptrap_table, &settings_snmptrap_heading);
		}

		output_endsection(nipper, subsection_end);
	}

	// SSH
	if (nipper->ios->ssh != 0)
	{
		output_parseTitle(settings_ssh_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_ssh_table, &settings_general_heading);
		if (nipper->ios->line != 0)
		{
			tempInt = 0;
			linePointer = nipper->ios->line;
			while (linePointer != 0)
			{
				if (linePointer->ssh == true)
					tempInt++;
				linePointer = linePointer->next;
			}
		}
		switch (tempInt)
		{
			case 0:
				fprintf(nipper->reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			default:
				fprintf(nipper->reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		switch (nipper->ios->ssh->protocolVersion)
		{
			case ssh_ver_1:
				fprintf(nipper->reportFile, "%sProtocol version%s1%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case ssh_ver_2:
				fprintf(nipper->reportFile, "%sProtocol version%s2%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			default:
				fprintf(nipper->reportFile, "%sProtocol version%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		if (nipper->ios->ssh->timeout > -1)
			fprintf(nipper->reportFile, "%sLogin time-out%s%d%s", tablerow_start, tablerow_mid, nipper->ios->ssh->timeout, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sLogin time-out%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->ios->ssh->retries > -1)
			fprintf(nipper->reportFile, "%sLogin retries%s%d%s", tablerow_start, tablerow_mid, nipper->ios->ssh->retries, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sLogin retries%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(nipper, false, settings_ssh_table, &settings_general_heading);
		addAbbreviation("SSH", false);

		// Show SSH interfaces
		if (nipper->ios->ssh->interface != 0)
		{
			output_table(nipper, true, settings_sshint_table, &settings_sshint_heading);
			fprintf(nipper->reportFile, "<table><caption>Table %d: %s.</caption><thead>\n<tr><th>Network Interface</th></tr></thead><tbody>\n", nipper->tableNum, settings_sshint_table);
			interfacePointer = nipper->ios->ssh->interface;
			while (interfacePointer != 0)
			{
				outputFriendly(interfacePointer->interface, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				interfacePointer = interfacePointer->next;
			}
			output_table(nipper, false, settings_sshint_table, &settings_sshint_heading);
		}

		output_endsection(nipper, subsection_end);
	}

	// NTP
	if (nipper->ios->ntp != 0)
	{
		output_parseTitle(settings_ntp_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_ntp_table, &settings_general_heading);
		if (nipper->ios->ntp->enabled == true)
			fprintf(nipper->reportFile, "%sNTP Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sNTP Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->ios->ntp->master == true)
			fprintf(nipper->reportFile, "%sNTP Master%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sNTP Master%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (nipper->ios->ntp->authentication == true)
			fprintf(nipper->reportFile, "%sNTP Authentication%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sNTP Authentication%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(nipper, false, settings_ntp_table, &settings_general_heading);
		addAbbreviation("NTP", false);

		if (nipper->ios->ntp->server != 0)
		{
			output_table(nipper, true, settings_ntpserv_table, &settings_ntpserv_heading);
			ntpHostPointer = nipper->ios->ntp->server;
			while (ntpHostPointer != 0)
			{
				outputFriendly(ntpHostPointer->ntpHost, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				if (ntpHostPointer->keyNum == -1)
					fprintf(nipper->reportFile, "None%s", tablerow_end);
				else
					fprintf(nipper->reportFile, "%d%s", ntpHostPointer->keyNum, tablerow_end);
				ntpHostPointer = ntpHostPointer->next;
			}
			output_table(nipper, false, settings_ntpserv_table, &settings_ntpserv_heading);
		}

		if (nipper->ios->ntp->peer != 0)
		{
			output_table(nipper, true, settings_ntppeer_table, &settings_ntppeer_heading);
			ntpHostPointer = nipper->ios->ntp->peer;
			while (ntpHostPointer != 0)
			{
				outputFriendly(ntpHostPointer->ntpHost, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, ntpHostPointer->keyNum, tablerow_end);
				ntpHostPointer = ntpHostPointer->next;
			}
			output_table(nipper, false, settings_ntppeer_table, &settings_ntppeer_heading);
		}

		if ((nipper->ios->ntp->key != 0) && (nipper->passwords == true))
		{
			output_table(nipper, true, settings_ntpkeys_table, &settings_ntpkeys_heading);
			ntpKeyPointer = nipper->ios->ntp->key;
			while (ntpKeyPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%d%s", tablerow_start, ntpKeyPointer->keyNum, tablerow_mid);
				if (ntpKeyPointer->trusted == false)
					fprintf(nipper->reportFile, "No");
				else
					fprintf(nipper->reportFile, "Yes");
				if (nipper->passwords == true)
				{
					outputFriendly(ntpKeyPointer->key, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tablerow_mid, tempString);
				}
				fprintf(nipper->reportFile, "%s", tablerow_end);
				ntpKeyPointer = ntpKeyPointer->next;
			}
			output_table(nipper, false, settings_ntpkeys_table, &settings_ntpkeys_heading);
		}

		if (nipper->ios->ntp->access != 0)
		{
			output_table(nipper, true, settings_ntpaccess_table, &settings_ntpaccess_heading);
			ntpGroupPointer = nipper->ios->ntp->access;
			while (ntpGroupPointer != 0)
			{
				switch (ntpGroupPointer->accessType)
				{
					case ntp_peer:
						fprintf(nipper->reportFile, "%sPeer%s%s%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
					case ntp_serve:
						fprintf(nipper->reportFile, "%sServe%s%s%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
					case ntp_serve_only:
						fprintf(nipper->reportFile, "%sServe-Only%s%s%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
					case ntp_query_only:
						fprintf(nipper->reportFile, "%sQuery-Only%s%s%s", tablerow_start, tablerow_mid, ntpGroupPointer->accessList, tablerow_end);
						break;
				}
				ntpGroupPointer = ntpGroupPointer->next;
			}
			output_table(nipper, false, settings_ntpaccess_table, &settings_ntpaccess_heading);
		}

		output_endsection(nipper, subsection_end);
	}

	// FTP
	if (nipper->ios->ftp != 0)
	{
		output_parseTitle(settings_ftp_title, nipper, rate_none, section_config);
		if (nipper->ios->ftp->username[0] != 0)
		{
			output_table(nipper, true, settings_ftp_table, &settings_general_heading);
			outputFriendly(nipper->ios->ftp->username, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sFTP Username%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			if (nipper->passwords == true)
			{
				outputFriendly(nipper->ios->ftp->password, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%sFTP Password%s%s %s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			}
			switch(nipper->ios->ftp->encryption)
			{
				case encrypt_type7:
					fprintf(nipper->reportFile, "%sPassword Encryption Type%sType-7%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case encrypt_md5:
					fprintf(nipper->reportFile, "%sPassword Encryption Type%sMD5%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case encrypt_none:
					fprintf(nipper->reportFile, "%sPassword Encryption Type%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
				case encrypt_unknown:
					fprintf(nipper->reportFile, "%sPassword Encryption Type%sUnknown%s", tablerow_start, tablerow_mid, tablerow_end);
					break;
			}
			output_table(nipper, false, settings_ftp_table, &settings_general_heading);
		}
		addAbbreviation("FTP", false);

		// Show FTP interfaces
		if (nipper->ios->ftp->interface != 0)
		{
			output_table(nipper, true, settings_ftpint_table, &settings_sshint_heading);
			interfacePointer = nipper->ios->ftp->interface;
			while (interfacePointer != 0)
			{
				outputFriendly(interfacePointer->interface, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				interfacePointer = interfacePointer->next;
			}
			output_table(nipper, false, settings_ftpint_table, &settings_sshint_heading);
		}

		output_endsection(nipper, subsection_end);
	}

	// HTTP
	if (nipper->ios->http != 0)
	{
		output_parseTitle(settings_http_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_http_table, &settings_general_heading);
		if (nipper->ios->http->enabled == true)
			fprintf(nipper->reportFile, "%sHTTP Server%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sHTTP Server%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		switch(nipper->ios->http->authentication)
		{
			case http_auth_enable:
				fprintf(nipper->reportFile, "%sAuthentication Type%sEnable Password%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case http_auth_aaa:
				fprintf(nipper->reportFile, "%sAuthentication Type%sAAA%s", tablerow_start, tablerow_mid, tablerow_end);
				addAbbreviation("AAA", false);
				break;
			case http_auth_tacacs:
				fprintf(nipper->reportFile, "%sAuthentication Type%sTACACS%s", tablerow_start, tablerow_mid, tablerow_end);
				addAbbreviation("TACACS", false);
				break;
			case http_auth_local:
				fprintf(nipper->reportFile, "%sAuthentication Type%sLocal Users%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		if (nipper->ios->http->accessClass[0] == 0)
			fprintf(nipper->reportFile, "%sAccess Class (Access List Number)%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(nipper->reportFile, "%sAccess Class (Access List Number)%s%s%sn", tablerow_start, tablerow_mid, nipper->ios->http->accessClass, tablerow_end);
		output_table(nipper, false, settings_http_table, &settings_general_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("HTTP", false);
	}

	// TACACS
	if (nipper->ios->tacacs != 0)
	{
		output_parseTitle(settings_tacacs_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_tacacs_table, &settings_general_heading);
		if (nipper->passwords == true)
		{
			outputFriendly(nipper->ios->tacacs->key, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sTACACS Key%s%s %s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		switch(nipper->ios->tacacs->encryption)
		{
			case encrypt_type7:
				fprintf(nipper->reportFile, "%sKey Encryption Type%sType-7%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case encrypt_md5:
				fprintf(nipper->reportFile, "%sKey Encryption Type%sMD5%s", tablerow_start, tablerow_mid, tablerow_end);
				addAbbreviation("MD5", false);
				break;
			case encrypt_none:
				fprintf(nipper->reportFile, "%sKey Encryption Type%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case encrypt_unknown:
				fprintf(nipper->reportFile, "%sKey Encryption Type%sUnknown%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		output_table(nipper, false, settings_tacacs_table, &settings_general_heading);
		addAbbreviation("TACACS", false);

		if (nipper->ios->tacacs->host != 0)
		{
			output_table(nipper, true, settings_tacacsserv_table, &settings_tacacsserv_heading);
			hostPointer = nipper->ios->tacacs->host;
			while (hostPointer != 0)
			{
				outputFriendly(hostPointer->server, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
				hostPointer = hostPointer->next;
			}
			output_table(nipper, false, settings_tacacsserv_table, &settings_tacacsserv_heading);
		}

		output_endsection(nipper, subsection_end);
	}

	// AAA
	if ((nipper->ios->aaa !=0) && (nipper->ios->aaa->groupServer !=0))
	{
		output_parseTitle(settings_aaa_title, nipper, rate_none, section_config);
		aaaGroupPointer = nipper->ios->aaa->groupServer;
		while (aaaGroupPointer != 0)
		{
			switch (aaaGroupPointer->type)
			{
				case aaa_radius:
					sprintf(tempString, "%s %s %s", settings_aaa_table1, aaaGroupPointer->username, settings_aaa_table2);
					addAbbreviation("RADIUS", false);
					break;
				case aaa_tacacs:
					sprintf(tempString, "%s %s %s", settings_aaa_table1, aaaGroupPointer->username, settings_aaa_table3);
					addAbbreviation("TACACS", false);
					break;
				case aaa_kerberos:
					sprintf(tempString, "%s %s %s", settings_aaa_table1, aaaGroupPointer->username, settings_aaa_table4);
					break;
			}
			outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
			output_table(nipper, true, tempString2, &settings_aaa_heading);
			aaaServerPointer = aaaGroupPointer->server;
			while (aaaServerPointer != 0)
			{
				outputFriendly(aaaServerPointer->server, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s%d%s%d%s", tablerow_start, tempString, tablerow_mid, aaaServerPointer->authPort, tablerow_mid, aaaServerPointer->acctPort, tablerow_mid);
				if (nipper->passwords == true)
				{
					outputFriendly(aaaServerPointer->key, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
				}
				switch(aaaServerPointer->encryption)
				{
					case encrypt_type7:
						fprintf(nipper->reportFile, "Type-7%s", tablerow_end);
						break;
					case encrypt_md5:
						fprintf(nipper->reportFile, "MD5%s", tablerow_end);
						addAbbreviation("MD5", false);
						break;
					case encrypt_none:
						fprintf(nipper->reportFile, "None%s", tablerow_end);
						break;
					case encrypt_unknown:
						fprintf(nipper->reportFile, "Unknown%s", tablerow_end);
						break;
				}
				aaaServerPointer = aaaServerPointer->next;
			}
			output_table(nipper, false, tempString2, &settings_aaa_heading);
			aaaGroupPointer = aaaGroupPointer->next;
		}
		output_endsection(nipper, subsection_end);
		addAbbreviation("AAA", false);
	}

	// Routing
	if ((nipper->ios->bgp != 0) || (vrrpIssues != 0) || (nipper->ios->eigrp != 0) || (nipper->ios->isis != 0) || (nipper->ios->rip != 0) || (nipper->ios->ospf != 0) || (nipper->ios->route != 0) || (nipper->ios->routeMap != 0))
	{
		output_parseTitle(settings_routing_title, nipper, rate_none, section_config);
		output_parseText(settings_routing_desc, nipper, section_none, rate_none, 0, 0);

		// Static Routes
		if (nipper->ios->route != 0)
		{
			output_table(nipper, true, settings_static_route_table, &settings_routeIOS_heading);
			routePointer = nipper->ios->route;
			while (routePointer != 0)
			{
				outputFriendly(routePointer->ipAddress, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				outputFriendly(routePointer->netMask, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
				outputFriendly(routePointer->gateway, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
				routePointer = routePointer->next;
			}
			output_table(nipper, false, settings_static_route_table, &settings_routeIOS_heading);
		}

		// Route Maps
		if (nipper->ios->routeMap != 0)
		{
			output_parseText(settings_routemap_desc, nipper, section_none, rate_none, 0, 0);
			output_table(nipper, true, settings_routemap_table, &settings_routeMap_heading);
			routeMapPointer = nipper->ios->routeMap;
			while (routeMapPointer != 0)
			{
				outputFriendly(routeMapPointer->mapTag, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				if (routeMapPointer->sequenceNo == 0)
					fprintf(nipper->reportFile, " %s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "%d%s", routeMapPointer->sequenceNo, tablerow_mid);
				if (routeMapPointer->permission == perm_permit)
					fprintf(nipper->reportFile, "Permit%s", tablerow_end);
				else
					fprintf(nipper->reportFile, "Deny%s", tablerow_end);
				routeMapPointer = routeMapPointer->next;
			}
			output_table(nipper, false, settings_routemap_table, &settings_routeMap_heading);
		}

		// VRRP
		if (vrrpIssues != 0)
		{
			output_parseText(settings_vrrp_desc, nipper, section_none, rate_none, 0, 0);
			output_table(nipper, true, settings_vrrp_table, &settings_vrrp_interface_heading);
			interfaceConfigPointer = nipper->ios->interface;
			while (interfaceConfigPointer != 0)
			{
				vrrpPointer = interfaceConfigPointer->vrrp;
				while (vrrpPointer != 0)
				{
					outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s%s%d%s", tablerow_start, tempString, tablerow_mid, vrrpPointer->group, tablerow_mid);
					if (vrrpPointer->shutdown == false)
						fprintf(nipper->reportFile, "Active%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "Off%s", tablerow_mid);
					outputFriendly(vrrpPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					fprintf(nipper->reportFile, "%s%s", vrrpPointer->ipAddress, tablerow_mid);
					if (vrrpPointer->ipAddress2[0] == 0)
						fprintf(nipper->reportFile, " %s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "%s%s", vrrpPointer->ipAddress2, tablerow_mid);
					fprintf(nipper->reportFile, "%d%s", vrrpPointer->priorityLevel, tablerow_mid);
					switch (vrrpPointer->authentication)
					{
						case vrrp_auth_none:
							fprintf(nipper->reportFile, "None%s", tablerow_mid);
							break;
						case vrrp_auth_text:
							fprintf(nipper->reportFile, "Clear Text%s", tablerow_mid);
							break;
						case vrrp_auth_md5:
							if (vrrpPointer->keyChain == false)
								fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
							else
								fprintf(nipper->reportFile, "MD5 Key%s", tablerow_mid);
							break;
					}
					if (nipper->passwords == true)
					{
						if (vrrpPointer->authString[0] == 0)
							fprintf(nipper->reportFile, " %s", tablerow_mid);
						else
						{
							outputFriendly(vrrpPointer->authString, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						}
					}
					if (interfaceConfigPointer->shutdown == true)
						fprintf(nipper->reportFile, "No%s", tablerow_end);
					else
						fprintf(nipper->reportFile, "Yes%s", tablerow_end);
					vrrpPointer = vrrpPointer->next;
				}
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			output_table(nipper, false, settings_vrrp_table, &settings_vrrp_interface_heading);
		}

		// RIP
		if (nipper->ios->rip != 0)
		{
			output_parseText(settings_rip_desc, nipper, section_none, rate_none, 0, 0);
			output_table(nipper, true, settings_rip_table, &settings_general_heading);
			if (nipper->ios->rip->version == rip_version1)
				fprintf(nipper->reportFile, "%sRIP Version%s1%s", tablerow_start, tablerow_mid, tablerow_end);
			else if (nipper->ios->rip->version == rip_version2)
				fprintf(nipper->reportFile, "%sRIP Version%s2%s", tablerow_start, tablerow_mid, tablerow_end);
			else if (nipper->ios->rip->version == rip_version1and2)
				fprintf(nipper->reportFile, "%sRIP Version%s1 and 2%s", tablerow_start, tablerow_mid, tablerow_end);
			if (nipper->ios->rip->autoSummary == true)
				fprintf(nipper->reportFile, "%sRoute Auto Summarisation%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sRoute Auto Summarisation%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			if (nipper->ios->rip->defaultRoute == true)
				fprintf(nipper->reportFile, "%sDefault Route Generation%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sDefault Route Generation%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			if (nipper->ios->rip->defaultRouteMap[0] != 0)
			{
				outputFriendly(nipper->ios->rip->defaultRouteMap, tempString, nipper->maxSize, nipper->outputFormat);
				fprintf(nipper->reportFile, "%sDefault Route Route-Map%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
			}
			if (nipper->ios->rip->defaultMetric == -1)
				fprintf(nipper->reportFile, "%sDefault Metric%sAutomatic%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sDefault Metric%s%d%s", tablerow_start, tablerow_mid, nipper->ios->rip->defaultMetric, tablerow_end);
			fprintf(nipper->reportFile, "%sInput Queue Depth%s%d%s", tablerow_start, tablerow_mid, nipper->ios->rip->inputQueue, tablerow_end);
			fprintf(nipper->reportFile, "%sRIP Update Delay%s%d milliseconds%s", tablerow_start, tablerow_mid, nipper->ios->rip->outputDelay, tablerow_end);
			if (nipper->ios->rip->validateUpdateSource == true)
				fprintf(nipper->reportFile, "%sValidate Update Source%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
			else
				fprintf(nipper->reportFile, "%sValidate Update Source%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
			output_table(nipper, false, settings_rip_table, &settings_general_heading);

			// RIP Interfaces
			if (nipper->ios->interface !=0)
			{
				output_table(nipper, true, settings_rip_interface_table, &settings_rip_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					// Check interface is valid for OSPF
					if ((interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (interfaceConfigPointer->ripPassive == false))
					{
						outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						outputFriendly(interfaceConfigPointer->ipAddress, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						switch (interfaceConfigPointer->ripAuthMode)
						{
							case rip_auth_none:
								fprintf(nipper->reportFile, "None%s", tablerow_mid);
								break;
							case rip_auth_text:
								fprintf(nipper->reportFile, "Clear Text%s", tablerow_mid);
								break;
							case rip_auth_md5:
								fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
								break;
							default:
								fprintf(nipper->reportFile, " %s", tablerow_mid);
								break;
						}
						if (interfaceConfigPointer->ripKeyChain[0] == 0)
							fprintf(nipper->reportFile, " %s", tablerow_mid);
						else
						{
							outputFriendly(interfaceConfigPointer->ripKeyChain, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						}
						switch (interfaceConfigPointer->ripSendVersion)
						{
							case rip_version1:
								fprintf(nipper->reportFile, "1%s", tablerow_mid);
								break;
							case rip_version2:
								fprintf(nipper->reportFile, "2%s", tablerow_mid);
								break;
							case rip_version1and2:
								fprintf(nipper->reportFile, "1 and 2%s", tablerow_mid);
								break;
							case rip_version_global:
								switch (nipper->ios->rip->version)
								{
									case rip_version1:
										fprintf(nipper->reportFile, "1%s", tablerow_mid);
										break;
									case rip_version2:
										fprintf(nipper->reportFile, "2%s", tablerow_mid);
										break;
									case rip_version1and2:
										fprintf(nipper->reportFile, "1 and 2%s", tablerow_mid);
										break;
								}
								break;
						}
						switch (interfaceConfigPointer->ripReceiveVersion)
						{
							case rip_version1:
								fprintf(nipper->reportFile, "1%s", tablerow_mid);
								break;
							case rip_version2:
								fprintf(nipper->reportFile, "2%s", tablerow_mid);
								break;
							case rip_version1and2:
								fprintf(nipper->reportFile, "1 and 2%s", tablerow_mid);
								break;
							case rip_version_global:
								switch (nipper->ios->rip->version)
								{
									case rip_version1:
										fprintf(nipper->reportFile, "1%s", tablerow_mid);
										break;
									case rip_version2:
										fprintf(nipper->reportFile, "2%s", tablerow_mid);
										break;
									case rip_version1and2:
										fprintf(nipper->reportFile, "1 and 2%s", tablerow_mid);
										break;
								}
								break;
						}
						switch (interfaceConfigPointer->ripV2Broadcast)
						{
							case true:
								fprintf(nipper->reportFile, "On%s", tablerow_mid);
								break;
							case false:
								fprintf(nipper->reportFile, "Off%s", tablerow_mid);
								break;
						}
						switch (interfaceConfigPointer->ripTriggered)
						{
							case true:
								fprintf(nipper->reportFile, "On%s", tablerow_end);
								break;
							case false:
								fprintf(nipper->reportFile, "Off%s", tablerow_end);
								break;
						}
					}
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, settings_rip_interface_table, &settings_rip_interface_heading);
			}

			if (nipper->ios->rip->neighbor != 0)
			{
				hostPointer = nipper->ios->rip->neighbor;
				output_table(nipper, true, settings_rip_neightbor_table, &settings_rip_neightbor_heading);
				while (hostPointer != 0)
				{
					outputFriendly(hostPointer->server, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
					hostPointer = hostPointer->next;
				}
				output_table(nipper, false, settings_rip_neightbor_table, &settings_rip_neightbor_heading);
			}

			if (nipper->ios->rip->network != 0)
			{
				hostPointer = nipper->ios->rip->network;
				output_table(nipper, true, settings_rip_networks_table, &settings_rip_networks_heading);
				while (hostPointer != 0)
				{
					outputFriendly(hostPointer->server, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_end);
					hostPointer = hostPointer->next;
				}
				output_table(nipper, false, settings_rip_networks_table, &settings_rip_networks_heading);
			}
			addAbbreviation("RIP", false);
		}

		// EIGRP
		if (nipper->ios->eigrp != 0)
		{
			output_parseText(settings_eigrp_desc, nipper, section_none, rate_none, 0, 0);

			// EIGRP Configuration
			eigrpPointer = nipper->ios->eigrp;
			while (eigrpPointer != 0)
			{
				sprintf(tempString, "EIGRP autonomous number %d configuration", eigrpPointer->process);
				output_table(nipper, true, tempString, &settings_general_heading);
				fprintf(nipper->reportFile, "%sRouter ID%s%s%s", tablerow_start, tablerow_mid, eigrpPointer->routerId, tablerow_end);
				fprintf(nipper->reportFile, "%sMaximum Hops%s%d%s", tablerow_start, tablerow_mid, eigrpPointer->maximumHops, tablerow_end);
				switch (eigrpPointer->stub)
				{
					case eigrp_stub_disabled:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case eigrp_stub_enabled:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case eigrp_stub_receive:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sReceive Only%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case eigrp_stub_connected:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sAdvertise Connected Routes%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case eigrp_stub_static:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sAdvertise Static Routes%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case eigrp_stub_summary:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sAdvertise Summary Routes%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
					case eigrp_stub_redist:
						fprintf(nipper->reportFile, "%sEIGRP Stub Router%sAdvertise Redistributed Routes%s", tablerow_start, tablerow_mid, tablerow_end);
						break;
				}
				if (eigrpPointer->autoSummary == true)
					fprintf(nipper->reportFile, "%sAuto Summary%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				else
					fprintf(nipper->reportFile, "%sAuto Summary%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				fprintf(nipper->reportFile, "%sInternal Administrative Distance%s%d%s", tablerow_start, tablerow_mid, eigrpPointer->internalDistance, tablerow_end);
				fprintf(nipper->reportFile, "%sExternal Administrative Distance%s%d%s", tablerow_start, tablerow_mid, eigrpPointer->externalDistance, tablerow_end);
				if (eigrpPointer->logNeighborChanges == true)
					fprintf(nipper->reportFile, "%sLog Neighbor Changes%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				else
					fprintf(nipper->reportFile, "%sLog Neighbor Changes%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				if (eigrpPointer->logNeighborWarnings == true)
					fprintf(nipper->reportFile, "%sLog Neighbor Warnings%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
				else
					fprintf(nipper->reportFile, "%sLog Neighbor Warnings%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
				eigrpNetworkPointer = eigrpPointer->network;
				while (eigrpNetworkPointer != 0)
				{
					fprintf(nipper->reportFile, "%sNetwork IP Address%s%s%s", tablerow_start, tablerow_mid, eigrpNetworkPointer->ipAddress, tablerow_end);
					if (eigrpNetworkPointer->netMask[0] != 0)
						fprintf(nipper->reportFile, "%sNetwork Netmask%s%s%s", tablerow_start, tablerow_mid, eigrpNetworkPointer->netMask, tablerow_end);
					eigrpNetworkPointer = eigrpNetworkPointer->next;
				}
				eigrpNeighborPointer = eigrpPointer->neighbor;
				while (eigrpNeighborPointer != 0)
				{
					fprintf(nipper->reportFile, "%sNetwork IP Address%s%s%s", tablerow_start, tablerow_mid, eigrpNeighborPointer->ipAddress, tablerow_end);
					if (eigrpNeighborPointer->interface[0] != 0)
						fprintf(nipper->reportFile, "%sNetwork Interface%s%s%s", tablerow_start, tablerow_mid, eigrpNeighborPointer->interface, tablerow_end);
					eigrpNeighborPointer = eigrpNeighborPointer->next;
				}
				output_table(nipper, false, tempString, &settings_general_heading);

				// Interfaces
				sprintf(tempString2, "EIGRP autonomous number %d interface configuration", eigrpPointer->process);
				output_table(nipper, true, tempString2, &settings_eigrp_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					eigrpAutonPointer = eigrpInterfaceConfig(interfaceConfigPointer, eigrpPointer->process);
					outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
					outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					outputFriendly(interfaceConfigPointer->ipAddress, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					if (interfaceConfigPointer->shutdown == true)
						fprintf(nipper->reportFile, "No%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
					if (eigrpAutonPointer->passive == true)
						fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "No%s", tablerow_mid);
					if (eigrpAutonPointer->md5Auth == true)
						fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "No%s", tablerow_mid);
					if (eigrpAutonPointer->keyChain[0] == 0)
						fprintf(nipper->reportFile, " %s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "%s%s", eigrpAutonPointer->keyChain, tablerow_mid);
					fprintf(nipper->reportFile, "%d%%%s", eigrpAutonPointer->bandwidth, tablerow_end);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, tempString2, &settings_eigrp_interface_heading);

				// Increment EIGRP Autonomous No.
				eigrpPointer = eigrpPointer->next;
			}
		}

		// BGP
		if (nipper->ios->bgp != 0)
		{
			output_parseText(settings_bgp_desc, nipper, section_none, rate_none, 0, 0);
			bgpPointer = nipper->ios->bgp;
			while (bgpPointer != 0)
			{
				sprintf(tempString, "BGP autonomous number %d configuration", bgpPointer->autonomousNo);
				output_table(nipper, true, tempString, &settings_general_heading);
				fprintf(nipper->reportFile, "%sAutonomous Number%s%d%s", tablerow_start, tablerow_mid, nipper->ios->bgp->autonomousNo, tablerow_end);
				if (bgpPointer->logNeighborChanges == true)
					fprintf(nipper->reportFile, "%sLog Neighbor Changes%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
				else
					fprintf(nipper->reportFile, "%sLog Neighbor Changes%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
				if (bgpPointer->dampening == true)
					fprintf(nipper->reportFile, "%sRoute Flap Dampening%sOn%s", tablerow_start, tablerow_mid, tablerow_end);
				else
					fprintf(nipper->reportFile, "%sRoute Flap Dampening%sOff%s", tablerow_start, tablerow_mid, tablerow_end);
				output_table(nipper, false, tempString, &settings_general_heading);
	
				if (bgpPointer->neighbor != 0)
				{
					sprintf(tempString2, "BGP autonomous number %d neighbors", bgpPointer->autonomousNo);
					output_table(nipper, true, tempString2, &settings_bgp_heading);
					neighborPointer = bgpPointer->neighbor;
					while (neighborPointer != 0)
					{
						outputFriendly(neighborPointer->host, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
						outputFriendly(neighborPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						if (nipper->passwords == true)
						{
							outputFriendly(neighborPointer->password, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
						}
						switch (neighborPointer->encryption)
						{
							case encrypt_type7:
								fprintf(nipper->reportFile, "Type-7%s", tablerow_mid);
								break;
							case encrypt_md5:
								fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
								addAbbreviation("MD5", false);
								break;
							case encrypt_none:
								if (neighborPointer->password[0] == 0)
									fprintf(nipper->reportFile, " %s", tablerow_mid);
								else
									fprintf(nipper->reportFile, "None%s", tablerow_mid);
								break;
							case encrypt_unknown:
								fprintf(nipper->reportFile, "Unknown%s", tablerow_mid);
								break;
						}
						if (neighborPointer->autonomousNo != -1)
							fprintf(nipper->reportFile, "%d%s", neighborPointer->autonomousNo, tablerow_mid);
						else
							fprintf(nipper->reportFile, " %s", tablerow_mid);
						if (neighborPointer->ttlHops == -1)
							fprintf(nipper->reportFile, "Off%s", tablerow_end);
						else
							fprintf(nipper->reportFile, "%d%s", neighborPointer->ttlHops, tablerow_end);
						neighborPointer = neighborPointer->next;
					}
					output_table(nipper, false, tempString2, &settings_bgp_heading);
				}

				// Interfaces
				sprintf(tempString2, "BGP autonomous number %d interface configuration", bgpPointer->autonomousNo);
				output_table(nipper, true, tempString2, &settings_bgp_interface_heading);
				interfaceConfigPointer = nipper->ios->interface;
				while (interfaceConfigPointer != 0)
				{
					bgpInterPointer = bgpInterfaceConfig(interfaceConfigPointer, bgpPointer->autonomousNo);
					outputFriendly(interfaceConfigPointer->name, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
					outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					outputFriendly(interfaceConfigPointer->ipAddress, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					if (interfaceConfigPointer->shutdown == true)
						fprintf(nipper->reportFile, "No%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
					if (bgpInterPointer->passive == true)
						fprintf(nipper->reportFile, "Yes%s", tablerow_end);
					else
						fprintf(nipper->reportFile, "No%s", tablerow_end);
					interfaceConfigPointer = interfaceConfigPointer->next;
				}
				output_table(nipper, false, tempString2, &settings_bgp_interface_heading);

				bgpPointer = bgpPointer->next;
			}
			addAbbreviation("BGP", false);
			addAbbreviation("TTL", false);
		}

		// OSPF
		if (nipper->ios->ospf != 0)
		{
			output_parseText(settings_ospf_desc, nipper, section_none, rate_none, 0, 0);

			ospfPointer = nipper->ios->ospf;
			while (ospfPointer != 0)
			{
				// OSPF Network
				if (nipper->ios->ospf->ospfNetwork != 0)
				{
					sprintf(tempString2, "OSPF process ID %d network areas", ospfPointer->processID);
					output_table(nipper, true, tempString2, &settings_ospf_network_heading);
					ospfNetworkPointer = nipper->ios->ospf->ospfNetwork;
					while (ospfNetworkPointer != 0)
					{
						fprintf(nipper->reportFile, "%s%s%s%s%s%s%s", tablerow_start, ospfNetworkPointer->network, tablerow_mid, ospfNetworkPointer->networkMask, tablerow_mid, ospfNetworkPointer->areaID, tablerow_end);
						ospfNetworkPointer = ospfNetworkPointer->next;
					}
					output_table(nipper, false, tempString2, &settings_ospf_network_heading);
				}
	
				// OSPF Areas
				if (nipper->ios->ospf->ospfArea != 0)
				{
					sprintf(tempString2, "OSPF process ID %d area configuration", ospfPointer->processID);
					output_table(nipper, true, tempString2, &settings_ospf_area_heading);
					ospfAreaPointer = nipper->ios->ospf->ospfArea;
					while (ospfAreaPointer != 0)
					{
						switch (ospfAreaPointer->areaAuth)
						{
							case auth_none:
								fprintf(nipper->reportFile, "%s%s%s%d%sNone%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, ospfAreaPointer->defaultCost, tablerow_mid, tablerow_end);
								break;
							case auth_clear:
								fprintf(nipper->reportFile, "%s%s%s%d%sClear-text%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, ospfAreaPointer->defaultCost, tablerow_mid, tablerow_end);
								break;
							case auth_md5:
								fprintf(nipper->reportFile, "%s%s%s%d%sMD5%s", tablerow_start, ospfAreaPointer->areaID, tablerow_mid, ospfAreaPointer->defaultCost, tablerow_mid, tablerow_end);
								break;
						}
						ospfAreaPointer = ospfAreaPointer->next;
					}
					output_table(nipper, false, tempString2, &settings_ospf_area_heading);
				}
	
				// OSPF Interfaces
				if (nipper->ios->interface !=0)
				{
					sprintf(tempString2, "OSPF process ID %d interface configuration", ospfPointer->processID);
					output_table(nipper, true, tempString2, &settings_ospf_interface_heading);
					interfaceConfigPointer = nipper->ios->interface;
					while (interfaceConfigPointer != 0)
					{
						// Check interface is valid for OSPF
						ospfInterPointer = ospfInterfaceConfig(interfaceConfigPointer, ospfPointer->processID);
						if ((interfaceConfigPointer->shutdown == false) && (strcmp(interfaceConfigPointer->ipAddress, "None") != 0) && (strncmp(interfaceConfigPointer->name, "Loopback", 8) != 0) && (ospfInterPointer->passive == false) && (ospfInterPointer->processID == ospfPointer->processID))
						{
							outputFriendly(interfaceConfigPointer->description, tempString, nipper->maxSize, nipper->outputFormat);
							fprintf(nipper->reportFile, "%s%s%s%s%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid, tempString, tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
							switch (interfaceConfigPointer->ospfAuthentication)
							{
								case auth_none:
									fprintf(nipper->reportFile, "None%s", tablerow_mid);
									break;
								case auth_clear:
									fprintf(nipper->reportFile, "Clear-Text%s", tablerow_mid);
									break;
								case auth_md5:
									fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
									addAbbreviation("MD5", false);
									break;
							}
							if (nipper->passwords == true)
							{
								outputFriendly(interfaceConfigPointer->ospfKey, tempString, nipper->maxSize, nipper->outputFormat);
								fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
							}
							switch (interfaceConfigPointer->ospfKeyEncryption)
							{
								case encrypt_type7:
									fprintf(nipper->reportFile, "Type-7%s", tablerow_mid);
									break;
								case encrypt_md5:
									fprintf(nipper->reportFile, "MD5%s", tablerow_mid);
									addAbbreviation("MD5", false);
									break;
								case encrypt_none:
									fprintf(nipper->reportFile, "None%s", tablerow_mid);
									break;
								case encrypt_unknown:
									fprintf(nipper->reportFile, "Unknown%s", tablerow_mid);
									break;
							}
							switch (interfaceConfigPointer->ospfFloodReduction)
							{
								case true:
									fprintf(nipper->reportFile, "On%s", tablerow_mid);
									break;
								case false:
									fprintf(nipper->reportFile, "Off%s", tablerow_mid);
									break;
								case unconfigured:
									fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
									break;
							}
							switch (interfaceConfigPointer->ospfNetwork)
							{
								case ospf_default:
									fprintf(nipper->reportFile, "Default");
									break;
								case ospf_broadcast:
									fprintf(nipper->reportFile, "Broadcast");
									break;
								case ospf_nonbroadcast:
									fprintf(nipper->reportFile, "Non-Broadcast");
									break;
								case ospf_p2p:
									fprintf(nipper->reportFile, "Point to Point");
									addAbbreviation("PPP", false);
									break;
								case ospf_p2multi:
									fprintf(nipper->reportFile, "Point to Multipoint");
									break;
								case ospf_p2multinon:
									fprintf(nipper->reportFile, "Point to Multipoint (Non-Broadcast)");
									break;
							}
							fprintf(nipper->reportFile, "%s", tablerow_end);
						}
						interfaceConfigPointer = interfaceConfigPointer->next;
					}
					output_table(nipper, false, tempString2, &settings_ospf_interface_heading);
				}
	
				// OSPF Neighbors
				if (nipper->ios->ospf->neighbor != 0)
				{
					sprintf(tempString2, "OSPF process ID %d neighbor configuration", ospfPointer->processID);
					output_table(nipper, true, tempString2, &settings_ospf_neighbor_heading);
					ospfNeighborPointer = nipper->ios->ospf->neighbor;
					while (ospfNeighborPointer != 0)
					{
						fprintf(nipper->reportFile, "%s%s%s%d%s%d%s%d%s", tablerow_start, ospfNeighborPointer->host, tablerow_mid, ospfNeighborPointer->priority, tablerow_mid, ospfNeighborPointer->pollInterval, tablerow_mid, ospfNeighborPointer->costNumber, tablerow_mid);
						if (ospfNeighborPointer->databaseFilter == false)
							fprintf(nipper->reportFile, "No%s", tablerow_end);
						else
							fprintf(nipper->reportFile, "Yes%s", tablerow_end);
						ospfNeighborPointer = ospfNeighborPointer->next;
					}
					output_table(nipper, false, tempString2, &settings_ospf_neighbor_heading);
				}

				ospfPointer = ospfPointer->next;
			}
		}

		output_endsection(nipper, subsection_end);
	}

	// Key Chains and Keys
	if ((nipper->passwords == true) && (nipper->ios->chain != 0))
	{
		output_parseTitle(settings_keys_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_keys_table, &settings_keys_heading);
		keyChainPointer = nipper->ios->chain;
		while (keyChainPointer != 0)
		{
			keyPointer = keyChainPointer->key;
			while (keyPointer != 0)
			{
				fprintf(nipper->reportFile, "%s%s%s%d", tablerow_start, keyChainPointer->name, tablerow_mid, keyPointer->keyNumber);
				if (nipper->passwords == true)
				{
					outputFriendly(keyPointer->key, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tablerow_mid, tempString);
				}
				fprintf(nipper->reportFile, "%s", tablerow_end);
				keyPointer = keyPointer->next;
			}
			keyChainPointer = keyChainPointer->next;
		}
		output_table(nipper, false, settings_keys_table, &settings_keys_heading);
		output_endsection(nipper, subsection_end);
	}

	// Lines
	if (nipper->ios->line !=0)
	{
		output_parseTitle(settings_lines_title, nipper, rate_none, section_config);
		output_parseText(settings_lines_text, nipper, section_none, rate_none, 0, 0);
		output_table(nipper, true, settings_lines_table, &settings_lines_heading);
		linePointer = nipper->ios->line;
		while (linePointer != 0)
		{
			switch (linePointer->lineType)
			{
				case line_console:
					fprintf(nipper->reportFile, "%sConsole%s", tablerow_start, tablerow_mid);
					break;
				case line_aux:
					fprintf(nipper->reportFile, "%sAuxiliary%s", tablerow_start, tablerow_mid);
					break;
				case line_vty:
					fprintf(nipper->reportFile, "%sVTY%s", tablerow_start, tablerow_mid);
					addAbbreviation("VTY", false);
					break;
				case line_tty:
					fprintf(nipper->reportFile, "%sTTY%s", tablerow_start, tablerow_mid);
					addAbbreviation("TTY", false);
					break;
			}
			fprintf(nipper->reportFile, "%d%s", linePointer->lineNumberStart, tablerow_mid);
			if (linePointer->lineNumberEnd == 0)
				fprintf(nipper->reportFile, " %s", tablerow_mid);
			else
				fprintf(nipper->reportFile, "%d%s", linePointer->lineNumberEnd, tablerow_mid);
			switch (linePointer->login)
			{
				case login_allowed:
					fprintf(nipper->reportFile, "Allowed%s", tablerow_mid);
					break;
				case login_disabled:
					fprintf(nipper->reportFile, "Denied%s", tablerow_mid);
					break;
				case login_local:
					fprintf(nipper->reportFile, "Local%s", tablerow_mid);
					break;
				case login_authentication:
					fprintf(nipper->reportFile, "AAA%s", tablerow_mid);
					break;
			}
			switch (linePointer->exec)
			{
				case true:
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
					break;
				case false:
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
					break;
			}
			switch (linePointer->authorization)
			{
				case true:
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
					break;
				case false:
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
					break;
			}
			switch (linePointer->accounting)
			{
				case true:
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
					break;
				case false:
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
					break;
			}
			switch (linePointer->telnet)
			{
				case true:
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
					break;
				case false:
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
					break;
			}
			switch (linePointer->ssh)
			{
				case true:
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
					break;
				case false:
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
					break;
			}
			fprintf(nipper->reportFile, "%ds%s%ds%s%ds%s%ds%s", linePointer->timeout, tablerow_mid, linePointer->execTimeout, tablerow_mid, linePointer->sessionTimeout, tablerow_mid, linePointer->absoluteTimeout, tablerow_mid);
			if (nipper->passwords == true)
				fprintf(nipper->reportFile, "%s%s", linePointer->password, tablerow_mid);
			if (linePointer->encryption == encrypt_type7)
				fprintf(nipper->reportFile, "Type-7%s", tablerow_end);
			else if (linePointer->encryption == encrypt_md5)
			{
				fprintf(nipper->reportFile, "MD5%s", tablerow_end);
				addAbbreviation("MD5", false);
			}
			else if (linePointer->encryption == encrypt_unknown)
				fprintf(nipper->reportFile, "Unknown%s", tablerow_end);
			else
			{
				if (linePointer->password[0] == 0)
					fprintf(nipper->reportFile, " %s", tablerow_end);
				else
					fprintf(nipper->reportFile, "None%s", tablerow_end);
			}
			linePointer = linePointer->next;
		}
		output_table(nipper, false, settings_lines_table, &settings_lines_heading);
		output_endsection(nipper, subsection_end);
	}

	// Interfaces
	if (nipper->ios->interface !=0)
	{
		output_parseTitle(settings_inter_title, nipper, rate_none, section_config);
		if (nipper->deviceType == type_ios_router)
			output_table(nipper, true, settings_inter_table, &settings_interIOS_heading);
		else
			output_table(nipper, true, settings_inter_table, &settings_interIOSswitch_heading);
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			fprintf(nipper->reportFile, "%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid);
			if (interfaceConfigPointer->shutdown == true)
				fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
			else
				fprintf(nipper->reportFile, "Yes%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
			if (strcmp(interfaceConfigPointer->ipAddress, "None") != 0)
			{
				if (interfaceConfigPointer->proxyArp == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				if (interfaceConfigPointer->ipUnreachables == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				if (interfaceConfigPointer->ipRedirects == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				if (interfaceConfigPointer->ipMaskReply == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				if (interfaceConfigPointer->ipDirectBroadcast == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else if (interfaceConfigPointer->ipDirectBroadcast == false)
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Default%s", tablerow_mid);
				if (interfaceConfigPointer->ntp == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				if ((strncmp(interfaceConfigPointer->name,"Loopback", 8) == 0) || (strncmp(interfaceConfigPointer->name, "Vlan", 4)== 0))
					fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				else if (interfaceConfigPointer->cdp == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			}
			else
			{
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
				fprintf(nipper->reportFile, "N/A%s", tablerow_mid);
			}
			if (nipper->deviceType == type_ios_router)
			{
				if (interfaceConfigPointer->uRPF == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
			}
			if (strcmp(interfaceConfigPointer->ipAddress, "None") != 0)
			{
				if (strncmp(interfaceConfigPointer->name, "Loopback", 8) == 0)
					fprintf(nipper->reportFile, "N/A%s", tablerow_end);
				else if (interfaceConfigPointer->mop == true)
					fprintf(nipper->reportFile, "On%s", tablerow_end);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_end);
			}
			else
				fprintf(nipper->reportFile, "N/A%s", tablerow_end);
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (nipper->deviceType == type_ios_router)
			output_table(nipper, false, settings_inter_table, &settings_interIOS_heading);
		else
			output_table(nipper, false, settings_inter_table, &settings_interIOSswitch_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("ARP", false);
		addAbbreviation("NTP", false);
		addAbbreviation("CDP", false);
		addAbbreviation("MOP", false);
		addAbbreviation("uRPF", false);
	}

	// VTP
	if (nipper->deviceType != type_ios_router)
	{
		output_parseTitle(settings_vtp_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_vtp_table, &settings_general_heading);
		switch (nipper->ios->vtpMode)
		{
			case vtp_mode_server:
				fprintf(nipper->reportFile, "%sVTP Mode%sServer%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case vtp_mode_client:
				fprintf(nipper->reportFile, "%sVTP Mode%sClient%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case vtp_mode_transparent:
				fprintf(nipper->reportFile, "%sVTP Mode%sTransparent%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
			case vtp_mode_none:
				fprintf(nipper->reportFile, "%sVTP Mode%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
				break;
		}
		if (nipper->ios->vtpDomain[0] != 0)
		{
			outputFriendly(nipper->ios->vtpDomain, tempString, nipper->maxSize, nipper->outputFormat);
			fprintf(nipper->reportFile, "%sVTP Domain%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		output_table(nipper, false, settings_vtp_table, &settings_general_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("VTP", false);
	}

	// Switch ports
	if ((nipper->ios->interface !=0) && (nipper->deviceType != type_ios_router))
	{
		output_parseTitle(settings_switchport_title, nipper, rate_none, section_config);
		output_table(nipper, true, settings_switchport_table, &settings_switchport_heading);
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((strncasecmp(interfaceConfigPointer->name, "Vlan", 4) != 0) && (strncasecmp(interfaceConfigPointer->name, "Loop", 4) != 0))
			{
				fprintf(nipper->reportFile, "%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid);
				if (interfaceConfigPointer->shutdown == true)
					fprintf(nipper->reportFile, "No%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
				fprintf(nipper->reportFile, "%s%s", interfaceConfigPointer->description, tablerow_mid);
				fprintf(nipper->reportFile, "%d%s", interfaceConfigPointer->switchportVLAN, tablerow_mid);
				switch (interfaceConfigPointer->switchportMode)
				{
					case sp_mode_access:
						fprintf(nipper->reportFile, "Access%s", tablerow_mid);
						break;
					case sp_mode_trunk:
						fprintf(nipper->reportFile, "Trunk%s", tablerow_mid);
						break;
					case sp_mode_dynamic:
						fprintf(nipper->reportFile, "Dynamic%s", tablerow_mid);
						break;
					case sp_mode_dot1qtunnel:
						fprintf(nipper->reportFile, "802.1q Tunnel%s", tablerow_mid);
						break;
					case sp_mode_pvlan_host:
						fprintf(nipper->reportFile, "Host%s", tablerow_mid);
						break;
					case sp_mode_pvlan_promiscuous:
						fprintf(nipper->reportFile, "Promiscuous%s", tablerow_mid);
						break;
				}
				if (interfaceConfigPointer->switchportSecurity == true)
					fprintf(nipper->reportFile, "On%s", tablerow_mid);
				else
					fprintf(nipper->reportFile, "Off%s", tablerow_mid);
				switch (interfaceConfigPointer->switchportViolation)
				{
					case sp_violation_shutdown:
						fprintf(nipper->reportFile, "Shutdown%s", tablerow_end);
						break;
					case sp_violation_restrict:
						fprintf(nipper->reportFile, "Restrict%s", tablerow_end);
						break;
					case sp_violation_protect:
						fprintf(nipper->reportFile, "Protect%s", tablerow_end);
						break;
				}
			}
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		output_table(nipper, false, settings_switchport_table, &settings_switchport_heading);
		output_endsection(nipper, subsection_end);
		addAbbreviation("PVLAN", false);
		addAbbreviation("VLAN", false);
	}

	// NAT
	if (nipper->ios->nat !=0)
	{
		output_parseTitle(settings_nat_title, nipper, rate_none, section_config);
		output_parseText(settings_natpat_desc, nipper, section_none, rate_none, 2, 0);

		// Static NAT
		tempInt = 0;
		natPointer = nipper->ios->nat;
		while (natPointer != 0)
		{
			if (natPointer->type == nat_static)
				tempInt++;
			natPointer = natPointer->next;
		}
		if (tempInt > 0)
		{
			output_table(nipper, true, settings_nat_static_table, &settings_natstatic_heading);
			natPointer = nipper->ios->nat;
			while (natPointer != 0)
			{
				if (natPointer->type == nat_static)
				{
					if (natPointer->inside == true)
						fprintf(nipper->reportFile, "%sInside Interface%s", tablerow_start, tablerow_mid);
					else
						fprintf(nipper->reportFile, "%sOutside Interface%s", tablerow_start, tablerow_mid);
					if (natPointer->ipSecESP == true)
						fprintf(nipper->reportFile, "Yes%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "No%s", tablerow_mid);
					outputFriendly(natPointer->local, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					outputFriendly(natPointer->global, tempString, nipper->maxSize, nipper->outputFormat);
					if (natPointer->ipSecESP == true)
						fprintf(nipper->reportFile, "%s%d%s", tempString, natPointer->interNumber, tablerow_end);
					else
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
				}
				natPointer = natPointer->next;
			}
			output_table(nipper, false, settings_nat_static_table, &settings_natstatic_heading);
		}

		// Dynamic NAT
		tempInt = 0;
		natPointer = nipper->ios->nat;
		while (natPointer != 0)
		{
			if ((natPointer->type == nat_dynamic_acl) || (natPointer->type == nat_dynamic_route))
				tempInt++;
			natPointer = natPointer->next;
		}
		if (tempInt > 0)
		{
			output_table(nipper, true, settings_nat_dynamic_table, &settings_natdynamic_heading);
			natPointer = nipper->ios->nat;
			while (natPointer != 0)
			{
				if ((natPointer->type == nat_dynamic_acl) || (natPointer->type == nat_dynamic_route))
				{
					if (natPointer->inside == true)
						fprintf(nipper->reportFile, "%sInside Interface%s", tablerow_start, tablerow_mid);
					else
						fprintf(nipper->reportFile, "%sOutside Interface%s", tablerow_start, tablerow_mid);
					if (natPointer->type == nat_dynamic_acl)
						fprintf(nipper->reportFile, "ACL%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "Route Map%s", tablerow_mid);
					outputFriendly(natPointer->local, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
					outputFriendly(natPointer->global, tempString, nipper->maxSize, nipper->outputFormat);
					if (natPointer->pool == true)
					{
						fprintf(nipper->reportFile, "Pool%s", tablerow_mid);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
					}
					else
					{
						fprintf(nipper->reportFile, "Interface%s", tablerow_mid);
						fprintf(nipper->reportFile, "%s%d%s", tempString, natPointer->interNumber, tablerow_end);
					}
				}
				natPointer = natPointer->next;
			}
			output_table(nipper, false, settings_nat_dynamic_table, &settings_natdynamic_heading);
		}

		// Port Static NAT
		tempInt = 0;
		natPointer = nipper->ios->nat;
		while (natPointer != 0)
		{
			if (natPointer->type == nat_port)
				tempInt++;
			natPointer = natPointer->next;
		}
		if (tempInt > 0)
		{
			output_table(nipper, true, settings_nat_portstatic_table, &settings_natportstat_heading);
			natPointer = nipper->ios->nat;
			while (natPointer != 0)
			{
				if (natPointer->type == nat_port)
				{
					if (natPointer->inside == true)
						fprintf(nipper->reportFile, "%sInside Interface%s", tablerow_start, tablerow_mid);
					else
						fprintf(nipper->reportFile, "%sOutside Interface%s", tablerow_start, tablerow_mid);
					if (natPointer->inter == true)
					{
						fprintf(nipper->reportFile, "%s %s %s %s%d%s", tablerow_mid, tablerow_mid, tablerow_mid, tablerow_mid, natPointer->globalPort, tablerow_end);
					}
					else
					{
						outputFriendly(natPointer->local, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						fprintf(nipper->reportFile, "%d%s", natPointer->localPort, tablerow_mid);
						outputFriendly(natPointer->global, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s%d%s", tempString, tablerow_mid, natPointer->globalPort, tablerow_end);
					}
				}
				natPointer = natPointer->next;
			}
			output_table(nipper, false, settings_nat_portstatic_table, &settings_natportstat_heading);
		}

		// Network Static NAT
		tempInt = 0;
		natPointer = nipper->ios->nat;
		while (natPointer != 0)
		{
			if (natPointer->type == nat_network)
				tempInt++;
			natPointer = natPointer->next;
		}
		if (tempInt > 0)
		{
			output_table(nipper, true, settings_nat_netstatic_table, &settings_natnetstatic_heading);
			natPointer = nipper->ios->nat;
			while (natPointer != 0)
			{
				if (natPointer->type == nat_network)
				{
					if (natPointer->inside == true)
						fprintf(nipper->reportFile, "%sInside Interface%s", tablerow_start, tablerow_mid);
					else
						fprintf(nipper->reportFile, "%sOutside Interface%s", tablerow_start, tablerow_mid);
						outputFriendly(natPointer->local, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						outputFriendly(natPointer->global, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_mid);
						outputFriendly(natPointer->mask, tempString, nipper->maxSize, nipper->outputFormat);
						fprintf(nipper->reportFile, "%s%s", tempString, tablerow_end);
				}
				natPointer = natPointer->next;
			}
			output_table(nipper, false, settings_nat_netstatic_table, &settings_natnetstatic_heading);
		}
		output_endsection(nipper, subsection_end);
	}

	// Access Lists
	if (nipper->ios->access !=0)
	{
		output_parseTitle(settings_accesslists_title, nipper, rate_none, section_config);
		output_parseText(settings_accesslists1, nipper, section_none, rate_none, 0, 0);
		output_parseText(settings_accesslists2, nipper, section_none, rate_none, 0, 0);

		// Interfaces with ACL applied
		tempInt = 0;
		interfaceConfigPointer = nipper->ios->interface;
		while (interfaceConfigPointer != 0)
		{
			if ((interfaceConfigPointer->accessGroupIn[0] != 0) || (interfaceConfigPointer->accessGroupOut[0] != 0))
				tempInt++;
			interfaceConfigPointer = interfaceConfigPointer->next;
		}
		if (tempInt > 0)
		{
			output_table(nipper, true, settings_acl_int_table, &settings_aclinter_heading);
			interfaceConfigPointer = nipper->ios->interface;
			while (interfaceConfigPointer != 0)
			{
				if ((interfaceConfigPointer->accessGroupIn[0] != 0) || (interfaceConfigPointer->accessGroupOut[0] != 0))
				{
					fprintf(nipper->reportFile, "%s%s%s", tablerow_start, interfaceConfigPointer->name, tablerow_mid);
					if (interfaceConfigPointer->shutdown == true)
						fprintf(nipper->reportFile, "No%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
					else
						fprintf(nipper->reportFile, "Yes%s%s%s", tablerow_mid, interfaceConfigPointer->ipAddress, tablerow_mid);
					if (interfaceConfigPointer->accessGroupIn[0] != 0)
						fprintf(nipper->reportFile, "%s%s", interfaceConfigPointer->accessGroupIn, tablerow_mid);
					else
						fprintf(nipper->reportFile, " %s", tablerow_mid);
					if (interfaceConfigPointer->accessGroupOut[0] != 0)
						fprintf(nipper->reportFile, "%s%s", interfaceConfigPointer->accessGroupOut, tablerow_end);
					else
						fprintf(nipper->reportFile, " %s", tablerow_end);
				}
				interfaceConfigPointer = interfaceConfigPointer->next;
			}
			output_table(nipper, false, settings_acl_int_table, &settings_aclinter_heading);
		}

		// Lines with ACL applied
		tempInt = 0;
		linePointer = nipper->ios->line;
		while (linePointer != 0)
		{
			if ((linePointer->aclIn[0] != 0) || (linePointer->aclOut[0] != 0))
				tempInt++;
			linePointer = linePointer->next;
		}
		if (tempInt > 0)
		{
			output_table(nipper, true, settings_acl_line_table, &settings_aclline_heading);
			linePointer = nipper->ios->line;
			while (linePointer != 0)
			{
				if ((linePointer->aclIn[0] != 0) || (linePointer->aclOut[0] != 0))
				{
					switch (linePointer->lineType)
					{
						case line_console:
							fprintf(nipper->reportFile, "%sConsole%s", tablerow_start, tablerow_mid);
							break;
						case line_aux:
							fprintf(nipper->reportFile, "%sAuxiliary%s", tablerow_start, tablerow_mid);
							break;
						case line_vty:
							fprintf(nipper->reportFile, "%sVTY%s", tablerow_start, tablerow_mid);
							addAbbreviation("VTY", false);
							break;
						case line_tty:
							fprintf(nipper->reportFile, "%sTTY%s", tablerow_start, tablerow_mid);
							addAbbreviation("TTY", false);
							break;
					}
					fprintf(nipper->reportFile, "%d%s", linePointer->lineNumberStart, tablerow_mid);
					if (linePointer->lineNumberEnd == 0)
						fprintf(nipper->reportFile, " %s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "%d%s", linePointer->lineNumberEnd, tablerow_mid);
					switch (linePointer->login)
					{
						case login_allowed:
							fprintf(nipper->reportFile, "Allowed%s", tablerow_mid);
							break;
						case login_disabled:
							fprintf(nipper->reportFile, "Denied%s", tablerow_mid);
							break;
						case login_local:
							fprintf(nipper->reportFile, "Local%s", tablerow_mid);
							break;
						case login_authentication:
							fprintf(nipper->reportFile, "AAA%s", tablerow_mid);
							break;
					}
					switch (linePointer->exec)
					{
						case true:
							fprintf(nipper->reportFile, "On%s", tablerow_mid);
							break;
						case false:
							fprintf(nipper->reportFile, "Off%s", tablerow_mid);
							break;
					}
					switch (linePointer->telnet)
					{
						case true:
							fprintf(nipper->reportFile, "On%s", tablerow_mid);
							break;
						case false:
							fprintf(nipper->reportFile, "Off%s", tablerow_mid);
							break;
					}
					switch (linePointer->ssh)
					{
						case true:
							fprintf(nipper->reportFile, "On%s", tablerow_mid);
							break;
						case false:
							fprintf(nipper->reportFile, "Off%s", tablerow_mid);
							break;
					}
					if (linePointer->aclIn[0] != 0)
						fprintf(nipper->reportFile, "%s%s", linePointer->aclIn, tablerow_mid);
					else
						fprintf(nipper->reportFile, " %s", tablerow_mid);
					if (linePointer->aclOut[0] != 0)
						fprintf(nipper->reportFile, "%s%s", linePointer->aclOut, tablerow_end);
					else
						fprintf(nipper->reportFile, " %s", tablerow_end);
				}
				linePointer = linePointer->next;
			}
			output_table(nipper, false, settings_acl_line_table, &settings_aclline_heading);
		}

		accessPointer = nipper->ios->access;
		while (accessPointer !=0)
		{
			// Standard Access List
			if (accessPointer->type == ios_acl_standard)
			{
				tempInt = 1;
				if (accessPointer->remark[0] == 0)
					sprintf(tempString, "%s %s", settings_accessstd_table, accessPointer->name);
				else
					sprintf(tempString, "%s %s (%s)", settings_accessstd_table, accessPointer->name, accessPointer->remark);
				outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
				output_table(nipper, true, tempString2, &settings_accessstd_heading);
				permissionPointer = accessPointer->permission;
				while (permissionPointer != 0)
				{
					fprintf(nipper->reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
					tempInt++;
					if (permissionPointer->permission == perm_permit)
						fprintf(nipper->reportFile, "Permit%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "Deny%s", tablerow_mid);
					outputFriendly(permissionPointer->source, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s %s", tempString, tablerow_mid);
					if (permissionPointer->log == true)
						fprintf(nipper->reportFile, "Yes");
					else
						fprintf(nipper->reportFile, "No");
					fprintf(nipper->reportFile, "%s", tablerow_end);
					permissionPointer = permissionPointer->next;
				}
				output_table(nipper, false, tempString2, &settings_accessstd_heading);
			}

			// Extended Access List
			if (accessPointer->type == ios_acl_extended)
			{
				tempInt = 1;
				if (accessPointer->remark[0] == 0)
					sprintf(tempString, "%s %s", settings_accessext_table, accessPointer->name);
				else
					sprintf(tempString, "%s %s (%s)", settings_accessext_table, accessPointer->name, accessPointer->remark);
				outputFriendly(tempString, tempString2, nipper->maxSize, nipper->outputFormat);
				output_table(nipper, true, tempString2, &settings_accessext_heading);
				permissionPointer = accessPointer->permission;
				while (permissionPointer != 0)
				{
					fprintf(nipper->reportFile, "%s%d%s", tablerow_start, tempInt, tablerow_mid);
					tempInt++;
					if (permissionPointer->permission == perm_permit)
						fprintf(nipper->reportFile, "Permit%s", tablerow_mid);
					else
						fprintf(nipper->reportFile, "Deny%s", tablerow_mid);
					outputFriendly(permissionPointer->source, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s %s%s %s", permissionPointer->protocol, tablerow_mid, tempString, tablerow_mid);
					addAbbreviation(permissionPointer->protocol, false);
					addAbbreviation(permissionPointer->sourceService, false);
					switch (permissionPointer->sourceOperator)
					{
						case perm_op_lt:
							if (nipper->outputFormat == output_html)
								fprintf(nipper->reportFile, "&lt;");
							else
								fprintf(nipper->reportFile, "<");
							fprintf(nipper->reportFile, " %s", permissionPointer->sourceService);
							break;
						case perm_op_gt:
							if (nipper->outputFormat == output_html)
								fprintf(nipper->reportFile, "&gt;");
							else
								fprintf(nipper->reportFile, ">");
							fprintf(nipper->reportFile, " %s", permissionPointer->sourceService);
							break;
						case perm_op_eq:
							fprintf(nipper->reportFile, "%s", permissionPointer->sourceService);
							break;
						case perm_op_neq:
							fprintf(nipper->reportFile, "not %s", permissionPointer->sourceService);
							break;
						case perm_op_range:
							fprintf(nipper->reportFile, "%s - %s", permissionPointer->sourceService, permissionPointer->sourceServiceEnd);
							addAbbreviation(permissionPointer->sourceServiceEnd, false);
							break;
						default:
							fprintf(nipper->reportFile, "any");
							break;
					}
					outputFriendly(permissionPointer->dest, tempString, nipper->maxSize, nipper->outputFormat);
					fprintf(nipper->reportFile, "%s%s %s", tablerow_mid, tempString, tablerow_mid);
					addAbbreviation(permissionPointer->destService, false);
					switch (permissionPointer->destOperator)
					{
						case perm_op_lt:
							if (nipper->outputFormat == output_html)
								fprintf(nipper->reportFile, "&lt;");
							else
								fprintf(nipper->reportFile, "<");
							fprintf(nipper->reportFile, " %s", permissionPointer->destService);
							break;
						case perm_op_gt:
							if (nipper->outputFormat == output_html)
								fprintf(nipper->reportFile, "&gt;");
							else
								fprintf(nipper->reportFile, ">");
							fprintf(nipper->reportFile, " %s", permissionPointer->destService);
							break;
						case perm_op_eq:
							fprintf(nipper->reportFile, "%s", permissionPointer->destService);
							break;
						case perm_op_neq:
							fprintf(nipper->reportFile, "not %s", permissionPointer->destService);
							break;
						case perm_op_range:
							fprintf(nipper->reportFile, "%s - %s", permissionPointer->destService, permissionPointer->destServiceEnd);
							addAbbreviation(permissionPointer->destServiceEnd, false);
							break;
						default:
							fprintf(nipper->reportFile, "any");
							break;
					}
					if (permissionPointer->log == true)
						fprintf(nipper->reportFile, "%sYes%s", tablerow_mid, tablerow_mid);
					else
						fprintf(nipper->reportFile, "%sNo%s", tablerow_mid, tablerow_mid);
					if (permissionPointer->established == true)
						fprintf(nipper->reportFile, "Established");
					if ((permissionPointer->established == true) && (permissionPointer->fragments == true))
						fprintf(nipper->reportFile, ", ");
					if (permissionPointer->fragments == true)
						fprintf(nipper->reportFile, "Fragments");
					fprintf(nipper->reportFile, " %s", tablerow_end);
					permissionPointer = permissionPointer->next;
				}
				output_table(nipper, false, tempString2, &settings_accessext_heading);
			}

			// Next Access List
			accessPointer = accessPointer->next;
		}

		output_endsection(nipper, subsection_end);
	}

	// End Section
	output_endsection(nipper, section_end);

	// Output end of file stuff
	output_fileend(nipper);
}

